@@ -100,6 +100,11 @@ APPKIT_EXPORT NSString * GSScreenNumber;
 - (void *) windowDevice: (int)win;
 
 - (void) beep;
+
+/* AppIcon/MiniWindow information */
+- (NSImage *) iconTileImage;
+- (NSSize) iconSize;
+
 @end
 
 /* ----------------------------------------------------------------------- */
@@ -137,6 +142,7 @@ APPKIT_EXPORT NSString * GSScreenNumber;
 - (void) docedited: (int) edited : (int) win;
 - (void) setinputstate: (int)state : (int)win;
 - (void) setinputfocus: (int) win;
+- (void) setalpha: (float)alpha: (int) win;
 
 - (NSPoint) mouselocation;
 - (NSPoint) mouseLocationOnScreen: (int)aScreen window: (int *)win;
@@ -145,7 +151,8 @@ APPKIT_EXPORT NSString * GSScreenNumber;
 - (void) hidecursor;
 - (void) showcursor;
 - (void) standardcursor: (int) style : (void**) cid;
-- (void) imagecursor: (NSPoint)hotp : (int)w : (int)h : (int) colors : (const char*) image : (void**) cid;
+- (void) imagecursor: (NSPoint)hotp : (int)w : (int)h : (int) colors
+		    : (const unsigned char *) image : (void**) cid;
 - (void) setcursorcolor: (NSColor *)fg : (NSColor *)bg : (void*) cid;
 
 @end
