@@ -143,12 +143,14 @@
 #include <AppKit/NSFileWrapper.h>
 #include <AppKit/NSLayoutManager.h>
 #include <AppKit/NSOutlineView.h>
+#include <AppKit/NSNib.h>
 #include <AppKit/NSRulerMarker.h>
 #include <AppKit/NSRulerView.h>
 #include <AppKit/NSTextContainer.h>
 #include <AppKit/NSTextStorage.h>
 #include <AppKit/NSUserInterfaceValidation.h>
 #include <AppKit/NSWindowController.h>
+#include <AppKit/NSWindow+Toolbar.h>
 #include <AppKit/NSToolbar.h>
 #include <AppKit/NSToolbarItem.h>
 #endif
