@@ -39,9 +39,17 @@
 
 typedef enum _NSControlTint {
     NSDefaultControlTint,
-    NSClearControlTint
+    NSClearControlTint,
+    NSBlueControlTint,
+    NSGraphiteControlTint
 } NSControlTint;
 
+typedef enum _NSControlSize {
+    NSRegularControlSize,
+    NSSmallControlSize,
+    NSMiniControlSize
+} NSControlSize;
+
 /*
  * NSColor is an abstract super class of the class cluster of the real colour classes.
  * For each colour space exists a specific subclass that implements the behaviour for 
@@ -191,6 +199,8 @@ typedef enum _NSControlTint {
 //
 // System colors stuff.
 //
++ (NSColor*) alternateSelectedControlColor;
++ (NSColor*) alternateSelectedControlTextColor;
 + (NSColor*) controlBackgroundColor;
 + (NSColor*) controlColor;
 + (NSColor*) controlHighlightColor;
@@ -206,6 +216,7 @@ typedef enum _NSControlTint {
 + (NSColor*) keyboardFocusIndicatorColor;
 + (NSColor*) knobColor;
 + (NSColor*) scrollBarColor;
++ (NSColor*) secondarySelectedControlColor;
 + (NSColor*) selectedControlColor;
 + (NSColor*) selectedControlTextColor;
 + (NSColor*) selectedKnobColor;
@@ -220,6 +231,8 @@ typedef enum _NSControlTint {
 + (NSColor*) windowFrameColor;
 + (NSColor*) windowFrameTextColor;
 
++ (NSArray*) controlAlternatingRowBackgroundColors;
+
 // Pattern colour
 - (NSImage*) patternImage;
 #endif
