@@ -55,11 +55,13 @@
 @class NSPasteboard;
 @class NSScreen;
 @class NSText;
-@class NSToolbar;
+@class NSToolbar; 
 @class NSView;
 @class NSWindowController;
 @class NSCachedImageRep;
 
+@class GSWindowDecorationView;
+
 /*
  * Window levels are taken from MacOS-X
  * NSDesktopWindowLevel is copied from Window maker and is intended to be
@@ -101,12 +103,41 @@ APPKIT_EXPORT NSSize NSTokenSize;
 
 @interface NSWindow : NSResponder <NSCoding>
 {
+  /*
+  A window really has three interesting frames:
+  
+  The screen frame. This is the frame of the _entire_ window on the screen,
+  including all decorations and borders (regardless of where they come from).
+  (On X, we can only guess what the screen frame is.)
+
+  The window frame. This is the frame of the backend window for this window,
+  and is thus the base of the coordinate system for the window. IOW, it's
+  the frame of the area we can draw into.
+
+  The contect rect. This is the frame of the content view.
+
+  Wrt. size, ScreenFrame >= WindowFrame >= ContentRect. When -gui doesn't
+  manage the window decorations, WindowFrame == ContentRect. When -gui does
+  manage the window decorations, WindowFrame will include the decorations,
+  and ScreenFrame == WindowFrame.
+
+
+  To get coordinate transforms and stuff right wrt. OpenStep, we really want
+  the window frame here.
+
+  For hysterical reasons, _frame used to be the screen frame. However, the
+  resulting inconsistencies caused a bunch of problems. Thus, _frame is the
+  window frame. The other rectangles/sizes passed around in NSWindow
+  methods are supposed to all be window frames.
+  */
   NSRect        _frame;
+
+
   NSSize        _minimumSize;
   NSSize        _maximumSize;
   NSSize        _increments;
   NSString	*_autosaveName;
-  id		_wv;
+  GSWindowDecorationView *_wv;
   id            _contentView;
   id            _firstResponder;
   id            _futureFirstResponder;
@@ -138,9 +169,9 @@ APPKIT_EXPORT NSSize NSTokenSize;
   NSWindowController *_windowController;
   int		_counterpart;
   float         _alphaValue;
-
-  NSToolbar    *_toolbar;
-  id            _toolbarView;
+  
+  NSToolbar    *_toolbar; // Not used (see NSWindow+Toolbar now)
+  id            _toolbarView; // Not used (see NSWindow+Toolbar now)
   NSCachedImageRep *_cachedImage;
   NSPoint       _cachedImageOrigin;
 
@@ -177,6 +208,7 @@ APPKIT_EXPORT NSSize NSTokenSize;
   id _defaultButtonCell;
   void          *_reserved_1;
   void          *_reserved_2;
+  
 }
 
 /*
@@ -186,15 +218,25 @@ APPKIT_EXPORT NSSize NSTokenSize;
 /*
  * Computing frame and content rectangles
  */
+
+/* These methods convert between the various frames discussed above. */
 + (NSRect) contentRectForFrameRect: (NSRect)aRect
 			 styleMask: (unsigned int)aStyle;
 
 + (NSRect) frameRectForContentRect: (NSRect)aRect
 			 styleMask: (unsigned int)aStyle;
 
-+ (NSRect) minFrameWidthWithTitle: (NSString *)aTitle
++ (NSRect) screenRectForFrameRect: (NSRect)aRect
 			styleMask: (unsigned int)aStyle;
 
++ (NSRect) frameRectForScreenRect: (NSRect)aRect
+			styleMask: (unsigned int)aStyle;
+
+/* Returns the smallest window width that will fit the given title and
+style. */
++ (float) minFrameWidthWithTitle: (NSString *)aTitle
+		       styleMask: (unsigned int)aStyle;
+
 
 /*
  * Initializing and getting a new NSWindow object
@@ -514,10 +556,6 @@ APPKIT_EXPORT NSSize NSTokenSize;
 
 #ifndef STRICT_OPENSTEP
 - (NSArray *) drawers;
-- (void) setToolbar: (NSToolbar*)toolbar;
-- (NSToolbar *) toolbar;
-- (void) toggleToolbarShown: (id)sender;
-- (void) runToolbarCustomizationPalette: (id)sender;
 - (id) initWithWindowRef: (void *)windowRef;
 - (void *)windowRef;
 - (void*) windowHandle;
