@@ -368,10 +368,6 @@ void __dummy_GMAppKit_functionForLinking() {}
     interval = [unarchiver decodeFloatWithName:@"interval"];
     [self setPeriodicDelay:delay interval:interval];
 
-    obj = [unarchiver decodeStringWithName:@"title"];
-    if (obj) [self setTitle:obj];
-    obj = [unarchiver decodeStringWithName:@"alternateTitle"];
-    if (obj) [self setAlternateTitle:obj];
     obj = [unarchiver decodeObjectWithName:@"alternateImage"];
     [self setAlternateImage:obj];
     [self setImagePosition:[unarchiver decodeIntWithName:@"imagePosition"]];
@@ -381,6 +377,10 @@ void __dummy_GMAppKit_functionForLinking() {}
     [self setKeyEquivalentModifierMask:[unarchiver decodeIntWithName:@"keyEquivalentModifierMask"]];
     [self setHighlightsBy:[unarchiver decodeIntWithName:@"highlightsBy"]];
     [self setShowsStateBy:[unarchiver decodeIntWithName:@"showsStateBy"]];
+    obj = [unarchiver decodeStringWithName:@"title"];
+    if (obj) [self setTitle:obj];
+    obj = [unarchiver decodeStringWithName:@"alternateTitle"];
+    if (obj) [self setAlternateTitle:obj];
     // }
 
     return self;
@@ -982,8 +982,7 @@ void __dummy_GMAppKit_functionForLinking() {}
   [archiver encodeArray:[self itemArray] withName:@"itemArray"];
 #endif
 
-  //[archiver encodeString:[self titleOfSelectedItem] withName:@"selectedItem"];
-  [archiver encodeString:[self title] withName:@"selectedItem"];
+  [archiver encodeString:[[self selectedItem] title] withName:@"selectedItem"];
 }
 
 + (id)createObjectForModelUnarchiver:(GMUnarchiver*)unarchiver
@@ -1224,14 +1223,20 @@ void __dummy_GMAppKit_functionForLinking() {}
 - (void)encodeWithModelArchiver:(GMArchiver*)archiver
 {
   NSPoint wnOrigin = [self frame].origin;
-  NSRect ctFrame = [[self contentView] frame];
+  NSRect ctFrame = [[self contentView] frame], minRect;
   unsigned int style;
 
   ctFrame.origin = wnOrigin;
 
+  /* convert minSize to GNUstep frame (without title bar and resize bar) */
+  minRect.origin = wnOrigin;
+  minRect.size = [self minSize];
+  minRect = [NSWindow contentRectForFrameRect:minRect
+                                    styleMask:[self styleMask]];
+
   [archiver encodeRect:ctFrame withName:@"contentFrame"];
   [archiver encodeSize:[self maxSize] withName:@"maxSize"];
-  [archiver encodeSize:[self minSize] withName:@"minSize"];
+  [archiver encodeSize:minRect.size withName:@"minSize"];
   [archiver encodeString:[self frameAutosaveName]
 	    withName:@"frameAutosaveName"];
   [archiver encodeInt:[self level] withName:@"level"];
