@@ -25,9 +25,9 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#include <AppKit/GSMemoryPanel.h>
-#include <AppKit/GSHbox.h>
-#include <AppKit/GSVbox.h>
+#include "GNUstepGUI/GSMemoryPanel.h"
+#include "GNUstepGUI/GSHbox.h"
+#include "GNUstepGUI/GSVbox.h"
 
 enum {
   OrderByClassName,
@@ -194,8 +194,6 @@ static GSMemoryPanel *sharedGSMemoryPanel = nil;
   [button setAction: @selector(update:)];
   [button setAutoresizingMask: NSViewMinXMargin];
   [button sizeToFit];
-  [button setFrameSize: NSMakeSize ([button frame].size.width + 6,
-				    [button frame].size.height + 4)];
 
   hbox = [GSHbox new];
   [hbox setDefaultMinXMargin: 5];
@@ -279,6 +277,17 @@ static GSMemoryPanel *sharedGSMemoryPanel = nil;
   return self;
 }
 
+- (void) dealloc
+{
+  RELEASE(table);
+  RELEASE(classArray);
+  RELEASE(countArray);
+  RELEASE(totalArray);
+  RELEASE(peakArray);
+
+  [super dealloc];
+}
+
 - (int) numberOfRowsInTableView: (NSTableView *)aTableView
 {
   return [countArray count];
@@ -345,6 +354,7 @@ static GSMemoryPanel *sharedGSMemoryPanel = nil;
       RELEASE (entry);
       i++;
     }
+  NSZoneFree(NSDefaultMallocZone(), classList);
 
   switch (orderingBy)
     {
