@@ -672,7 +672,7 @@ static NSString         *disabledName = @".GNUstepDisabled";
  *      contains the full title of the service.
  *      Return nil if this is not one of our service menu items.
  */
-- (NSString*) item2title: (NSMenuItem*)item
+- (NSString*) item2title: (id<NSMenuItem>)item
 {
   unsigned      pos;
 
@@ -1132,7 +1132,7 @@ static NSString         *disabledName = @".GNUstepDisabled";
   return NO;
 }
 
-- (BOOL) validateMenuItem: (NSMenuItem*)item
+- (BOOL) validateMenuItem: (id<NSMenuItem>)item
 {
   NSString      *title = [self item2title: item];
   NSDictionary  *info = [_title2info objectForKey: title];
@@ -1497,7 +1497,19 @@ NSPerformService(NSString *serviceItem, NSPasteboard *pboard)
    * If the service provider is a remote object, we can set timeouts on
    * the NSConnection so we don't hang waiting for it to reply.
    */
-  if ([provider isProxy] == YES)
+  /*
+  This check for a remote object is ugly. When GSListener is reworked,
+  this should be improved.
+
+  For now, we can't use -isProxy since GSListener is a proxy, and we can't
+  use -isKindOfClass: since it gets forwarded. Fortunately, -class isn't
+  forwarded, so that's what we use.
+
+  (Note, though, that we can't even use
+  [provider class] == [GSListener class] since [GSListener -class] returns
+  NULL instead of the real class.)
+  */
+  if ([provider class] == [NSDistantObject class])
     {
       NSConnection	*connection;
 
