@@ -15,6 +15,9 @@
    Date: September 2000
    Extracted from NSText, reorganised to specification
 
+   Author: Pierre-Yves Rivaille <pyrivail@ens-lyon.fr>
+   Date: September 2002
+
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
@@ -40,15 +43,16 @@
 #include <Foundation/NSScanner.h>
 #include <Foundation/NSCharacterSet.h>
 
-#include <AppKit/NSGraphics.h>
-#include <AppKit/NSColor.h>
-#include <AppKit/NSText.h>
-#include <AppKit/NSTextView.h>
-#include <AppKit/NSTextStorage.h>
-#include <AppKit/NSTextContainer.h>
-#include <AppKit/NSStringDrawing.h>
+#include "AppKit/NSGraphics.h"
+#include "AppKit/NSColor.h"
+#include "AppKit/NSText.h"
+#include "AppKit/NSTextView.h"
+#include "AppKit/NSTextStorage.h"
+#include "AppKit/NSTextContainer.h"
+#include "AppKit/NSStringDrawing.h"
+#include "AppKit/NSParagraphStyle.h"
 
-#include "GSSimpleLayoutManager.h"
+#include "GNUstepGUI/GSSimpleLayoutManager.h"
 
 #define HUGE 1e7
 
@@ -111,6 +115,24 @@ static NSCharacterSet *invSelectionWordGranularitySet;
 @end
 // end: _GNULineLayoutInfo------------------------------------------------------
 
+/* This is the height of the line/insertion point of the default font
+ * - used when there is no text/font in the textview.  FIXME - in many
+ * cases we use this, but it's inappropriate - consider if there is
+ * not a way to get a more appropriate font (eg, the last font the
+ * user was typing in if we're after a newline etc).  Anyway, better
+ * using this function than hardcoding '12' :-)
+ */
+static inline float defaultFontHeight ()
+{
+  NSFont *userFont;
+  NSRect fontRect;
+  
+  userFont = [NSFont userFontOfSize: 0];
+  fontRect = [userFont boundingRectForFont];
+  
+  return fontRect.size.height;
+}
+
 
 @interface GSSimpleLayoutManager (Private)
 
@@ -131,8 +153,10 @@ static NSCharacterSet *invSelectionWordGranularitySet;
 		  delta: (int)insertionDelta
 	    inTextContainer:(NSTextContainer *)aTextContainer;
 // low level, override but never invoke (use setNeedsDisplayForLineRange:)
-- (void) drawLinesInLineRange: (NSRange)aRange;
-- (void) drawSelectionAsRangeNoCaret: (NSRange)aRange;
+- (void) drawLinesInLineRange: (NSRange)aRange
+		      atPoint: (NSPoint)containerOrigin;
+- (void) drawSelectionAsRangeNoCaret: (NSRange)aRange
+			     atPoint: (NSPoint)containerOrigin;
 @end
 
 @implementation GSSimpleLayoutManager
@@ -159,20 +183,20 @@ static NSCharacterSet *invSelectionWordGranularitySet;
 {
   self = [super init];
 
-  _lineLayoutInformation = [[NSMutableArray alloc] init];
+  _lineLayoutInfo = [[NSMutableArray alloc] init];
   return self;
 }
 
 - (void) dealloc
 {
-  RELEASE(_lineLayoutInformation);
+  RELEASE(_lineLayoutInfo);
 
   [super dealloc];
 }
 
 - (void) setTextStorage: (NSTextStorage*)aTextStorage
 {
-  [_lineLayoutInformation removeAllObjects];
+  [_lineLayoutInfo removeAllObjects];
 
   [super setTextStorage: aTextStorage];
 }
@@ -180,13 +204,13 @@ static NSCharacterSet *invSelectionWordGranularitySet;
 // Returns the currently used bounds for all the text
 - (NSRect)usedRectForTextContainer:(NSTextContainer *)aTextContainer
 {
-  if ([_lineLayoutInformation count])
+  if ([_lineLayoutInfo count])
     {
       NSEnumerator *lineEnum;
       _GNULineLayoutInfo *currentInfo;
       NSRect retRect = NSMakeRect (0, 0, 0, 0);
 
-      for ((lineEnum = [_lineLayoutInformation objectEnumerator]);
+      for ((lineEnum = [_lineLayoutInfo objectEnumerator]);
 	   (currentInfo = [lineEnum nextObject]);)
 	{
 	  retRect = NSUnionRect (retRect, currentInfo->usedRect);
@@ -201,9 +225,8 @@ static NSCharacterSet *invSelectionWordGranularitySet;
 		inTextContainer: (NSTextContainer*)aTextContainer
  fractionOfDistanceThroughGlyph: (float*)partialFraction
 {
-  _GNULineLayoutInfo *currentInfo = [_lineLayoutInformation 
-				      objectAtIndex: 
-					[self lineLayoutIndexForPoint: point]];
+  unsigned index = [self lineLayoutIndexForPoint: point];
+  _GNULineLayoutInfo *currentInfo = [_lineLayoutInfo objectAtIndex: index];
   NSRect rect = currentInfo->usedRect;
   NSRange range = currentInfo->glyphRange;
   int i;
@@ -218,13 +241,41 @@ static NSCharacterSet *invSelectionWordGranularitySet;
     {
       *partialFraction = 0.0;
     }
+
+  /* Make sure min is >= 0.  */
+  min = MAX(0, min);
+
+  /* The point is before the beginning of the line.  Return the first
+     location on the line (or 0).  */
   if (x <= fmin)
     {
-      return MAX(0, min - 1);
+      return min;
     }
+  /* The point is after the end of the line.  */
   if (x >= fmax)
     {
-      return MAX(0, max);
+      if (max == 0)
+	{
+	  return 0;
+	}
+      else if ((max == [_textStorage length])
+	       && (index == [_lineLayoutInfo count] - 1))
+	{
+	  /* We are on the very last line and we are asked for the
+             glyph index on a point after the end of the line.  Return
+             end of text then.  */
+	  return MAX(0, [_textStorage length]);
+	}
+      else
+	{
+	  /* In all other cases, this is a normal line (== any line
+	     but the very last one); a newline or space is ending it,
+	     and when you click after the end of the line, we don't
+	     want to go to the next line: return the position just
+	     before the newline or space (that is, return the position
+	     just after the last displayed glyph on that line).  */
+	  return MAX(0, max - 1);
+	}
     }
   if (range.length == 1)
     {
@@ -264,7 +315,7 @@ static NSCharacterSet *invSelectionWordGranularitySet;
 	      *partialFraction = 1.0 - (w1 - x)/(w1 - w2); 
 	    }
 	}
-      return MAX(0, i-1);
+      return MAX(0, i - 1);
     }
   return MAX(0, min - 1);
 }
@@ -274,14 +325,13 @@ static NSCharacterSet *invSelectionWordGranularitySet;
 {
   _GNULineLayoutInfo *currentInfo;
   
-  if (![_textStorage length] || ![_lineLayoutInformation count])
+  if (![_textStorage length] || ![_lineLayoutInfo count])
     {
-      return NSMakeRect(0, 0, 0, 12);
+      return NSMakeRect (0, 0, 0, defaultFontHeight ());      	
     }
   
-  currentInfo = [_lineLayoutInformation 
-		  objectAtIndex: [self lineLayoutIndexForGlyphIndex: 
-					 index]];
+  currentInfo = [_lineLayoutInfo objectAtIndex: 
+				   [self lineLayoutIndexForGlyphIndex: index]];
 
   if (lineFragmentRange)
     {
@@ -291,23 +341,51 @@ static NSCharacterSet *invSelectionWordGranularitySet;
   return currentInfo->lineFragmentRect;
 }
 
+- (NSRect)lineFragmentUsedRectForGlyphAtIndex: (unsigned)index 
+			       effectiveRange: (NSRange *)lineFragmentRange
+{
+  _GNULineLayoutInfo *currentInfo;
+  
+  if (![_textStorage length] || ![_lineLayoutInfo count])
+    {
+      // we return the line fragment from the virtual newline
+      // that we added at the end of the empty text
+      if (lineFragmentRange)
+	{
+	  lineFragmentRange->location=0;
+	  lineFragmentRange->length=0;
+	}
+      return ((_GNULineLayoutInfo *)[_lineLayoutInfo lastObject])->usedRect;
+    }
+  
+  currentInfo = [_lineLayoutInfo objectAtIndex: 
+				   [self lineLayoutIndexForGlyphIndex: index]];
+
+  if (lineFragmentRange)
+    {
+      *lineFragmentRange = currentInfo->glyphRange;
+    }
+
+  return currentInfo->usedRect;
+}
+
 - (NSPoint)locationForGlyphAtIndex:(unsigned)index
 {
   float x;
   unsigned start;
   _GNULineLayoutInfo *currentInfo;
   
-  if (![_textStorage length] || ![_lineLayoutInformation count])
+  if (![_textStorage length] || ![_lineLayoutInfo count])
     {
       return NSMakePoint(0, 0);
     }
     
-  currentInfo = [_lineLayoutInformation 
-		    objectAtIndex: [self lineLayoutIndexForGlyphIndex: 
-					     index]];
+  currentInfo = [_lineLayoutInfo objectAtIndex: 
+				   [self lineLayoutIndexForGlyphIndex: index]];
   if (index >= NSMaxRange (currentInfo->glyphRange))
     {
-      return NSMakePoint (NSMaxX (currentInfo->usedRect), 0);
+      x = [self _sizeOfRange: currentInfo->glyphRange].width;
+      return NSMakePoint (x, 0); 
     }
   
   start = currentInfo->glyphRange.location;
@@ -323,32 +401,84 @@ static NSCharacterSet *invSelectionWordGranularitySet;
 {
   _GNULineLayoutInfo *currentInfo;
   unsigned i1, i2;
-  NSRect rect1;
+  NSRect rect1, rect2;
+  NSSize size;
+  NSRect rect;
 
-  if (![_textStorage length] || ![_lineLayoutInformation count])
+  if (![_textStorage length] || ![_lineLayoutInfo count])
     {
-      return NSMakeRect(0, 0, 0, 12);
+      return NSMakeRect (0, 0, 0, defaultFontHeight ());
     }
 
   i1 = [self lineLayoutIndexForGlyphIndex: aRange.location];
   i2 = [self lineLayoutIndexForGlyphIndex: NSMaxRange(aRange)];
 
-  // This is not exacty what we need, but should be correct enough
-  currentInfo = [_lineLayoutInformation objectAtIndex: i1];
-  rect1 = currentInfo->lineFragmentRect;
 
-  if (i1 != i2)
+  if (i1 == i2)
     {
-      currentInfo = [_lineLayoutInformation objectAtIndex: i2];
-      rect1 = NSUnionRect(rect1, currentInfo->lineFragmentRect);
+      // the range is not multiline.
+
+      currentInfo = [_lineLayoutInfo objectAtIndex: i1];
+      rect1 = currentInfo->usedRect;
+      size = [self _sizeOfRange: 
+		     NSMakeRange 
+		   (currentInfo->glyphRange.location, 
+		    aRange.location -
+		    currentInfo->glyphRange.location)];
+      rect1.origin.x += size.width;
+      rect1.size.width = [self _sizeOfRange: 
+				 NSMakeRange 
+			       (aRange.location -
+				currentInfo->glyphRange.location, 1)].width;
+      // rect1 is the rect for the first glyph in the range
+      
+      currentInfo = [_lineLayoutInfo objectAtIndex: i2];
+      rect2 = currentInfo->usedRect;
+      size = [self _sizeOfRange: 
+		     NSMakeRange 
+		   (currentInfo->glyphRange.location, 
+		    NSMaxRange(aRange) -
+		    currentInfo->glyphRange.location)];
+      rect2.origin.x += size.width;
+      rect2.size.width = [self _sizeOfRange: 
+				 NSMakeRange 
+			       (NSMaxRange(aRange) -
+				currentInfo->glyphRange.location, 1)].width;
+      // rect2 is the rect for the last glyph in the range
+
+      // TODO: it might not work if some glyph draw out of the usedRect.
+      // (is this really possible ?)
     }
   else
     {
-      //float width = [aTextContainer containerSize].width;
-      // FIXME: When we are on one line we may need only part of it
+      // this is a multiline range, therefore the bounding rect goes
+      // from the beginning of a line till the end
+      // getting the lineFragmentRects will give the correct result
+      currentInfo = [_lineLayoutInfo objectAtIndex: i1];
+      rect1 = currentInfo->lineFragmentRect;
+      
+      currentInfo = [_lineLayoutInfo objectAtIndex: i2];
+      rect2 = currentInfo->lineFragmentRect;
     }
 
-  return rect1;
+
+
+
+  // this does compute the smallest rect enclosing 
+  // rect1 (or rect1.origin if rect1 is empty)
+  // and rect2 (or rect2.origin if rect1 is empty)
+
+  rect = NSMakeRect(MIN(NSMinX(rect1), NSMinX(rect2)),
+                    MIN(NSMinY(rect1), NSMinY(rect2)), 0, 0);
+
+  rect = NSMakeRect(NSMinX(rect),
+                    NSMinY(rect),
+                    MAX(NSMaxX(rect1), NSMaxX(rect2)) - NSMinX(rect),
+                    MAX(NSMaxY(rect1), NSMaxY(rect2)) - NSMinY(rect));
+
+
+  return rect;
+
 }
 
 - (NSRange)glyphRangeForBoundingRect:(NSRect)aRect 
@@ -474,9 +604,22 @@ static NSCharacterSet *invSelectionWordGranularitySet;
   lineRange = [self rebuildForRange: aRange
 		    delta: delta
 		    inTextContainer: aTextContainer];
+
   [[aTextContainer textView] sizeToFit];
   [[aTextContainer textView] invalidateTextContainerOrigin];
 
+  if ([aTextContainer textView])
+    {
+      NSRange sr = [[aTextContainer textView] selectedRange];
+      if (sr.length == 0
+	  && aRange.location <= sr.location
+	  && aRange.location + aRange.length >= sr.location)
+	{
+	  [[aTextContainer textView] 
+	    updateInsertionPointStateAndRestartTimer: YES];
+	}
+    }
+
   /* FIXME - it was reported that at this point lineRange is no longer
    * correct ...  looks like sizeToFit / invalidateTextContainerOrigin
    * migth cause additional relayout.  */
@@ -493,7 +636,8 @@ static NSCharacterSet *invSelectionWordGranularitySet;
   unsigned end = [self lineLayoutIndexForGlyphIndex: NSMaxRange(glyphRange)];
   NSRange lineRange = NSMakeRange(start, end + 1 - start);
 
-  [self drawLinesInLineRange: lineRange];
+  [self drawLinesInLineRange: lineRange
+	atPoint: containerOrigin];
 
   // We have to redraw the part of the selection that is inside
   // the redrawn lines
@@ -502,7 +646,8 @@ static NSCharacterSet *invSelectionWordGranularitySet;
   if ((selectedRange.length &&
        NSLocationInRange(newRange.location, selectedRange)))
     {
-      [self drawSelectionAsRangeNoCaret: newRange];
+      [self drawSelectionAsRangeNoCaret: newRange
+	    atPoint: containerOrigin];
     }
 }
 
@@ -510,11 +655,10 @@ static NSCharacterSet *invSelectionWordGranularitySet;
 	       forGlyphRange: (NSRange)glyphRange
 		    usedRect: (NSRect)usedRect
 {
-  [_lineLayoutInformation addObject:
-      [_GNULineLayoutInfo
-	  lineLayoutWithRange: glyphRange
-	  rect: fragmentRect
-	  usedRect: usedRect]];
+  [_lineLayoutInfo addObject: [_GNULineLayoutInfo
+				lineLayoutWithRange: glyphRange
+				rect: fragmentRect
+				usedRect: usedRect]];
 }
 
 - (void) setLocation: (NSPoint)aPoint
@@ -542,12 +686,20 @@ forStartOfGlyphRange: (NSRange)glyphRange
 {
   int i;
   int min = 0;
-  int max = MAX(0, [_lineLayoutInformation count] - 1);
+  int max = MAX(0, (int)[_lineLayoutInfo count] - 1);
   float y = point.y;
-  float fmin = NSMinY([[_lineLayoutInformation objectAtIndex: 0] lineFragmentRect]);
-  float fmax = NSMaxY([[_lineLayoutInformation lastObject] lineFragmentRect]);
+  float fmin;
+  float fmax;
   NSRect rect;
 
+  if (max == 0)
+    {
+      return 0;
+    }
+
+  fmin = NSMinY([[_lineLayoutInfo objectAtIndex: 0] lineFragmentRect]);
+  fmax = NSMaxY([[_lineLayoutInfo lastObject] lineFragmentRect]);
+
   if (y >= fmax)
     return max;
 
@@ -558,7 +710,7 @@ forStartOfGlyphRange: (NSRange)glyphRange
   i = (int)((max - min) * (y - fmin) / (fmax - fmin)) + min;
   while (min < max)
     {
-      _GNULineLayoutInfo *ci = [_lineLayoutInformation objectAtIndex: i];
+      _GNULineLayoutInfo *ci = [_lineLayoutInfo objectAtIndex: i];
 
       rect = ci->lineFragmentRect;
 
@@ -584,17 +736,19 @@ forStartOfGlyphRange: (NSRange)glyphRange
 {
   int i;
   int min = 0;
-  int max = MAX(0, (int)[_lineLayoutInformation count] - 1);
+  int max = MAX(0, (int)[_lineLayoutInfo count] - 1);
   unsigned y = anIndex;
   unsigned fmin;
   unsigned fmax;
   NSRange range;
 
-  if (!max)
-    return 0;
+  if (max == 0)
+    {
+      return 0;
+    }
 
-  fmin = [[_lineLayoutInformation objectAtIndex: 0] glyphRange].location;
-  fmax = NSMaxRange([[_lineLayoutInformation lastObject] glyphRange]);
+  fmin = [[_lineLayoutInfo objectAtIndex: 0] glyphRange].location;
+  fmax = NSMaxRange([[_lineLayoutInfo lastObject] glyphRange]);
 
   if (y >= fmax)
     return max;
@@ -606,7 +760,7 @@ forStartOfGlyphRange: (NSRange)glyphRange
   i = (int)((max - min) * (y - fmin) / (fmax - fmin)) + min;
   while (min < max)
     {
-      _GNULineLayoutInfo *ci = [_lineLayoutInformation objectAtIndex: i];
+      _GNULineLayoutInfo *ci = [_lineLayoutInfo objectAtIndex: i];
 
       range = ci->glyphRange;
 
@@ -636,16 +790,21 @@ forStartOfGlyphRange: (NSRange)glyphRange
   unsigned startIndex;
   unsigned endIndex;
 
-  if (startLine >= [_lineLayoutInformation count])
-    currentInfo = [_lineLayoutInformation lastObject];
+  if ([_lineLayoutInfo count] == 0)
+    {
+      return NSMakeRange (0, 0);
+    }
+
+  if (startLine >= [_lineLayoutInfo count])
+    currentInfo = [_lineLayoutInfo lastObject];
   else
-    currentInfo = [_lineLayoutInformation objectAtIndex: startLine];
+    currentInfo = [_lineLayoutInfo objectAtIndex: startLine];
   startIndex = currentInfo->glyphRange.location;
 
-  if (endLine >= [_lineLayoutInformation count])
-    currentInfo = [_lineLayoutInformation lastObject];
+  if (endLine >= [_lineLayoutInfo count])
+    currentInfo = [_lineLayoutInfo lastObject];
   else
-    currentInfo = [_lineLayoutInformation objectAtIndex: endLine];
+    currentInfo = [_lineLayoutInfo objectAtIndex: endLine];
   endIndex = NSMaxRange(currentInfo->glyphRange);
 
   return NSMakeRange(startIndex, endIndex - startIndex);
@@ -664,15 +823,15 @@ forStartOfGlyphRange: (NSRange)glyphRange
   container = [self textContainerForGlyphAtIndex: index effectiveRange: NULL];
   if (container)
     width = [container containerSize].width;
-  if (![_textStorage length] || ![_lineLayoutInformation count])
+  if (![_textStorage length] || ![_lineLayoutInfo count])
     {
-      return NSMakeRect(0, 0, width, 12);
+      return NSMakeRect(0, 0, width, defaultFontHeight ());
     }
 
-  currentInfo = [_lineLayoutInformation lastObject];
+  currentInfo = [_lineLayoutInfo lastObject];
   if (index >= NSMaxRange(currentInfo->glyphRange))
     {
-      NSRect rect = currentInfo->lineFragmentRect;
+      NSRect rect = currentInfo->usedRect;
 
       return NSMakeRect(NSMaxX (rect), rect.origin.y,
 			width - NSMaxX (rect),
@@ -680,11 +839,10 @@ forStartOfGlyphRange: (NSRange)glyphRange
     }
 
 
-  currentInfo = [_lineLayoutInformation 
-		  objectAtIndex: [self lineLayoutIndexForGlyphIndex: 
-					 index]];
+  currentInfo = [_lineLayoutInfo objectAtIndex: 
+				   [self lineLayoutIndexForGlyphIndex: index]];
   start = currentInfo->glyphRange.location;
-  rect = currentInfo->lineFragmentRect;
+  rect = currentInfo->usedRect;
   x = rect.origin.x + [self _sizeOfRange: NSMakeRange(start, index-start)].width;
      
   return NSMakeRect(x, rect.origin.y, NSMaxX (rect) - x,
@@ -692,6 +850,7 @@ forStartOfGlyphRange: (NSRange)glyphRange
 }
 
 - (void) drawSelectionAsRangeNoCaret: (NSRange)aRange
+			     atPoint: (NSPoint)containerOrigin
 {
   unsigned i, count;
   NSTextContainer *aTextContainer;
@@ -706,7 +865,9 @@ forStartOfGlyphRange: (NSRange)glyphRange
 
   for (i = 0; i < count; i++)
     {
-	NSHighlightRect (rects[i]);
+      rects[i].origin.x += containerOrigin.x;
+      rects[i].origin.y += containerOrigin.y;
+      NSHighlightRect (rects[i]);
     }
 }
 
@@ -718,49 +879,68 @@ forStartOfGlyphRange: (NSRange)glyphRange
 
   startLine = [self lineLayoutIndexForPoint: upperLeftPoint];
   endLine = [self lineLayoutIndexForPoint: lowerRightPoint];
-  if (++endLine > [_lineLayoutInformation count])
-    endLine = [_lineLayoutInformation count];
+  if (++endLine > [_lineLayoutInfo count])
+    endLine = [_lineLayoutInfo count];
 
   return NSMakeRange(startLine, endLine - startLine);
 }
 
-// relies on _lineLayoutInformation
-- (void) drawLinesInLineRange: (NSRange)aRange;
+// relies on _lineLayoutInfo
+- (void) drawLinesInLineRange: (NSRange)aRange
+		      atPoint: (NSPoint)containerOrigin
 {
-  NSArray *linesToDraw = [_lineLayoutInformation subarrayWithRange: aRange];
+  NSArray *linesToDraw;
   NSEnumerator *lineEnum;
   _GNULineLayoutInfo *currentInfo;
 
+  if ([_lineLayoutInfo count] == 0)
+    {
+      return;
+    }
+
+  linesToDraw = [_lineLayoutInfo subarrayWithRange: aRange];
+
   for ((lineEnum = [linesToDraw objectEnumerator]);
        (currentInfo = [lineEnum nextObject]);)
     {
+      NSRect rect;
+      rect = currentInfo->lineFragmentRect;
+      rect.origin.x += containerOrigin.x;
+      rect.origin.y += containerOrigin.y;
       [_textStorage drawRange: currentInfo->glyphRange
-		    inRect: currentInfo->lineFragmentRect];
+		    inRect: rect];
     }
 }
 
 - (void) setNeedsDisplayForLineRange: (NSRange)redrawLineRange
 		     inTextContainer: (NSTextContainer *)aTextContainer 
 {
-  if ([_lineLayoutInformation count]
-      && NSMaxRange (redrawLineRange) < [_lineLayoutInformation count]
+  if ([_lineLayoutInfo count]
+      && NSMaxRange (redrawLineRange) <= [_lineLayoutInfo count]
       && redrawLineRange.length)
     {
       _GNULineLayoutInfo *firstInfo
-	= [_lineLayoutInformation objectAtIndex: redrawLineRange.location];
+	= [_lineLayoutInfo objectAtIndex: redrawLineRange.location];
       NSRect displayRect = firstInfo->lineFragmentRect;
       float width = 0;
       if (aTextContainer)
 	width = [aTextContainer containerSize].width;
-
       if (redrawLineRange.length > 1)
 	  displayRect = NSUnionRect(displayRect,
-				    [[_lineLayoutInformation
+				    [[_lineLayoutInfo
 					 objectAtIndex: 
-					     (int)NSMaxRange(redrawLineRange) - 1] lineFragmentRect]);
+					     (int)NSMaxRange(redrawLineRange) 
+				       - 1] lineFragmentRect]);
 
-      displayRect.size.width = width - displayRect.origin.x;
-      [[aTextContainer textView] setNeedsDisplayInRect: displayRect];
+      displayRect.size.width = width;
+      if ([aTextContainer textView])
+	{
+	  displayRect.origin.x +=
+	    [[aTextContainer textView] textContainerInset].width;
+	  displayRect.origin.y +=
+	    [[aTextContainer textView] textContainerInset].height;
+	  [[aTextContainer textView] setNeedsDisplayInRect: displayRect];
+	}
     }
 }
 
@@ -768,7 +948,7 @@ forStartOfGlyphRange: (NSRange)glyphRange
 		    offset: (int)relocOffset
 		floatTrift: (float*)yDisplacement
 {
-  _GNULineLayoutInfo *lastInfo = [_lineLayoutInformation lastObject];
+  _GNULineLayoutInfo *lastInfo = [_lineLayoutInfo lastObject];
   // The start character in the ghostArray
   unsigned nextChar = NSMaxRange(lastInfo->glyphRange) - relocOffset;
   NSEnumerator *relocEnum;
@@ -806,7 +986,7 @@ NSDebugLog(@"Reloc %@ to %d", NSStringFromRange(currReloc->glyphRange), nextChar
 	  currReloc->lineFragmentRect.origin.y += yReloc;
 	  currReloc->usedRect.origin.y += yReloc;
 	}
-      [_lineLayoutInformation addObject:  currReloc];
+      [_lineLayoutInfo addObject:  currReloc];
     }
 
   return YES;
@@ -841,7 +1021,7 @@ scanRange(NSScanner *scanner, NSCharacterSet* aSet)
   NSString *allText = [_textStorage string];
   unsigned length = [allText length];
   unsigned paraPos;
-  int maxLines = [_lineLayoutInformation count];
+  int maxLines = [_lineLayoutInfo count];
   int aLine = 0;
   // for optimization detection
   NSMutableArray *ghostArray = nil;
@@ -856,7 +1036,7 @@ scanRange(NSScanner *scanner, NSCharacterSet* aSet)
       if (nextLine < maxLines)
 	{
 	  // remember old array for optimization purposes
-	  ghostArray = AUTORELEASE([[_lineLayoutInformation
+	  ghostArray = AUTORELEASE([[_lineLayoutInfo
 				      subarrayWithRange:
 					NSMakeRange (nextLine, 
 						     maxLines - nextLine)]
@@ -866,16 +1046,17 @@ scanRange(NSScanner *scanner, NSCharacterSet* aSet)
       aLine = MAX(0, insertionLineIndex - 1);
       if (aLine)
 	{
-	  _GNULineLayoutInfo *lastValidLineInfo = [_lineLayoutInformation 
+	  _GNULineLayoutInfo *lastValidLineInfo = [_lineLayoutInfo 
 						    objectAtIndex: aLine - 1];
-	  NSRect aRect = lastValidLineInfo->lineFragmentRect;
+	  NSRect aRect = lastValidLineInfo->usedRect;
 	  
 	  startingIndex = NSMaxRange(lastValidLineInfo->glyphRange);
-	  drawingPoint = aRect.origin;
+	  drawingPoint.x = 0;
+	  drawingPoint.y = aRect.origin.y;
 	  drawingPoint.y += aRect.size.height;
 	}
       
-      [_lineLayoutInformation removeObjectsInRange:
+      [_lineLayoutInfo removeObjectsInRange:
 				NSMakeRange (aLine, maxLines - aLine)];
     }
   
@@ -887,11 +1068,39 @@ scanRange(NSScanner *scanner, NSCharacterSet* aSet)
 
       // FIXME: This should be done via extra line fragment
       // If there is no text add one empty box
-      [_lineLayoutInformation
-	  addObject: [_GNULineLayoutInfo
-			 lineLayoutWithRange: NSMakeRange (0, 0)
-			 rect: NSMakeRect (0, 0, width, 12)
-			 usedRect: NSMakeRect (0, 0, 1, 12)]];
+      {
+	NSParagraphStyle *style = 
+	  (NSParagraphStyle*)[_textStorage
+			       attribute: NSParagraphStyleAttributeName
+			       atIndex: 0
+			       effectiveRange: NULL];
+	
+	if ([style alignment] == NSRightTextAlignment)
+	  {
+	    [_lineLayoutInfo
+	      addObject: [_GNULineLayoutInfo
+			   lineLayoutWithRange: NSMakeRange (0, 0)
+			   rect: NSMakeRect (0, 0, width, defaultFontHeight ())
+			   usedRect: NSMakeRect (width-1, 0, 1, defaultFontHeight ())]];
+	  }
+	else if ([style alignment] == NSCenterTextAlignment)
+	  {
+	    [_lineLayoutInfo
+	      addObject: [_GNULineLayoutInfo
+			   lineLayoutWithRange: NSMakeRange (0, 0)
+			   rect: NSMakeRect (0, 0, width, defaultFontHeight ())
+			   usedRect: NSMakeRect ((int) width/2, 0, 1, defaultFontHeight ())]];
+	  }
+	else
+	  {
+	    [_lineLayoutInfo
+	      addObject: [_GNULineLayoutInfo
+			   lineLayoutWithRange: NSMakeRange (0, 0)
+			   rect: NSMakeRect (0, 0, width, defaultFontHeight ())
+			   usedRect: NSMakeRect (0, 0, 1, defaultFontHeight ())]];
+	  }
+      }
+
       return NSMakeRange(0,1);
     }
       
@@ -966,13 +1175,13 @@ scanRange(NSScanner *scanner, NSCharacterSet* aSet)
 	  if (NSIsEmptyRect(fragmentRect))
  	    {
 	      // No more space in the text container, give up doing the layout
-	      int a = MAX (1, [_lineLayoutInformation count] - aLine);
+	      int a = MAX (1, [_lineLayoutInfo count] - aLine);
 	      return NSMakeRange(aLine, a);
 	    }
 
 	  width = fragmentRect.size.width - 2 * padding;
 	  usedLineRect = fragmentRect;
-	  usedLineRect.origin.x += padding;
+	  //	  usedLineRect.origin.x += padding;
 	  usedLineRect.size = NSZeroSize;
 	  
 	  // scan the individual words to the end of the line
@@ -1065,14 +1274,20 @@ scanRange(NSScanner *scanner, NSCharacterSet* aSet)
 	    {
 	      // newline - induced premature lineending: flush
 	      if (eol.length)
-	        {
+		{
 		  // Add information for the line break
 		  scannerPosition += eol.length;
 		  usedLineRect.size.width += 1;
-		  // FIXME: This should use the real font size!!
+		  // FIXME: This should use the real font size!! not
+                  // the default one !!
+		  // This should be fixed (04-07-02)
 		  if (usedLineRect.size.height == 0)
 		    {
-		      usedLineRect.size.height = 12;
+  		      // usedLineRect.size.height = defaultFontHeight ();
+		      usedLineRect.size.height = 
+			[self _sizeOfRange: NSMakeRange(startingLineCharIndex,
+							eol.length)].height;
+
 		    }
 		}
 	    }
@@ -1083,6 +1298,29 @@ scanRange(NSScanner *scanner, NSCharacterSet* aSet)
 	  // the to big
 	  fragmentRect.size.height = usedLineRect.size.height;
 	  
+	  // adjust the usedLineRect depending on the justification
+	  {
+	    NSParagraphStyle *style = 
+	      (NSParagraphStyle*)[_textStorage
+				   attribute: NSParagraphStyleAttributeName
+				   atIndex: startingLineCharIndex
+				   effectiveRange: NULL];
+
+	    if ([style alignment] == NSLeftTextAlignment)
+	      {
+	      }
+	    else if ([style alignment] == NSRightTextAlignment)
+	      {
+		usedLineRect.origin.x +=
+		  fragmentRect.size.width - usedLineRect.size.width - padding;
+	      }
+	    else if ([style alignment] == NSCenterTextAlignment)
+	      {
+		usedLineRect.origin.x +=
+		  (int) (fragmentRect.size.width - usedLineRect.size.width) / 2;
+	      }
+	  }
+
 	  // This range is to small, as there are more lines that fit
 	  // into the container
 	  [self setTextContainer: aTextContainer 
@@ -1120,7 +1358,7 @@ scanRange(NSScanner *scanner, NSCharacterSet* aSet)
 		  // y displacement: redisplay all remaining lines
 		  if (yDisplacement)
 		    {
-		      erg = [_lineLayoutInformation count] - aLine;
+		      erg = [_lineLayoutInfo count] - aLine;
 		    }
 		  else 
 		    {
@@ -1144,19 +1382,60 @@ scanRange(NSScanner *scanner, NSCharacterSet* aSet)
       if (aTextContainer)
 	width = [aTextContainer containerSize].width;
 
-      [_lineLayoutInformation
-	addObject: [_GNULineLayoutInfo
-		     lineLayoutWithRange: NSMakeRange (length, 0)
-		     rect: NSMakeRect (drawingPoint.x, drawingPoint.y, 
-				       width, 12)
-		     usedRect: NSMakeRect (drawingPoint.x, drawingPoint.y, 
-					   1, 12)]];      
+      {
+	NSParagraphStyle *style = 
+	  (NSParagraphStyle*)[_textStorage
+			       attribute: NSParagraphStyleAttributeName
+			       atIndex: length - 1
+			       effectiveRange: NULL];
+	
+	if ([style alignment] == NSRightTextAlignment)
+	  {
+	    [_lineLayoutInfo
+	      addObject: [_GNULineLayoutInfo
+			   lineLayoutWithRange: NSMakeRange (length, 0)
+			   rect:
+			     NSMakeRect (drawingPoint.x 
+					 + (width - drawingPoint.x),
+					 drawingPoint.y, 
+					 1, defaultFontHeight ())
+			   usedRect: 
+			     NSMakeRect (drawingPoint.x
+					 + (width - drawingPoint.x),
+					 drawingPoint.y, 
+					 1, defaultFontHeight ())]];
+	  }
+	else if ([style alignment] == NSCenterTextAlignment)
+	  {
+	    [_lineLayoutInfo
+	      addObject: [_GNULineLayoutInfo
+			   lineLayoutWithRange: NSMakeRange (length, 0)
+			   rect: 
+			     NSMakeRect (drawingPoint.x 
+					 + (int)(width - drawingPoint.x) / 2,
+					 drawingPoint.y, 
+					 1, defaultFontHeight ())
+			   usedRect: 
+			     NSMakeRect (drawingPoint.x
+					 + (int)(width - drawingPoint.x) / 2,
+					 drawingPoint.y, 
+					 1, defaultFontHeight ())]];      
+	  }
+	else
+	  {
+	    [_lineLayoutInfo
+	      addObject: [_GNULineLayoutInfo
+			   lineLayoutWithRange: NSMakeRange (length, 0)
+			   rect: NSMakeRect (drawingPoint.x, drawingPoint.y, 
+					     width, defaultFontHeight ())
+			   usedRect: NSMakeRect (drawingPoint.x, drawingPoint.y, 
+						 1, defaultFontHeight ())]];      
+	  }
+      }
     }
   
-
-  
   // lines actually updated (optimized drawing)
-  return NSMakeRange(aLine, MAX(1, [_lineLayoutInformation count] - aLine));
+  return NSMakeRange(aLine, MAX(1, [_lineLayoutInfo count] - aLine));
 }
 
 
@@ -1189,7 +1468,7 @@ scanRange(NSScanner *scanner, NSCharacterSet* aSet)
     {
       return 0;
     }
-  else if (upFlag == NO  &&  line == ([_lineLayoutInformation count] - 1))
+  else if (upFlag == NO  &&  line == ([_lineLayoutInfo count] - 1))
     {
       return [_textStorage length];
     }
@@ -1204,7 +1483,7 @@ scanRange(NSScanner *scanner, NSCharacterSet* aSet)
       line += 1;
     }
 
-  lineInfo = [_lineLayoutInformation objectAtIndex: line];
+  lineInfo = [_lineLayoutInfo objectAtIndex: line];
   
   rect  = lineInfo->usedRect;
   range = lineInfo->glyphRange;
