@@ -25,22 +25,10 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#include <AppKit/GSTable.h>
-
-// This is meant to make drawing invisible views a little faster
-@interface GSTransparentView : NSView
-@end 
-@implementation GSTransparentView
--(void) lockFocusInRect: (NSRect)rect 
-{
-}
--(void) unlockFocusNeedsFlush: (BOOL)flush 
-{
-}
-@end 
-
+#include "GNUstepGUI/GSTable.h"
 
 @interface GSTable (Private)
+-(void) _updateForNewFrameSize: (NSSize)newFrameSize;
 -(void) _updateRowSize: (int)row;
 -(void) _updateColumnSize: (int)column;
 -(void) _updateRowOrigin: (int)row;
@@ -57,6 +45,15 @@
   if (self == [GSTable class])
     [self setVersion: 1];
 }
+
+/* It was reported that the inherited +new implementation doesn't work
+ * on OSX.  Override it with a sane implementation to get portability
+ * to OSX.  */
++(id) new
+{
+  return [[self alloc] init];
+}
+
 //
 // Instance Methods
 //
@@ -89,7 +86,7 @@
   _maxYBorder = 0;
 
   _jails = NSZoneMalloc (NSDefaultMallocZone (), 
-			 sizeof (GSTransparentView *) * (rows * columns));
+			 sizeof (NSView *) * (rows * columns));
   _expandRow = NSZoneMalloc (NSDefaultMallocZone (), 
 			     sizeof (BOOL) * rows);
   _expandColumn = NSZoneMalloc (NSDefaultMallocZone (), 
@@ -200,7 +197,7 @@
   
   _minimumSize.width += borderChange;
   tableSize.width += borderChange; 
-  [self setFrameSize: tableSize];
+  [super setFrameSize: tableSize];
 
   _minXBorder = aBorder;
 }
@@ -216,7 +213,7 @@
   
   _minimumSize.width += borderChange;
   tableSize.width += borderChange; 
-  [self setFrameSize: tableSize];
+  [super setFrameSize: tableSize];
 
   _maxXBorder = aBorder;
 }
@@ -240,7 +237,7 @@
   
   _minimumSize.height += borderChange;
   tableSize.height += borderChange; 
-  [self setFrameSize: tableSize];
+  [super setFrameSize: tableSize];
 
   _minYBorder = aBorder;
 }
@@ -256,7 +253,7 @@
   
   _minimumSize.height += borderChange;
   tableSize.height += borderChange; 
-  [self setFrameSize: tableSize];
+  [super setFrameSize: tableSize];
 
   _maxYBorder = aBorder;
 }
@@ -415,7 +412,7 @@
   
   if (tableNeedResize)
     {
-      [self setFrameSize: tableFrame.size];
+      [super setFrameSize: tableFrame.size];
     }
   
       
@@ -447,11 +444,9 @@
   else // !_havePrisoner
     {
       if (prisonerNeedResize)
-	_jails[jailNumber] = [[GSTransparentView alloc] 
-			       initWithFrame: oldFrame];
+	_jails[jailNumber] = [[NSView alloc] initWithFrame: oldFrame];
       else // !prisonerNeedResize
-	_jails[jailNumber] = [[GSTransparentView alloc] 
-			       initWithFrame: theFrame];
+	_jails[jailNumber] = [[NSView alloc] initWithFrame: theFrame];
 
       [_jails[jailNumber] setAutoresizingMask: NSViewNotSizable];
       [_jails[jailNumber] setAutoresizesSubviews: YES];
@@ -478,101 +473,21 @@
     [_jails[jailNumber] setFrame: theFrame];
   _havePrisoner[jailNumber] = YES;
 } 
--(void) resizeWithOldSuperviewSize: (NSSize)oldSize
+
+/* resizeWithOldSuperviewSize: automatically calls setFrame: */
+
+- (void) setFrame: (NSRect)frame
 {
-  float originShift;
-  float dimensionIncrement;
-  unsigned int i;
-  // YES if the whole GSTable needs an update
-  BOOL tableNeedUpdate = NO;
-  NSSize oldFrameSize = [self frame].size;
-  NSSize newFrameSize;
-  
-  [super resizeWithOldSuperviewSize: oldSize];
-  newFrameSize = [self frame].size; 
-
-  //
-  // Width
-  //
-  if (newFrameSize.width <= _minimumSize.width)
-    {
-      if (oldFrameSize.width > _minimumSize.width)
-	{
-	  originShift = _minXBorder;
-	  for (i = 0; i < _numberOfColumns; i++)
-	    {  
-	      _columnDimension[i] = _minColumnDimension[i];
-	      _columnXOrigin[i] = originShift;
-	      originShift += _minColumnDimension[i];
-	    }
-	  tableNeedUpdate = YES;
-	}
-    }
-    else // newFrameSize.width > _minimumSize.width
-    {
-      if (oldFrameSize.width < _minimumSize.width)
-	oldFrameSize.width = _minimumSize.width;
-
-      if ((newFrameSize.width != oldFrameSize.width) && _expandingColumnNumber)
-	{  
-	  originShift = 0;
-	  dimensionIncrement = newFrameSize.width - oldFrameSize.width;
-	  dimensionIncrement = dimensionIncrement / _expandingColumnNumber; 
-	  for (i = 0; i < _numberOfColumns; i++)
-	    {
-	      _columnXOrigin[i] += originShift;
-	      if (_expandColumn[i])
-		{
-		  _columnDimension[i] += dimensionIncrement;
-		  originShift += dimensionIncrement;
-		}
-	    }
-	  tableNeedUpdate = YES;
-	}
-    }
-  //
-  // Height 
-  //
-  if (newFrameSize.height <= _minimumSize.height)
-    {
-      if (oldFrameSize.height > _minimumSize.height)
-	{
-	  originShift = _minYBorder;
-	  for (i = 0; i < _numberOfRows; i++)
-	    {  
-	      _rowDimension[i] = _minRowDimension[i];
-	      _rowYOrigin[i] = originShift;
-	      originShift += _minRowDimension[i];
-	    }
-	  tableNeedUpdate = YES;
-	}
-    }
-    else // newFrameSize.height > _minimumSize.height
-    {
-      if (oldFrameSize.height < _minimumSize.height)
-	oldFrameSize.height = _minimumSize.height;
-
-      if ((newFrameSize.height != oldFrameSize.height) && _expandingRowNumber)
-	{  
-	  originShift = 0;
-	  dimensionIncrement = newFrameSize.height - oldFrameSize.height;
-	  dimensionIncrement = dimensionIncrement / _expandingRowNumber; 
-	  for (i = 0; i < _numberOfRows; i++)
-	    {
-	      _rowYOrigin[i] += originShift;
-	      if (_expandRow[i])
-		{
-		  _rowDimension[i] += dimensionIncrement;
-		  originShift += dimensionIncrement;
-		}
-	    }
-	  tableNeedUpdate = YES;
-	}
-    }
-
-  if (tableNeedUpdate)
-    [self _updateWholeTable];
+  [self _updateForNewFrameSize: frame.size];
+  [super setFrame: frame];
 }
+
+- (void) setFrameSize: (NSSize)newFrameSize
+{
+  [self _updateForNewFrameSize: newFrameSize];
+  [super setFrameSize: newFrameSize];
+}
+
 //
 // Minimum Size
 //
@@ -590,7 +505,7 @@
   // This should never happen but anyway. 
   if ((_numberOfColumns == 0) || (_numberOfRows == 0))
     {
-      [self setFrameSize: NSZeroSize];
+      [super setFrameSize: NSZeroSize];
       return;
     }
   
@@ -611,7 +526,7 @@
       _rowDimension[i] = _minRowDimension[i];
     }
   [self _updateWholeTable];
-  [self setFrameSize: _minimumSize];
+  [super setFrameSize: _minimumSize];
 }
 //
 // Adding Rows and Columns
@@ -629,7 +544,7 @@
 				 * (sizeof (BOOL)));
   _jails = NSZoneRealloc (NSDefaultMallocZone (), _jails, 
 			 (_numberOfRows * _numberOfColumns) 
-			  * sizeof (GSTransparentView *));
+			  * sizeof (NSView *));
 
   for (j = (_numberOfRows - 1) * _numberOfColumns; 
        j < (_numberOfRows * _numberOfColumns); j++)
@@ -671,7 +586,7 @@
 				 * (sizeof (BOOL)));
   _jails = NSZoneRealloc (NSDefaultMallocZone (), _jails, 
 			  (_numberOfRows * _numberOfColumns) 
-			  * sizeof (GSTransparentView *));
+			  * sizeof (NSView *));
   // Reorder the jails
   for (j = (_numberOfRows - 1); j >= 0; j--)
     {
@@ -845,7 +760,7 @@
 
   // 
   _jails = NSZoneMalloc (NSDefaultMallocZone (), 
-			sizeof (GSTransparentView *) 
+			sizeof (NSView *) 
 			 * (_numberOfRows * _numberOfColumns));
 
   _havePrisoner = NSZoneMalloc (NSDefaultMallocZone (), 
@@ -922,16 +837,105 @@
 
   return self;
 }
-// These are meant to speed things up, since the table is invisible. 
--(void) lockFocusInRect: (NSRect)rect 
-{
-}
--(void) unlockFocusNeedsFlush: (BOOL)flush 
-{
-}
 @end
 
 @implementation GSTable (Private)
+/* Updates the subviews and locations for a new frame size.  */
+- (void) _updateForNewFrameSize: (NSSize)newFrameSize
+{
+  NSSize oldFrameSize = [self frame].size;
+  float originShift;
+  float dimensionIncrement;
+  int i;
+  // YES if the whole GSTable needs an update
+  BOOL tableNeedUpdate = NO;
+
+  //
+  // Width
+  //
+  if (newFrameSize.width <= _minimumSize.width)
+    {
+      if (oldFrameSize.width > _minimumSize.width)
+	{
+	  originShift = _minXBorder;
+	  for (i = 0; i < _numberOfColumns; i++)
+	    {  
+	      _columnDimension[i] = _minColumnDimension[i];
+	      _columnXOrigin[i] = originShift;
+	      originShift += _minColumnDimension[i];
+	    }
+	  tableNeedUpdate = YES;
+	}
+    }
+    else // newFrameSize.width > _minimumSize.width
+    {
+      if (oldFrameSize.width < _minimumSize.width)
+	oldFrameSize.width = _minimumSize.width;
+
+      if ((newFrameSize.width != oldFrameSize.width) && _expandingColumnNumber)
+	{  
+	  originShift = 0;
+	  dimensionIncrement = newFrameSize.width - oldFrameSize.width;
+	  dimensionIncrement = dimensionIncrement / _expandingColumnNumber; 
+	  for (i = 0; i < _numberOfColumns; i++)
+	    {
+	      _columnXOrigin[i] += originShift;
+	      if (_expandColumn[i])
+		{
+		  _columnDimension[i] += dimensionIncrement;
+		  originShift += dimensionIncrement;
+		}
+	    }
+	  tableNeedUpdate = YES;
+	}
+    }
+  //
+  // Height 
+  //
+  if (newFrameSize.height <= _minimumSize.height)
+    {
+      if (oldFrameSize.height > _minimumSize.height)
+	{
+	  originShift = _minYBorder;
+	  for (i = 0; i < _numberOfRows; i++)
+	    {  
+	      _rowDimension[i] = _minRowDimension[i];
+	      _rowYOrigin[i] = originShift;
+	      originShift += _minRowDimension[i];
+	    }
+	  tableNeedUpdate = YES;
+	}
+    }
+    else // newFrameSize.height > _minimumSize.height
+    {
+      if (oldFrameSize.height < _minimumSize.height)
+	oldFrameSize.height = _minimumSize.height;
+
+      if ((newFrameSize.height != oldFrameSize.height) && _expandingRowNumber)
+	{  
+	  originShift = 0;
+	  dimensionIncrement = newFrameSize.height - oldFrameSize.height;
+	  dimensionIncrement = dimensionIncrement / _expandingRowNumber; 
+	  for (i = 0; i < _numberOfRows; i++)
+	    {
+	      _rowYOrigin[i] += originShift;
+	      if (_expandRow[i])
+		{
+		  _rowDimension[i] += dimensionIncrement;
+		  originShift += dimensionIncrement;
+		}
+	    }
+	  tableNeedUpdate = YES;
+	}
+    }
+
+  if (tableNeedUpdate)
+    {
+      [self _updateWholeTable];
+    }
+}
+
+
 //
 // After computing new theoretical sizes/positions,
 // use the following methods to update the real table view
