@@ -25,7 +25,7 @@
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
 */
-#include <AppKit/NSTextStorage.h>
+#include "AppKit/NSTextStorage.h"
 
 @class NSMutableString;
 @class NSMutableArray;
