@@ -45,7 +45,10 @@
 #include <Foundation/NSArray.h>
 #include <Foundation/NSDebug.h>
 #include <Foundation/NSZone.h>
-#include <AppKit/NSTextStorage.h>
+#include <Foundation/NSLock.h>
+#include <Foundation/NSThread.h>
+#include <Foundation/NSNotification.h>
+#include "AppKit/NSTextStorage.h"
 #include "GSTextStorage.h"
 
 #define		SANITY_CHECKS	0
@@ -59,7 +62,7 @@
 #define GSI_MAP_VTYPES	GSUNION_INT
 #define	GSI_MAP_NOCLEAN	1
 
-#include <base/GSIMap.h>
+#include <GNUstepBase/GSIMap.h>
 
 static NSDictionary	*blank;
 static NSLock		*attrLock = nil;
@@ -285,6 +288,7 @@ _setAttributesFrom(
   if (aRange.length <= 0)
     {
       attr = blank;
+      range = aRange; /* Set to satisfy the loop condition below. */
     }
   else
     {
@@ -396,8 +400,13 @@ _attributesAtIndexEffectiveRange(
 #if	SANITY_CHECKS
 
 #define	SANITY()	[self sanity]
-	
-- (void) sanity
+#else
+#define	SANITY()	
+#endif
+
+/* We always compile in this method so that it is available from
+ * regression test cases.  */
+- (void) _sanity
 {
   GSTextInfo	*info;
   unsigned	i;
@@ -416,9 +425,6 @@ _attributesAtIndexEffectiveRange(
       l = info->loc;
     }
 }
-#else
-#define	SANITY()	
-#endif
 
 /*
  * If we are multi-threaded, we must guard access to the uniquing set.
@@ -543,6 +549,7 @@ _attributesAtIndexEffectiveRange(
 {
   unsigned	tmpLength, arrayIndex, arraySize;
   NSRange	effectiveRange;
+  NSRange	originalRange = range;
   unsigned	afterRangeLoc, beginRangeLoc;
   NSDictionary	*attrs;
   NSZone	*z = [self zone];
@@ -551,6 +558,7 @@ _attributesAtIndexEffectiveRange(
   if (range.length == 0)
     {
       NSWarnMLog(@"Attempt to set attribute for zero-length range", 0);
+      return;
     }
   if (attributes == nil)
     {
@@ -638,6 +646,7 @@ SANITY();
       if (info->attrs == attributes)
 	{
 	  unCacheAttributes(attributes);
+	  RELEASE(attributes);
 	}
       else
 	{
@@ -649,6 +658,7 @@ SANITY();
   else if (info->attrs == attributes)
     {
       unCacheAttributes(attributes);
+      RELEASE(attributes);
     }
   else
     {
@@ -660,7 +670,7 @@ SANITY();
   
 SANITY();
   [self edited: NSTextStorageEditedAttributes
-	 range: range
+	 range: originalRange
 changeInLength: 0];
 }
 
@@ -728,7 +738,7 @@ SANITY();
       info = OBJECTAT(arrayIndex);
       if (info->loc < NSMaxRange(range))
 	{
-	  int	next = arrayIndex + 1;
+	  unsigned int	next = arrayIndex + 1;
 
 	  while (next < arraySize)
 	    {
