@@ -28,14 +28,10 @@
 
 // See GSHbox.m for comments
 // This file is derived from GSVbox.m
-#include <AppKit/GSVbox.h>
-#include <AppKit/NSBox.h>
+#include "GNUstepGUI/GSVbox.h"
+#include "AppKit/NSBox.h"
 
 @implementation GSVbox: GSTable
-{
-  BOOL _haveViews;
-  float _defaultMinYMargin;
-}
 //
 // Class methods
 //
