@@ -343,26 +343,33 @@ NSApplication	*NSApp = nil;
 // Class variables
 static NSCell* dragCell = nil;
 static NSCell* tileCell = nil;
-static BOOL useWindowmakerIconBackground = NO;
+
+static NSSize scaledIconSizeForSize(NSSize imageSize)
+{
+  NSSize iconSize, retSize;
+  
+  iconSize = [GSCurrentServer() iconSize];
+  retSize.width = imageSize.width * iconSize.width / 64;
+  retSize.height = imageSize.height * iconSize.height / 64;
+  return retSize;
+}
 
 + (void) initialize
 {
-  NSImage	*defImage = [NSImage imageNamed: @"GNUstep"];
   NSImage	*tileImage;
+  NSSize	iconSize;
 
-  dragCell = [[NSCell alloc] initImageCell: defImage];
+  iconSize = [GSCurrentServer() iconSize];
+  /* _appIconInit will set our image */
+  dragCell = [[NSCell alloc] initImageCell: nil];
   [dragCell setBordered: NO];
-  if ([[NSUserDefaults standardUserDefaults]
-	  boolForKey: @"GSUseWindowmakerIconBackground"])
-    {
-      useWindowmakerIconBackground = YES; 
-    }
-  else
-    {
-      tileImage = [NSImage imageNamed: @"common_Tile"];
-      tileCell = [[NSCell alloc] initImageCell: tileImage];
-      [tileCell setBordered: NO];
-    }
+  
+  tileImage = [[GSCurrentServer() iconTileImage] copy];
+  [tileImage setScalesWhenResized: YES];
+  [tileImage setSize: iconSize];
+  tileCell = [[NSCell alloc] initImageCell: tileImage];
+  RELEASE(tileImage);
+  [tileCell setBordered: NO];
 }
 
 - (BOOL) acceptsFirstMouse: (NSEvent*)theEvent
@@ -390,20 +397,13 @@ static BOOL useWindowmakerIconBackground = NO;
 
 - (void) drawRect: (NSRect)rect
 {
-  if (useWindowmakerIconBackground)
-    {
-      PScompositerect(_bounds.origin.x,
-      		      _bounds.origin.y,
-		      _bounds.size.width,
-		      _bounds.size.height,
-		      NSCompositeClear);
-    }
-  else
-    {
-      [tileCell drawWithFrame: NSMakeRect(0,0,64,64) inView: self];
-    }
+  NSSize iconSize = [GSCurrentServer() iconSize];
+  
+  [tileCell drawWithFrame: NSMakeRect(0, 0, iconSize.width, iconSize.height)
+  		   inView: self];
+  [dragCell drawWithFrame: NSMakeRect(0, 0, iconSize.width, iconSize.height)
+		   inView: self];
   
-  [dragCell drawWithFrame: NSMakeRect(8,8,48,48) inView: self];
   if ([NSApp isHidden])
     {
       NSRectEdge mySides[] = {NSMinXEdge, NSMinYEdge, NSMaxXEdge, NSMaxYEdge};
@@ -424,6 +424,64 @@ static BOOL useWindowmakerIconBackground = NO;
 {
   if ([theEvent clickCount] >= 2)
     {
+      /* If the app is not hidden we need to order front any visible and 
+      	 possibly obscured windows. If we are active we need to order front 
+	 the key or main window again, otherwise unhide: will unhide and
+	 activate us. Do this all in a way which will not change the stacking
+	 order. 
+
+	 There are 3 possibilities:
+	 1. app is active and has no obscured windows
+	 2. app is active and has obscured windows
+	 3. app is inactive.  
+	 I don't know of any way to tell 1 from 2 and do nothing.
+
+	 This should ideally set aWin NSWindowBelow the key or main window
+	 but with most windowmanagers. It doesn't seem to place the receiving 
+	 window immediately below like it should. 
+       */
+      if ([NSApp isHidden] == NO)
+	{
+	  int i;
+	  NSArray *windows = RETAIN([NSApp windows]);
+      
+	  for (i = 0; i < [windows count]; i++)
+	    { 
+	      NSWindow *aWin = [windows objectAtIndex:i];
+              
+	      if ([aWin isVisible] == YES && [aWin isMiniaturized] == NO
+		  && aWin != [NSApp keyWindow] && aWin != [NSApp mainWindow]
+		  && aWin != [self window] 
+		  && ([aWin styleMask] & NSMiniWindowMask) == 0)
+	        {
+		  [aWin orderFrontRegardless];
+		}
+	    }
+	
+	  if ([NSApp isActive] == YES)
+	    {
+	      if ([NSApp keyWindow] != nil)
+		{
+		  [[NSApp keyWindow] orderFront: self];
+		}
+	      else if ([NSApp mainWindow] != nil)
+		{
+		  [[NSApp mainWindow] makeKeyAndOrderFront: self];
+		}
+	      else
+		{
+		  /* We need give input focus to some window otherwise we'll 
+		     never get keyboard events. FIXME: doesn't work. */
+		  NSWindow *menu_window= [[NSApp mainMenu] window];
+		  NSDebugLLog(@"Focus", @"No key on activation - make menu key");
+		  [GSServerForWindow(menu_window) setinputfocus:
+					[menu_window windowNumber]];
+		}
+	    }
+	  
+          RELEASE(windows);
+	}
+       
       [NSApp unhide: self];
     }
   else
@@ -502,14 +560,13 @@ static BOOL useWindowmakerIconBackground = NO;
 
 - (void) setImage: (NSImage *)anImage
 {
-  [dragCell setImage: anImage];
-
-  if ([self lockFocusIfCanDraw])
-    {
-      [self drawRect: NSMakeRect(0,0,64,64)];
-      [self unlockFocus];
-      [_window flushWindow];
-    }
+  NSImage *imgCopy = [anImage copy];
+  NSSize imageSize = [imgCopy size]; 
+  [imgCopy setScalesWhenResized: YES];
+  [imgCopy setSize: scaledIconSizeForSize(imageSize)];
+  [dragCell setImage: imgCopy];
+  RELEASE(imgCopy);
+  [self setNeedsDisplay: YES];
 }
 
 @end
@@ -629,7 +686,7 @@ static BOOL useWindowmakerIconBackground = NO;
   GSDisplayServer *srv;
   /* Initialization must be enclosed in an autorelease pool.  */
   CREATE_AUTORELEASE_POOL (_app_init_pool);
-  
+
   /* 
    * Set NSApp as soon as possible, since other gui classes (which
    * we refer or use in this method) might be calling [NSApplication
@@ -637,26 +694,26 @@ static BOOL useWindowmakerIconBackground = NO;
    * those calls to succeed.  
    */
   NSApp = self;
-  
+
   /* Initialize the backend here.  */
   initialize_gnustep_backend();
 
   /* Load user-defined bundles */
   gsapp_user_bundles();
-  
+
   /* Connect to our window server.  */
   srv = [GSDisplayServer serverWithAttributes: nil];
   RETAIN(srv);
   [GSDisplayServer setCurrentServer: srv];
-  
+
   /* Create a default context.  */
   _default_context = [NSGraphicsContext graphicsContextWithAttributes: nil];
   RETAIN(_default_context);
   [NSGraphicsContext setCurrentContext: _default_context];
-  
+
   /* Initialize font manager.  */
   [NSFontManager sharedFontManager];
-  
+
   _hidden = [[NSMutableArray alloc] init];
   _inactive = [[NSMutableArray alloc] init];
   _unhide_on_activation = YES;
@@ -666,19 +723,19 @@ static BOOL useWindowmakerIconBackground = NO;
   //_app_is_active = NO;
   //_main_menu = nil;
   _windows_need_update = YES;
-  
+
   /* Set a new exception handler for the gui library.  */
   NSSetUncaughtExceptionHandler (_NSAppKitUncaughtExceptionHandler);
-  
+
   _listener = [GSServicesManager newWithApplication: self];
-  
+
   /* NSEvent doesn't use -init so we use +alloc instead of +new.  */
   _current_event = [NSEvent alloc]; // no current event
   null_event = [NSEvent alloc];    // create dummy event
-  
+
   /* We are the end of responder chain.  */
   [self setNextResponder: nil];
-  
+
   RELEASE (_app_init_pool);
 }
 
@@ -869,10 +926,12 @@ static BOOL useWindowmakerIconBackground = NO;
       [_listener application: self printFile: filePath];
       [self terminate: self];
     }
-  else if ([_delegate respondsToSelector:
-    @selector(applicationShouldOpenUntitledFile:)]
-    && ([_delegate applicationShouldOpenUntitledFile: self] == YES)
-    && [_delegate respondsToSelector: @selector(applicationOpenUntitledFile:)])
+  else if (![defs boolForKey: @"autolaunch"]
+	   && [_delegate respondsToSelector:
+		@selector(applicationShouldOpenUntitledFile:)]
+	   && ([_delegate applicationShouldOpenUntitledFile: self])
+	   && [_delegate respondsToSelector:
+		@selector(applicationOpenUntitledFile:)])
     {
       [_delegate applicationOpenUntitledFile: self];
     }
@@ -1160,6 +1219,11 @@ static BOOL useWindowmakerIconBackground = NO;
   [NSException raise: NSAbortModalException format: @"abortModal"];
 }
 
+/**
+ * Set up modal session for theWindow, and, if it is not visible already,
+ * puts it up on screen, centering it if it is an NSPanel.  It is then
+ * ordered front and made key or main window.
+ */
 - (NSModalSession) beginModalSessionForWindow: (NSWindow*)theWindow
 {
   NSModalSession theSession;
@@ -1173,12 +1237,13 @@ static BOOL useWindowmakerIconBackground = NO;
   _session = theSession;
 
   /*
-   * The NSWindow documentation says runModalForWindow centers panels.
-   * Here would seem the best place to do it.
+   * Displaying / raising window but centering panel only if not up
+   * seems to match the behavior on OS X (Panther).
    */
   if ([theWindow isKindOfClass: [NSPanel class]])
     {
-      [theWindow center];
+      if ([theWindow isVisible] == NO)
+          [theWindow center];
       [theWindow setLevel: NSModalPanelWindowLevel];
     }
   [theWindow orderFrontRegardless];
@@ -1235,6 +1300,12 @@ static BOOL useWindowmakerIconBackground = NO;
   NSZoneFree(NSDefaultMallocZone(), theSession);
 }
 
+/**
+ * Starts modal event loop for given window, after calling
+ * -beginModalSessionForWindow:.  Loop is broken only by stopModal: ,
+ * -stopModalWithCode: , or -abortModal: , at which time -endModalSession:
+ * is called.
+ */
 - (int) runModalForWindow: (NSWindow*)theWindow
 {
   NSModalSession theSession = 0;
@@ -1289,14 +1360,22 @@ static BOOL useWindowmakerIconBackground = NO;
 /** 
 <p>
 Processes any events for a modal session described by the theSession
-variable. Before processing the events, it makes the session window key
-and orders the window front, so there is no need to do this
-separately. When finished, it returns the state of the session (i.e.
+variable. When finished, it returns the state of the session (i.e.
 whether it is still running or has been stopped, etc) 
 </p>
 <p>If there are no pending events for the session, this method returns
 immediately.
 </p>
+<p>
+ Although Apple's docs state that, before processing the events, it makes the
+ session window key and orders the window front, this method does not attempt
+ to do this, because: 1) we don't want to interfere with use of other apps
+ during modal session for this app; 2) occasionally other windows are active
+ and should be usable during modal sessions (e.g., a popup dialog from a modal
+ window); 3) most of the time -beginModalSessionForWindow: will have been
+ called in advance.  If the latter is not the case, you may need to order the
+ window front yourself in advance.
+</p>
 <p>
 See Also: -runModalForWindow:
 </p>
@@ -1315,20 +1394,6 @@ See Also: -runModalForWindow:
 		  format: @"runModalSession: with wrong session"];
     }
 
-  IF_NO_GC(pool = [arpClass new]);
-
-  [theSession->window orderFrontRegardless];
-  if ([theSession->window canBecomeKeyWindow] == YES)
-    {
-      [theSession->window makeKeyWindow];
-    }
-  else if ([theSession->window canBecomeMainWindow] == YES)
-    {
-      [theSession->window makeMainWindow];
-    }
-
-  RELEASE (pool);
-
   // Use the default context for all events.
   srv = GSCurrentServer();
 
@@ -1502,7 +1567,7 @@ See -runModalForWindow:
 - (void) sendEvent: (NSEvent *)theEvent
 {
   NSEventType type;
-  
+
   type = [theEvent type];
   switch (type)
     {
@@ -1591,7 +1656,7 @@ IF_NO_GC(NSAssert([event retainCount] > 0, NSInternalInconsistencyException));
        * If we are not in a tracking loop, we may want to unhide a hidden
        * because the mouse has been moved.
        */
-      if (mode != NSEventTrackingRunLoopMode)
+      if (mode != NSEventTrackingRunLoopMode && event != null_event)
 	{
 	  if ([NSCursor isHiddenUntilMouseMoves])
 	    {
@@ -1825,7 +1890,7 @@ image.
 
   if (_app_icon_window != nil)
     {
-      [[_app_icon_window contentView] setImage: anImage];
+      [(NSAppIconView *)[_app_icon_window contentView] setImage: anImage];
     }
 
   // Swap the old image for the new one wherever it's used
@@ -2221,19 +2286,22 @@ image.
  */ 
 - (void) setMainMenu: (NSMenu*)aMenu
 {
-  if (_main_menu != nil && _main_menu != aMenu)
+  if (_main_menu == aMenu)
     {
-      [_main_menu close];
-      [[_main_menu window] setLevel: NSSubmenuWindowLevel];
+      return;
+    }
+
+  if (_main_menu != nil)
+    {
+      [_main_menu setMain: NO];
     }
 
   ASSIGN(_main_menu, aMenu);
 
-  // Set the title of the window.
-  // This wont be displayed, but the window manager may need it.
-  [[_main_menu window] setTitle: [[NSProcessInfo processInfo] processName]];
-  [[_main_menu window] setLevel: NSMainMenuWindowLevel];
-  [_main_menu setGeometry];
+  if (_main_menu != nil)
+    {
+      [_main_menu setMain: YES];
+    }
 }
 
 - (void) rightMouseDown: (NSEvent*)theEvent
@@ -2412,10 +2480,21 @@ image.
 	break;
       i++;
     }
-  item = [_windows_menu insertItemWithTitle: aString
+
+  if ([aWindow canBecomeKeyWindow])
+    {
+      item = [_windows_menu insertItemWithTitle: aString
+			action: @selector(makeKeyAndOrderFront:)
+			keyEquivalent: @""
+			atIndex: i];
+    }
+  else 
+    {
+      item = [_windows_menu insertItemWithTitle: aString
 			action: @selector(orderFront:)
 			keyEquivalent: @""
 			atIndex: i];
+    }
   [item setTarget: aWindow];
 
   // When changing for a window with a file, we should also set the image.
@@ -2452,6 +2531,9 @@ image.
 	  NSString	*t = [aWindow title];
 	  NSString	*f = [aWindow representedFilename];
 
+	  f = [NSString stringWithFormat: @"%@  --  %@",
+	  				  [f lastPathComponent],
+					  [f stringByDeletingLastPathComponent]];
 	  [self changeWindowsItem: aWindow
 			    title: t
 			 filename: [t isEqual: f]];
@@ -2507,7 +2589,10 @@ image.
 	  {
 	    NSString	*t = [win title];
 	    NSString	*f = [win representedFilename];
-	    
+
+	    f = [NSString stringWithFormat: @"%@  --  %@",
+                                         [f lastPathComponent],
+                                         [f stringByDeletingLastPathComponent]];
 	    [self changeWindowsItem: win
 		  title: t
 		  filename: [t isEqual: f]];
@@ -2812,20 +2897,21 @@ image.
 - _appIconInit
 {
   NSAppIconView	*iv;
+  NSSize iconSize = [GSCurrentServer() iconSize];
+  NSRect iconRect = NSMakeRect(0, 0, iconSize.width, iconSize.height);
 
   if (_app_icon == nil)
     {
       [self setApplicationIconImage: [NSImage imageNamed: @"GNUstep"]];
     }
 
-  _app_icon_window = [[NSIconWindow alloc] initWithContentRect: 
-					    NSMakeRect(0,0,64,64)
+  _app_icon_window = [[NSIconWindow alloc] initWithContentRect: iconRect 
 				styleMask: NSIconWindowMask
 				  backing: NSBackingStoreRetained
 				    defer: NO
 				   screen: nil];
 
-  iv = [[NSAppIconView alloc] initWithFrame: NSMakeRect(0,0,64,64)];
+  iv = [[NSAppIconView alloc] initWithFrame: iconRect]; 
   [iv setImage: _app_icon];
   [_app_icon_window setContentView: iv];
   RELEASE(iv);
@@ -2874,6 +2960,7 @@ image.
   if (_key_window == obj)
     {
       _key_window = nil;
+      [NSCursor setHiddenUntilMouseMoves: NO];
     }
   else
     {
