@@ -133,13 +133,13 @@
 - (void) setType: (NSCellType)aType
 {
   /* We do nothing here (we match the Mac OS X behavior) because with
-   * NSButtonCell GNUstep implementation the cell type is binded to the image
-   * position. Such behavior has been choosen because it permits to have
+   * NSButtonCell GNUstep implementation the cell type is bound to the image
+   * position. We implemented this behavior because it permits to have
    * -setFont: -setTitle -setImage: methods which are symetrical by not altering
-   * the cell type, morevover a cell type is more characterized by the potential
-   * visibility of the image (which is under the control of the method
-   * -setImagePosition:) than by the value of the image ivar itself (related to
-   * -setImage: method).  
+   * directly the cell type and to validate the fact that the cell type is more
+   * characterized by the potential visibility of the image (which is under the
+   * control of the method -setImagePosition:) than by the value of the image
+   * ivar itself (related to -setImage: method).  
    * On Mac OS X, the NSButtonCell cell type is NSTextCellType by default or
    * NSImageCellType if the initialization has been done with -initImageCell:,
    * it should be noted that the cell type never changes later.
@@ -363,6 +363,22 @@
   return _cell.image_position;
 }
 
+- (void) setImage: (NSImage *)anImage
+{
+  if (anImage) 
+    {
+      NSAssert ([anImage isKindOfClass: [NSImage class]],
+		NSInvalidArgumentException);
+    }
+  
+  if (_cell.image_position == NSNoImage)
+    {
+      [self setImagePosition: NSImageOnly];
+    }
+  
+  ASSIGN (_cell_image, anImage);
+}
+
 - (void) setAlternateImage: (NSImage*)anImage
 {
   ASSIGN(_altImage, anImage);
@@ -371,6 +387,18 @@
 - (void) setImagePosition: (NSCellImagePosition)aPosition
 {
   _cell.image_position = aPosition;
+   
+  // In the GNUstep NSButtonCell implementation, the cell type depends only on
+  // the image position. 
+    
+  if (_cell.image_position == NSNoImage)
+    {
+      [super setType: NSTextCellType];
+    }
+  else
+    {
+      [super setType: NSImageCellType];
+    }
 }
 
 /*
@@ -1220,7 +1248,7 @@
     {
       [_sound play];
     }
-    
+
   [super performClickWithFrame: cellFrame inView: controlView];
 }
 
@@ -1284,30 +1312,114 @@
 
   if ([aDecoder allowsKeyedCoding])
     {
-      NSString *alternateContents = [aDecoder decodeObjectForKey: @"NSAlternateContents"];
-      NSImage *alternateImage = [aDecoder decodeObjectForKey: @"NSAlternateImage"];
       //NSControl *control = [aDecoder decodeObjectForKey: @"NSControlView"];
-      NSString *key = [aDecoder decodeObjectForKey: @"NSKeyEquivalent"];
-      int bFlags;
-      int bFlags2;
       int delay = 0;
       int interval = 0;
       
-      [self setAlternateImage: alternateImage];
-      [self setAlternateTitle: alternateContents];
-      [self setKeyEquivalent: key];
-
+      if ([aDecoder containsValueForKey: @"NSKeyEquivalent"])
+        {
+	  [self setKeyEquivalent: [aDecoder decodeObjectForKey: @"NSKeyEquivalent"]];
+	}
+      if ([aDecoder containsValueForKey: @"NSNormalImage"])
+        {
+	  [self setImage: [aDecoder decodeObjectForKey: @"NSNormalImage"]];
+	}
+      if ([aDecoder containsValueForKey: @"NSAlternateImage"])
+        {
+	  id image;
+	  
+	  image = [aDecoder decodeObjectForKey: @"NSAlternateImage"];
+	  // This test works around an Apple bug, where a font gets encoded here. 
+	  if ([image isKindOfClass: [NSImage class]])
+	    {
+	      [self setAlternateImage: image];
+	    }
+	}
+      if ([aDecoder containsValueForKey: @"NSAlternateContents"])
+        {
+	  [self setAlternateTitle: [aDecoder decodeObjectForKey: @"NSAlternateContents"]];
+	}
       if ([aDecoder containsValueForKey: @"NSButtonFlags"])
         {
+	  int bFlags;
+	  int highlights = 0;  
+	  int show_state = NSNoCellMask;  
+
 	  bFlags = [aDecoder decodeIntForKey: @"NSButtonFlags"];
+
+	  [self setTransparent: (bFlags & 0x8000)];
 	  // FIXME
+	  if ((bFlags & 0x800000) == 0x800000)
+	    {
+	      [self setBordered: YES];
+	    }
+	  if ((bFlags & 0x6000000) == 0x6000000)
+	    {
+	      highlights |= NSChangeBackgroundCellMask;
+	    }
+	  if ((bFlags & 0x8000000) == 0x8000000)
+	    {
+	      highlights |= NSContentsCellMask;
+	    }
+	  if ((bFlags & 0x30000000) == 0x30000000)
+	    {
+	      show_state |= NSChangeBackgroundCellMask;
+	    }
+	  if ((bFlags & 0x40000000) == 0x40000000)
+	    {
+	      show_state |= NSContentsCellMask;
+	    }
+	  if ((bFlags & 0x80000000) == 0x80000000)
+	    {
+	      highlights |= NSPushInCellMask;
+	    }
+	  [self setHighlightsBy: highlights];
+	  [self setShowsStateBy: show_state];
+	  [self setImagePosition: NSImageLeft];
 	}
       if ([aDecoder containsValueForKey: @"NSButtonFlags2"])
         {
-	  bFlags2 = [aDecoder decodeIntForKey: @"NSButtonFlags2"];
-	  // FIXME
-	}
+	  int bFlags2;
 
+	  bFlags2 = [aDecoder decodeIntForKey: @"NSButtonFlags2"];
+	  [self setShowsBorderOnlyWhileMouseInside: (bFlags2 & 0x8)];
+
+	  // FIXME
+	  switch (bFlags2 & 0x7)
+	    {
+	      case 1:
+		[self setBezelStyle: NSRoundedBezelStyle];
+		break;
+	      case 2:
+		[self setBezelStyle: NSRegularSquareBezelStyle];
+		break;
+	      case 3:
+		[self setBezelStyle: NSThickSquareBezelStyle];
+		break;
+	      case 4:
+		[self setBezelStyle: NSThickerSquareBezelStyle];
+		break;
+	      case 5:
+		//[self setBezelStyle: NSDisclosureBezelStyle];
+		break;
+	      case 6:
+		[self setBezelStyle: NSShadowlessSquareBezelStyle];
+		break;
+	      case 7:
+		[self setBezelStyle: NSCircularBezelStyle];
+		break;
+/*
+	      case 32:
+		[self setBezelStyle: NSTexturedSquareBezelStyle];
+		break;
+	      case 33:
+		[self setBezelStyle: NSHelpButtonBezelStyle];
+		break;
+*/
+	      default:
+		break;
+	    }
+	}
       if ([aDecoder containsValueForKey: @"NSPeriodicDelay"])
         {
 	  delay = [aDecoder decodeIntForKey: @"NSPeriodicDelay"];
