@@ -567,6 +567,9 @@ static NSColor	*shadowCol;
 
 - (NSCellType) type
 {
+  if (_cell.type == NSImageCellType && _cell_image == nil)
+    return NSNullCellType;
+    
   return _cell.type;
 }
 
@@ -867,6 +870,7 @@ static NSColor	*shadowCol;
 
 - (NSText*) setUpFieldEditorAttributes: (NSText*)textObject
 {
+  [textObject setString: @""];
   [textObject setTextColor: [self textColor]];
   if (_cell.contents_is_attributed_string == NO)
     {
@@ -885,6 +889,7 @@ static NSColor	*shadowCol;
   [textObject setRichText: _cell.is_rich_text];
   [textObject setImportsGraphics: _cell.imports_graphics];
   [textObject setSelectedRange: NSMakeRange(0, 0)];
+  [textObject scrollRangeToVisible: NSMakeRange(0, 0)];
 
   return textObject;
 }
@@ -1551,6 +1556,7 @@ static NSColor	*shadowCol;
 	  }
 	break;
 
+      default:
       case NSNullCellType:
 	//  macosx instead returns a 'very big size' here; we return NSZeroSize
 	s = NSZeroSize;
@@ -1780,7 +1786,8 @@ static NSColor	*shadowCol;
 		      editor: (NSText*)textObject
 {
   NSRect titleRect = [self titleRectForBounds: aRect];
-  NSSize maxSize = NSMakeSize(1e8, titleRect.size.height);
+  /* See comments in NSStringDrawing.m about the choice of maximum size. */
+  NSSize maxSize = NSMakeSize(1e6, titleRect.size.height);
   NSClipView *cv = [[NSClipView alloc] 
 		       initWithFrame: titleRect];
   NSTextContainer *ct = [(NSTextView*)textObject textContainer];
@@ -1789,8 +1796,8 @@ static NSColor	*shadowCol;
   RELEASE(cv);
   [cv setAutoresizesSubviews: NO];
   [cv setDocumentView: textObject];
-  [textObject setFrame: [cv bounds]];
-  [textObject setHorizontallyResizable: YES];
+  [textObject setFrame: NSMakeRect(0, 0, maxSize.width, maxSize.height)];
+  [textObject setHorizontallyResizable: NO];
   [textObject setVerticallyResizable: NO];
   [textObject setMaxSize: maxSize];
   [textObject setMinSize: titleRect.size];
@@ -1899,6 +1906,7 @@ static NSColor	*shadowCol;
 
   [textObject sizeToFit];
   [textObject setSelectedRange: NSMakeRange (selStart, selLength)];
+  [textObject scrollRangeToVisible: NSMakeRange(selStart, selLength)];
   [textObject setDelegate: anObject];
   [[controlView window] makeFirstResponder: textObject];
 }
@@ -2006,7 +2014,8 @@ static NSColor	*shadowCol;
   if ([aDecoder allowsKeyedCoding])
     {
       NSString *contents = [aDecoder decodeObjectForKey: @"NSContents"];
-      NSFont *support = [aDecoder decodeObjectForKey: @"NSSupport"];
+
+      self = [self initTextCell: contents];
 
       if ([aDecoder containsValueForKey: @"NSCellFlags"])
         {
@@ -2022,6 +2031,10 @@ static NSColor	*shadowCol;
 	  [self setSelectable: ((cFlags & 0x200001) == 0x200001)];
 	  [self setBezeled: ((cFlags & 0x400000) == 0x400000)];
 	  [self setBordered: ((cFlags & 0x800000) == 0x800000)];
+	  if ((cFlags & 0x4000000) == 0x4000000)
+	    {
+	      [self setType: NSTextCellType];
+	    }
 	  [self setEditable: ((cFlags & 0x10000000) == 0x10000000)];
 	  // This bit flag is the other way around!
 	  [self setEnabled: ((cFlags & 0x20000000) != 0x20000000)];
@@ -2034,21 +2047,49 @@ static NSColor	*shadowCol;
 	  int cFlags2;
       
 	  cFlags2 = [aDecoder decodeIntForKey: @"NSCellFlags2"];
+	  [self setSendsActionOnEndEditing: (cFlags2 & 0x400000)];
 	  [self setAllowsMixedState: ((cFlags2 & 0x1000000) == 0x1000000)];
 	  [self setRefusesFirstResponder: ((cFlags2 & 0x2000000) == 0x2000000)];
+	  if ((cFlags2 & 0x4000000) == 0x4000000)
+	    {
+	      [self setAlignment: NSRightTextAlignment];
+	    }
+	  if ((cFlags2 & 0x8000000) == 0x8000000)
+	    {
+	      [self setAlignment: NSCenterTextAlignment];
+	    }
+	  if ((cFlags2 & 0xC000000) == 0xC000000)
+	    {
+	      [self setAlignment: NSJustifiedTextAlignment];
+	    }
+	  if ((cFlags2 & 0x10000000) == 0x10000000)
+	    {
+	      [self setAlignment: NSNaturalTextAlignment];
+	    }
+
 	  [self setImportsGraphics: ((cFlags2 & 0x20000000) == 0x20000000)];
 	  [self setAllowsEditingTextAttributes: ((cFlags2 & 0x40000000) == 0x40000000)];
 	  // FIXME
 	}
+      if ([aDecoder containsValueForKey: @"NSSupport"])
+        {
+	  NSFont *support = [aDecoder decodeObjectForKey: @"NSSupport"];
 
-      self = [self initTextCell: contents];
-      [self setFont: support];
+	  [self setFont: support];
+	}
+      if ([aDecoder containsValueForKey: @"NSFormatter"])
+        {
+	  NSFormatter *formatter = [aDecoder decodeObjectForKey: @"NSFormatter"];
+
+	  [self setFormatter: formatter];
+	}
     }
   else
     {
       BOOL flag;
       unsigned int tmp_int;
-      
+      id formatter, menu;
+
       [aDecoder decodeValueOfObjCType: @encode(id) at: &_contents];
       [aDecoder decodeValueOfObjCType: @encode(id) at: &_cell_image];
       [aDecoder decodeValueOfObjCType: @encode(id) at: &_font];
@@ -2101,10 +2142,12 @@ static NSColor	*shadowCol;
       [aDecoder decodeValueOfObjCType: @encode(unsigned int) 
 		                   at: &_mouse_down_flags];
       [aDecoder decodeValueOfObjCType: @encode(unsigned int) at: &_action_mask];
-      [aDecoder decodeValueOfObjCType: @encode(id) at: &_formatter];
-      [aDecoder decodeValueOfObjCType: @encode(id) at: &_menu];
+      [aDecoder decodeValueOfObjCType: @encode(id) at: &formatter];
+      [self setFormatter: formatter];
+      [aDecoder decodeValueOfObjCType: @encode(id) at: &menu];
+      [self setMenu: menu];
       [aDecoder decodeValueOfObjCType: @encode(id) at: &_represented_object];
-      
+
       if (_formatter != nil)
         {
 	  NSString *contents;
