@@ -24,17 +24,20 @@
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
 */
 
-#include <gnustep/gui/config.h>
+#include "config.h"
 #include <Foundation/NSNotification.h>
 
-#include <AppKit/NSClipView.h>
-#include <AppKit/NSCursor.h>
-#include <AppKit/NSColor.h>
-#include <AppKit/NSWindow.h>
-#include <AppKit/NSGraphics.h>
-#include <AppKit/PSOperators.h>
+#include "AppKit/NSClipView.h"
+#include "AppKit/NSCursor.h"
+#include "AppKit/NSColor.h"
+#include "AppKit/NSGraphics.h"
+#include "AppKit/NSTableView.h"
+#include "AppKit/NSWindow.h"
+#include "AppKit/PSOperators.h"
 
-@class NSTableView;
+#include <math.h>
+
+DEFINE_RINT_IF_MISSING
 
 /*
  * Return the biggest integral (in device space) rect contained in rect. 
@@ -201,18 +204,18 @@ static inline NSRect integralRect (NSRect rect, NSView *view)
   [self addCursorRect: _bounds cursor: _cursor];
 }
 
-- (void) scrollToPoint: (NSPoint)point
+- (void) scrollToPoint: (NSPoint)aPoint
 {
-  [self setBoundsOrigin: [self constrainScrollPoint: point]];
+  [self setBoundsOrigin: [self constrainScrollPoint: aPoint]];
 }
 
-- (void) setBoundsOrigin: (NSPoint)point
+- (void) setBoundsOrigin: (NSPoint)aPoint
 {
   NSRect originalBounds = _bounds;
   NSRect newBounds = originalBounds;
   NSRect intersection;
 
-  newBounds.origin = point;
+  newBounds.origin = aPoint;
 
   if (NSEqualPoints (originalBounds.origin, newBounds.origin))
     {
@@ -230,6 +233,8 @@ static inline NSRect integralRect (NSRect rect, NSView *view)
 	 scrolling.  Then, document view needs to redraw the remaining
 	 areas. */
 
+	
+
       /* Common part - which is a first approx of what we could
          copy... */
       intersection = NSIntersectionRect (originalBounds, newBounds);
@@ -350,8 +355,8 @@ static inline NSRect integralRect (NSRect rect, NSView *view)
   /* ?? TODO: Understand the following code - and add explanatory comment */
   if ([NSView focusView] == _documentView)
     {
-      PStranslate (NSMinX (originalBounds) - point.x, 
-		   NSMinY (originalBounds) - point.y);
+      PStranslate (NSMinX (originalBounds) - aPoint.x, 
+		   NSMinY (originalBounds) - aPoint.y);
     }
   
   [_super_view reflectScrolledClipView: self];
@@ -399,10 +404,29 @@ static inline NSRect integralRect (NSRect rect, NSView *view)
      do the scrolling, the difference is an integer and so we can copy
      the image translating it by an integer in device space - and not
      by a float. */
+  /*
   new = [self convertPoint: new  toView: nil];
   new.x = (int)new.x;
   new.y = (int)new.y;
   new = [self convertPoint: new  fromView: nil];
+  */
+
+  /*
+     We don't make it an integer this way anymore.
+     This is not needed when _copiesOnScroll is not set.
+     If _copiesOnScroll is set, we make sure the difference between old
+     position and new position is an integer so we can copy the image 
+     easily. 
+  */
+  if (_copiesOnScroll)
+    {
+      new.x =
+	_bounds.origin.x + 
+	(rint(new.x - _bounds.origin.x));
+      new.y =
+	_bounds.origin.y +
+	(rint(new.y - _bounds.origin.y));
+    }
 
   return new;
 }
@@ -457,6 +481,8 @@ static inline NSRect integralRect (NSRect rect, NSView *view)
 - (BOOL) autoscroll: (NSEvent*)theEvent
 {
   NSPoint new;
+  NSPoint delta;
+  NSRect r;
 
   if (_documentView == nil)
     {
@@ -465,12 +491,29 @@ static inline NSRect integralRect (NSRect rect, NSView *view)
   
   new = [_documentView convertPoint: [theEvent locationInWindow] 
 		       fromView: nil];
-  new = [self constrainScrollPoint: new];
 
-  if (NSPointInRect(new, [self documentVisibleRect]))
-  {
+  r = [self documentVisibleRect];
+
+  if (new.x < NSMinX(r))
+    delta.x = new.x - NSMinX(r);
+  else if (new.x > NSMaxX(r))
+    delta.x = new.x - NSMaxX(r);
+  else
+    delta.x = 0;
+
+  if (new.y < NSMinY(r))
+    delta.y = new.y - NSMinY(r);
+  else if (new.y > NSMaxY(r))
+    delta.y = new.y - NSMaxY(r);
+  else
+    delta.y = 0;
+
+  new.x = _bounds.origin.x + delta.x;
+  new.y = _bounds.origin.y + delta.y;
+
+  new = [self constrainScrollPoint: new];
+  if (NSEqualPoints(new, _bounds.origin))
     return NO;
-  }
 
   [self setBoundsOrigin: new];
   return YES;
@@ -675,23 +718,42 @@ static inline NSRect integralRect (NSRect rect, NSView *view)
 
 - (id) initWithCoder: (NSCoder*)aDecoder
 {
-  NSView *document;
-
   self = [super initWithCoder: aDecoder];
-  [self setAutoresizesSubviews: YES];
 
-  [aDecoder decodeValueOfObjCType: @encode(id) at: &_backgroundColor];
-  [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_copiesOnScroll];
-  [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_drawsBackground];
-  [aDecoder decodeValueOfObjCType: @encode(id) at: &_cursor];
-
-  if ([[self subviews] count] > 0)
+  if ([aDecoder allowsKeyedCoding])
     {
-      document = AUTORELEASE(RETAIN([[self subviews] objectAtIndex: 0]));
-      [self removeSubview: document];
-      [self setDocumentView: document];
-    }
+      int flags;
+      
+      [self setBackgroundColor: [aDecoder decodeObjectForKey: @"NSBGColor"]];
+      [self setDocumentCursor: [aDecoder decodeObjectForKey: @"NSCursor"]];
+      [self setDocumentView: [aDecoder decodeObjectForKey: @"NSDocView"]];
 
+      if ([aDecoder containsValueForKey: @"NScvFlags"])
+        {
+	  flags = [aDecoder decodeIntForKey: @"NScvFlags"];
+	  // FIXME setCopiesOnScroll: setDrawsBackground: 
+	}
+    }
+  else
+    {
+      NSView *document;
+      BOOL temp;
+      
+      [self setAutoresizesSubviews: YES];
+      
+      [self setBackgroundColor: [aDecoder decodeObject]];
+      [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_copiesOnScroll];
+      [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &temp];
+      [self setDrawsBackground: temp];
+      [aDecoder decodeValueOfObjCType: @encode(id) at: &_cursor];
+      
+      if ([[self subviews] count] > 0)
+        {
+	  document = AUTORELEASE(RETAIN([[self subviews] objectAtIndex: 0]));
+	  [self removeSubview: document];
+	  [self setDocumentView: document];
+	}
+    }
   return self;
 }
 @end
