@@ -186,7 +186,7 @@ void initSystemColors(void)
   
   colorStrings = [[NSMutableDictionary alloc]
 		     initWithObjectsAndKeys:
-			 lightGray, @"controlBackgroundColor",
+		     lightGray, @"controlBackgroundColor",
 		     lightGray, @"controlColor",
 		     lightGray, @"controlHighlightColor",
 		     white, @"controlLightHighlightColor",
@@ -214,6 +214,11 @@ void initSystemColors(void)
 		     lightGray, @"windowBackgroundColor",
 		     black, @"windowFrameColor",
 		     white, @"windowFrameTextColor",
+		     black, @"alternateSelectedControlColor",
+		     white, @"alternateSelectedControlTextColor",
+		     white, @"rowBackgroundColor",
+		     lightGray, @"alternateRowBackgroundColor",
+		     lightGray, @"secondarySelectedControlColor",
 		     //gray, @"windowFrameColor",
 		     //black, @"windowFrameTextColor",
 		     nil];
@@ -573,6 +578,16 @@ systemColorWithName(NSString *name)
 //
 // System colors stuff.
 //
++ (NSColor*) alternateSelectedControlColor
+{
+  return systemColorWithName(@"alternateSelectedControlColor");
+}
+
++ (NSColor*) alternateSelectedControlTextColor
+{
+  return systemColorWithName(@"alternateSelectedControlTextColor");
+}
+
 + (NSColor*) controlBackgroundColor
 {
   return systemColorWithName(@"controlBackgroundColor");
@@ -648,6 +663,11 @@ systemColorWithName(NSString *name)
   return systemColorWithName(@"scrollBarColor");
 }
 
++ (NSColor*) secondarySelectedControlColor
+{
+  return systemColorWithName(@"secondarySelectedControlColor");
+}
+
 + (NSColor*) selectedControlColor
 {
   return systemColorWithName(@"selectedControlColor");
@@ -713,6 +733,12 @@ systemColorWithName(NSString *name)
   return systemColorWithName(@"windowFrameTextColor");
 }
 
++ (NSArray*) controlAlternatingRowBackgroundColors
+{
+  return [NSArray arrayWithObjects: systemColorWithName(@"rowBackgroundColor"), 
+		  systemColorWithName(@"alternateRowBackgroundColor"), nil];
+}
+
 ////////////////////////////////////////////////////////////
 //
 // Instance methods
@@ -1056,7 +1082,7 @@ systemColorWithName(NSString *name)
 	  float red = 0.0;
 	  float green = 0.0;
 	  float blue = 0.0;
-	  float alpha = 0.0;
+	  float alpha = 1.0;
 	  NSString *str;
 	  NSScanner *scanner;
 	  
@@ -1069,8 +1095,10 @@ systemColorWithName(NSString *name)
 	      [scanner scanFloat: &red];
 	      [scanner scanFloat: &green];
 	      [scanner scanFloat: &blue];
+	      RELEASE(scanner);
+	      RELEASE(str);
 	    }
-	  
+
 	  self = [NSColor colorWithCalibratedRed: red
 			  green: green
 			  blue: blue
@@ -1081,7 +1109,7 @@ systemColorWithName(NSString *name)
 	  unsigned length;
 	  const uint8_t *data;
 	  float white = 0.0;
-	  float alpha = 0.0;
+	  float alpha = 1.0;
 	  NSString *str;
 	  NSScanner *scanner;
 	  
@@ -1092,6 +1120,8 @@ systemColorWithName(NSString *name)
 	      str = [[NSString alloc] initWithCString: data length: length];
 	      scanner = [[NSScanner alloc] initWithString: str];
 	      [scanner scanFloat: &white];
+	      RELEASE(scanner);
+	      RELEASE(str);
 	    }
 	  
 	  self = [NSColor colorWithDeviceWhite: white
