@@ -25,22 +25,23 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */ 
 
-#include <gnustep/gui/config.h>
+#include "config.h"
 #include <Foundation/NSBundle.h>
 #include <Foundation/NSEnumerator.h>
 #include <Foundation/NSFileManager.h>
 #include <Foundation/NSLock.h>
+#include <Foundation/NSNotification.h>
 #include <Foundation/NSPathUtilities.h>
-#include <AppKit/NSButtonCell.h>
-#include <AppKit/NSColor.h>
-#include <AppKit/NSColorPanel.h>
-#include <AppKit/NSColorPicker.h>
-#include <AppKit/NSColorPicking.h>
-#include <AppKit/NSColorWell.h>
-#include <AppKit/NSPasteboard.h>
-#include <AppKit/NSWindow.h>
-#include <AppKit/NSApplication.h>
-#include <AppKit/IMLoading.h>
+#include "AppKit/NSButtonCell.h"
+#include "AppKit/NSColor.h"
+#include "AppKit/NSColorPanel.h"
+#include "AppKit/NSColorPicker.h"
+#include "AppKit/NSColorPicking.h"
+#include "AppKit/NSColorWell.h"
+#include "AppKit/NSImage.h"
+#include "AppKit/NSPasteboard.h"
+#include "AppKit/NSWindow.h"
+#include "GNUstepGUI/IMLoading.h"
 
 #define MAX_ALPHA_VALUE 100.0
 static NSLock *_gs_gui_color_panel_lock = nil;
@@ -49,6 +50,21 @@ static int _gs_gui_color_picker_mask = NSColorPanelAllModesMask;
 // FIXME: This should be NSWheelModeColorPanel 
 static int _gs_gui_color_picker_mode = NSRGBModeColorPanel;
 
+
+@implementation NSApplication (NSColorPanel)
+
+- (void) orderFrontColorPanel: sender
+{ 
+  NSColorPanel *colorPanel = [NSColorPanel sharedColorPanel];
+
+  if (colorPanel)
+    [colorPanel orderFront: nil];
+  else
+    NSBeep();
+}
+
+@end
+
 @interface NSColorPanel (PrivateMethods)
 - (void) _loadPickers;
 - (void) _loadPickerAtPath: (NSString *)path;
@@ -152,6 +168,7 @@ static int _gs_gui_color_picker_mode = NSRGBModeColorPanel;
     }
 
   [_pickerMatrix addRowWithCells: cells];
+  RELEASE(cells);
   [_pickerMatrix setCellSize: NSMakeSize(size.width / count, size.height)];
   [_pickerMatrix setTarget: self];
   [_pickerMatrix setAction: @selector(_showNewPicker:)];
@@ -308,7 +325,7 @@ static int _gs_gui_color_picker_mode = NSRGBModeColorPanel;
     [well setBordered: NO];
     [well setEnabled: NO];
     [well setTarget: _colorWell];
-    [well setAction: @selector(takeColorFrom:)];
+    [well setAction: @selector(_bottomWellAction:)];
     [swatchView addSubview: well];
     RELEASE(well);
   }
@@ -339,6 +356,11 @@ static int _gs_gui_color_picker_mode = NSRGBModeColorPanel;
   [_currentPicker setColor: [_colorWell color]];
 }
 
+- (void) _bottomWellAction: (id) sender
+{
+  [self setColor: [sender color]];
+}
+
 @end
 
 @implementation NSColorPanel
