@@ -71,6 +71,7 @@
       _documentIndex = untitledCount++;
       _windowControllers = [[NSMutableArray alloc] init];
       fileTypes = [[self class] readableTypes];
+      _docFlags.hasUndoManager = YES;
 
       /* Set our default type */
       if ([fileTypes count])
@@ -578,12 +579,56 @@
 
 - (BOOL)validateMenuItem: (NSMenuItem *)anItem
 {
-  if ([anItem action] == @selector(revertDocumentToSaved:))
-    return ([self fileName] != nil && [self isDocumentEdited]);
+  BOOL result = YES;
+  SEL  action = [anItem action];
 
   // FIXME should validate spa popup items; return YES if it's a native type.
+  if (sel_eq(action, @selector(revertDocumentToSaved:)))
+    {
+      result = ([self fileName] != nil && [self isDocumentEdited]);
+    }
+  else if (sel_eq(action, @selector(undo:)))
+    {
+      if(_undoManager == nil)
+	{
+	  result = NO;
+	}
+      else
+	{
+	  if([_undoManager canUndo])
+	    {
+	      [anItem setTitle: [_undoManager undoMenuItemTitle]];
+	      result = YES;
+	    }
+	  else
+	    {
+	      [anItem setTitle: [_undoManager undoMenuTitleForUndoActionName: @""]];
+	      result = NO;
+	    }
+	}
+    }
+  else if (sel_eq(action, @selector(redo:)))
+    {
+      if(_undoManager == nil)
+	{
+	  result = NO;
+	}
+      else
+	{
+	  if([_undoManager canRedo])
+	    {
+	      [anItem setTitle: [_undoManager redoMenuItemTitle]];
+	      result = YES;
+	    }
+	  else
+	    {
+	      [anItem setTitle: [_undoManager redoMenuTitleForUndoActionName: @""]];
+	      result = NO;
+	    }
+	}
+    }
     
-  return YES;
+  return result;
 }
 
 - (BOOL)validateUserInterfaceItem: (id <NSValidatedUserInterfaceItem>)anItem
@@ -929,4 +974,14 @@
   [self updateChangeCount: NSChangeDone];
 }
 
+- (void) undo: (id)sender
+{
+  [[self undoManager] undo];
+}
+
+- (void) redo: (id)sender
+{
+  [[self undoManager] redo];
+}
+
 @end
