@@ -27,9 +27,9 @@
 
 #include <Foundation/NSCoder.h>
 #include <Foundation/NSNotification.h>
-#include <AppKit/NSWindow.h>
-#include <AppKit/NSView.h>
-#include <AppKit/NSDrawer.h>
+#include "AppKit/NSWindow.h"
+#include "AppKit/NSView.h"
+#include "AppKit/NSDrawer.h"
 
 static NSNotificationCenter *nc = nil;
 
@@ -72,6 +72,12 @@ static NSNotificationCenter *nc = nil;
 {
   RELEASE(_contentView);
 
+  if (_delegate != nil)
+    {
+      [nc removeObserver: _delegate name: nil object: self];
+      _delegate = nil;
+    }
+
   [super dealloc];
 }
 
