@@ -30,7 +30,7 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#include <gnustep/gui/config.h>
+#include "config.h"
 #include <Foundation/NSDictionary.h>
 #include <Foundation/NSLock.h>
 #include <Foundation/NSTimer.h>
@@ -40,12 +40,14 @@
 #include <Foundation/NSException.h>
 #include <Foundation/NSDebug.h>
 
-#include <AppKit/NSEvent.h>
-#include <AppKit/NSApplication.h>
-#include <AppKit/NSWindow.h>
-#include <AppKit/NSGraphicsContext.h>
-#include <AppKit/NSGraphics.h>
-#include <AppKit/PSOperators.h>
+#include "AppKit/NSEvent.h"
+#include "AppKit/NSApplication.h"
+#include "AppKit/NSWindow.h"
+#include "AppKit/NSGraphicsContext.h"
+#include "AppKit/NSGraphics.h"
+#include "AppKit/PSOperators.h"
+
+#include "GNUstepGUI/GSDisplayServer.h"
 
 /*
  *	gstep-base has a faster mechanism to get the current thread.
@@ -70,7 +72,6 @@ static Class eventClass;
 {
   if (self == [NSEvent class])
     {
-      NSDebugLog(@"Initialize NSEvent class\n");
       [self setVersion: 1];
       eventClass = [NSEvent class];
     }
@@ -236,10 +237,7 @@ static Class eventClass;
  */
 + (NSPoint) mouseLocation
 {
-  float x, y;
-
-  PSmouselocation(&x, &y);
-  return NSMakePoint(x, y);
+  return [GSCurrentServer() mouselocation];
 }
 
 + (NSEvent*) otherEventWithType: (NSEventType)type
@@ -285,7 +283,7 @@ static Class eventClass;
   NSTimer		*timer;
   NSMutableDictionary	*dict = GSCurrentThreadDictionary();
 
-  NSDebugLog (@"startPeriodicEventsAfterDelay: withPeriod: ");
+  NSDebugLLog (@"NSEvent", @"startPeriodicEventsAfterDelay: withPeriod: ");
 
   if ([dict objectForKey: timerKey])
     [NSException raise: NSInternalInconsistencyException
@@ -324,7 +322,7 @@ static Class eventClass;
 				     data1: 0
 				     data2: 0];
 
-  NSDebugLog (@"_timerFired: ");
+  NSDebugLLog (@"NSEvent", @"_timerFired: ");
   [NSApp postEvent: periodicEvent atStart: NO];
 }
 
@@ -336,7 +334,7 @@ static Class eventClass;
   NSTimer		*realTimer;
   NSMutableDictionary	*dict = GSCurrentThreadDictionary();
 
-  NSDebugLog (@"_registerRealTimer: ");
+  NSDebugLLog (@"NSEvent", @"_registerRealTimer: ");
   {
     NSTimeInterval	timeInterval;
     NSEvent		*periodicEvent;
@@ -370,7 +368,7 @@ static Class eventClass;
   NSTimer		*timer;
   NSMutableDictionary	*dict = GSCurrentThreadDictionary();
 
-  NSDebugLog (@"stopPeriodicEvents");
+  NSDebugLLog (@"NSEvent", @"stopPeriodicEvents");
   timer = [dict objectForKey: timerKey];
   [timer invalidate];
   [dict removeObjectForKey: timerKey];
@@ -582,7 +580,7 @@ static Class eventClass;
 
 - (NSString*) description
 {
-  const char* eventTypes[] = {
+static const char *eventTypes[] = {
     "leftMouseDown",
     "leftMouseUp",
     "otherMouseDown",
@@ -674,7 +672,7 @@ static Class eventClass;
 	return [NSString stringWithFormat:
 	  @"NSEvent: eventType = %s, point = { %f, %f }, modifiers = %u,"
 	  @" time = %f, window = %d, dpsContext = %p,"
-	  @" event number = %d, click = %d, pressure = %f",
+	  @" event number = %d, click = %d, pressure = %f"
 	  @" button = %d, deltaX = %f, deltaY = %f, deltaZ = %f",
 	  eventTypes[event_type], location_point.x, location_point.y,
 	  modifier_flags, event_time, window_num, event_context,
