@@ -148,6 +148,7 @@ keyForFont(NSString *name, const float *matrix, BOOL fix,
     <item>NSFont                    Helvetica (System Font)</item>
     <item>NSLabelFont               System font</item>
     <item>NSMenuFont                System font</item>
+    <item>NSMenuBarFont             System font</item>
     <item>NSMessageFont             System font</item>
     <item>NSPaletteFont             System bold font</item>
     <item>NSTitleBarFont            System bold font</item>
@@ -164,6 +165,7 @@ keyForFont(NSString *name, const float *matrix, BOOL fix,
     <item>NSFontSize                12 (System Font Size)</item>
     <item>NSLabelFontSize           (none)</item>
     <item>NSMenuFontSize            (none)</item>
+    <item>NSMiniFontSize            6</item>
     <item>NSMessageFontSize         (none)</item>
     <item>NSPaletteFontSize         (none)</item>
     <item>NSSmallFontSize           9</item>
@@ -220,6 +222,7 @@ RolePaletteFont,
 RoleToolTipsFont,
 RoleControlContentFont,
 RoleLabelFont,
+RoleMenuBarFont,
 RoleMax
 };
 
@@ -256,7 +259,8 @@ static font_role_info_t font_roles[RoleMax]={
   {@"NSPaletteFont"       , RoleBoldSystemFont, nil, nil},
   {@"NSToolTipsFont"      , RoleSystemFont    , nil, nil},
   {@"NSControlContentFont", RoleSystemFont    , nil, nil},
-  {@"NSLabelFont"         , RoleSystemFont    , nil, nil}
+  {@"NSLabelFont"         , RoleSystemFont    , nil, nil},
+  {@"NSMenuBarFont"       , RoleSystemFont    , nil, nil}
 };
 
 
@@ -329,7 +333,7 @@ static NSFont *getNSFont(float fontSize, int role)
 
   font_role = role * 2;
 
-  defaultSize = (fontSize == 0.0);
+  defaultSize = (fontSize <= 0.0);
   if (defaultSize)
     {
       if (font_roles[role].cachedFont)
@@ -451,7 +455,8 @@ static void setNSFont(NSString *key, NSFont *font)
 
 /** 
  * Return the default bold font for use in menus and heading in standard
- * gui components.<br />
+ * gui components.  If fontSize is <= 0, the default
+ * size is used.<br />
  * This is deprecated in MacOSX
  */
 + (NSFont*) boldSystemFontOfSize: (float)fontSize
@@ -461,7 +466,8 @@ static void setNSFont(NSString *key, NSFont *font)
 
 /** 
  * Return the default font for use in menus and heading in standard
- * gui components.<br />
+ * gui components.  If fontSize is <= 0, the default
+ * size is used.<br />
  * This is deprecated in MacOSX
  */
 + (NSFont*) systemFontOfSize: (float)fontSize
@@ -471,7 +477,8 @@ static void setNSFont(NSString *key, NSFont *font)
 
 /** 
  * Return the default fixed pitch font for use in locations other
- * than standard gui components.
+ * than standard gui components.  If fontSize is <= 0, the default
+ * size is used.
  */
 + (NSFont*) userFixedPitchFontOfSize: (float)fontSize
 {
@@ -480,7 +487,8 @@ static void setNSFont(NSString *key, NSFont *font)
 
 /** 
  * Return the default font for use in locations other
- * than standard gui components.
+ * than standard gui components.  If fontSize is <= 0, the default
+ * size is used.
  */
 + (NSFont*) userFontOfSize: (float)fontSize
 {
@@ -529,6 +537,11 @@ static void setNSFont(NSString *key, NSFont *font)
   return getNSFont(fontSize, RoleMenuFont);
 }
 
++ (NSFont*) menuBarFontOfSize: (float)fontSize
+{
+  return getNSFont(fontSize, RoleMenuBarFont);
+}
+
 + (NSFont*) titleBarFontOfSize: (float)fontSize
 {
   return getNSFont(fontSize, RoleTitleBarFont);
@@ -588,6 +601,29 @@ static void setNSFont(NSString *key, NSFont *font)
   return fontSize;
 }
 
++ (float) systemFontSizeForControlSize: (NSControlSize)controlSize
+{
+  switch (controlSize)
+    {
+      case NSMiniControlSize:
+        {
+	  float fontSize = [defaults floatForKey: @"NSMiniFontSize"];
+  
+	  if (fontSize == 0)
+	    {
+	      fontSize = 6;
+	    }
+	  
+	  return fontSize;
+	}
+      case NSSmallControlSize:
+	return [self smallSystemFontSize];
+      case NSRegularControlSize:
+      default:
+	return [self systemFontSize];
+    }
+}
+
 /**
  * Returns an autoreleased font with name aFontName and matrix fontMatrix.<br />
  * The fontMatrix is a standard size element matrix as used in PostScript
@@ -912,6 +948,24 @@ static BOOL flip_hack;
   return [fontInfo widthOfString: string];
 }
 
+- (unsigned) numberOfGlyphs
+{
+  // FIXME
+  return 0;
+}
+
+- (NSCharacterSet*) coveredCharacterSet
+{
+  // FIXME
+  return nil;
+}
+
+- (NSFontDescriptor*) fontDescriptor
+{
+  // FIXME
+  return nil;
+}
+
 /* The following methods have to be implemented by backends */
 
 //
@@ -1178,6 +1232,9 @@ static BOOL flip_hack;
 	      case RoleLabelFont:
 		new = [NSFont labelFontOfSize: size];
 		break;
+	      case RoleMenuBarFont:
+		new = [NSFont menuBarFontOfSize: size];
+		break;
 
 	      default:
 		NSDebugLLog(@"NSFont", @"unknown role %i", the_role);
