@@ -30,12 +30,14 @@
 #include <Foundation/NSFileManager.h>
 #include <Foundation/NSString.h>
 #include <Foundation/NSBundle.h>
-#include <AppKit/NSAttributedString.h>
-#include <AppKit/NSApplication.h>
-#include <AppKit/NSWorkspace.h>
-#include <AppKit/NSHelpManager.h>
+#include <Foundation/NSArchiver.h>
+#include "AppKit/NSAttributedString.h"
+#include "AppKit/NSApplication.h"
+#include "AppKit/NSWorkspace.h"
+#include "AppKit/NSHelpManager.h"
+#include "AppKit/NSGraphics.h"
 
-#include <AppKit/GSHelpManagerPanel.h>
+#include "GNUstepGUI/GSHelpManagerPanel.h"
 
 @implementation NSBundle (NSHelpManager)
 
@@ -108,10 +110,6 @@
 @end
 
 @implementation NSHelpManager
-{
-@private
-  NSMapTable* contextHelpTopics;
-}
 
 static NSHelpManager *_gnu_sharedHelpManager = nil;
 static BOOL _gnu_contextHelpActive = NO;
