@@ -23,8 +23,20 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */ 
 
-#include <gnustep/gui/config.h>
-#include <AppKit/NSHelpPanel.h>
+#include "config.h"
+#include "AppKit/NSHelpPanel.h"
+#include "AppKit/NSHelpManager.h"
+
+
+@implementation NSApplication (NSHelpPanel)
+
+- (void) orderFrontHelpPanel: sender
+{
+  // This is implemented in NSHelpManager.m
+  [self showHelp: sender];
+}
+
+@end
 
 @implementation NSHelpPanel
 
