@@ -2,7 +2,7 @@
 
    <abstract>The image cell class</abstract>
 
-   Copyright (C) 1999 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2005 Free Software Foundation, Inc.
 
    Author: Jonathan Gapen <jagapen@smithlab.chem.wisc.edu>
    Date: 1999
@@ -211,7 +211,7 @@ scaleProportionally(NSSize imageSize, NSRect canvasRect)
 {
   NSPoint	position;
   BOOL		is_flipped = [controlView isFlipped];
-  NSSize        imageSize;
+  NSSize        imageSize, realImageSize;
 
   NSDebugLLog(@"NSImageCell", @"NSImageCell drawInteriorWithFrame called");
 
@@ -221,41 +221,34 @@ scaleProportionally(NSSize imageSize, NSRect canvasRect)
   // leave room for the frame
   cellFrame = [self drawingRectForBounds: cellFrame];
 
+  realImageSize = [_cell_image size];
+
   switch (_imageScaling)
     {
       case NSScaleProportionally:
 	{
 	  NSDebugLLog(@"NSImageCell", @"NSScaleProportionally");
-	  [_cell_image setScalesWhenResized:YES];
-	  [_cell_image setSize: scaleProportionally (_original_image_size, 
-						     cellFrame)];
+	  imageSize = scaleProportionally (realImageSize, cellFrame);
 	  break;
 	}
       case NSScaleToFit:
 	{
 	  NSDebugLLog(@"NSImageCell", @"NSScaleToFit");
-	  [_cell_image setScalesWhenResized:YES];
-	  [_cell_image setSize: cellFrame.size];
+	  imageSize = cellFrame.size;
 	  break;
 	}
+      default:
       case NSScaleNone:
 	{
 	  NSDebugLLog(@"NSImageCell", @"NSScaleNone");
-	  [_cell_image setScalesWhenResized:NO];
-	  // don't let the image size overrun the space available
-	  if (_original_image_size.width > cellFrame.size.width
-	    || _original_image_size.height > cellFrame.size.height)
-	    [_cell_image setSize: cellFrame.size];
-	  else
-	    [_cell_image setSize: _original_image_size];
+	  imageSize = realImageSize;
 	  break;
 	}
     }
 
-  imageSize = [_cell_image size];
-
   switch (_imageAlignment)
     {
+      default:
       case NSImageAlignLeft:
 	position.x = xLeftInRect(imageSize, cellFrame);
 	position.y = yCenterInRect(imageSize, cellFrame, is_flipped);
@@ -296,10 +289,18 @@ scaleProportionally(NSSize imageSize, NSRect canvasRect)
 
   // account for flipped views
   if (is_flipped)
-    position.y += imageSize.height;
+    {
+      position.y += imageSize.height;
+      imageSize.height = -imageSize.height;
+    }
 
   // draw!
-  [_cell_image compositeToPoint: position operation: NSCompositeSourceOver];
+  [_cell_image drawInRect: NSMakeRect(position.x, position.y,
+				      imageSize.width, imageSize.height)
+		 fromRect: NSMakeRect(0, 0, realImageSize.width,
+				      realImageSize.height)
+		operation: NSCompositeSourceOver
+		 fraction: 1.0];
 
   if (_cell.shows_first_responder)
     NSDottedFrameRect(cellFrame);
