@@ -23,13 +23,14 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */ 
 
-#include <AppKit/NSEvent.h>
-#include <AppKit/NSInputManager.h>
-#include <AppKit/NSInputServer.h>
-#include <AppKit/NSText.h>
+#include <Foundation/NSException.h>
+#include "AppKit/NSEvent.h"
+#include "AppKit/NSInputManager.h"
+#include "AppKit/NSInputServer.h"
+#include "AppKit/NSText.h"
 
 /* For NSBeep () */
-#include <AppKit/NSApplication.h>
+#include "AppKit/NSGraphics.h"
 
 #include "GSKeyBindingAction.h"
 #include "GSKeyBindingTable.h"
@@ -144,7 +145,7 @@ static NSInputManager *currentInputManager = nil;
 
 + (BOOL) parseKey: (NSString *)key 
     intoCharacter: (unichar *)character
-     andModifiers: (int *)modifiers
+     andModifiers: (unsigned int *)modifiers
 {
   int flags = 0;
   unichar c = 0;
@@ -242,7 +243,7 @@ static NSInputManager *currentInputManager = nil;
 }
 
 + (NSString *) describeKeyStroke: (unichar)character
-		   withModifiers: (int)modifiers
+		   withModifiers: (unsigned int)modifiers
 {
   NSMutableString *description = [NSMutableString new];
   int i;
@@ -291,45 +292,41 @@ static NSInputManager *currentInputManager = nil;
 
 - (void) loadBindingsFromFile: (NSString *)fullPath
 {
-  NS_DURING
-    {
-      NSDictionary *bindings;
+  NSDictionary *bindings;
       
-      bindings = [NSDictionary dictionaryWithContentsOfFile: fullPath];
-      if (bindings == nil)
-	{
-	  [NSException raise];
-	}
-      
-      [_rootBindingTable loadBindingsFromDictionary: bindings];
-    }
-  NS_HANDLER
+  bindings = [NSDictionary dictionaryWithContentsOfFile: fullPath];
+  if (bindings == nil)
     {
       NSLog (@"Unable to load KeyBindings from file %@", fullPath);
     }
-  NS_ENDHANDLER 
+  else
+    {
+      [_rootBindingTable loadBindingsFromDictionary: bindings];
+    }
 }
 
 - (void) loadBindingsWithName: (NSString *)fileName
 {
   NSArray *paths;
   NSEnumerator *enumerator;
-  NSString *bundlePath;
-  
-  paths = NSSearchPathForDirectoriesInDomains (GSLibrariesDirectory,
+  NSString *libraryPath;
+  NSFileManager *fileManager = [NSFileManager defaultManager];
+
+  paths = NSSearchPathForDirectoriesInDomains (NSLibraryDirectory,
 					       NSAllDomainsMask, YES);
   /* paths are in the order - user, network, local, root. Instead we
      want to load keybindings in the order root, local, network, user
      - so that user can override root - for this reason we use a
      reverseObjectEnumerator.  */
   enumerator = [paths reverseObjectEnumerator];
-  while ((bundlePath = [enumerator nextObject]) != nil)
+  while ((libraryPath = [enumerator nextObject]) != nil)
     {
-      NSBundle *bundle = [NSBundle bundleWithPath: bundlePath];
-      NSString *fullPath = [bundle pathForResource: fileName
-				   ofType: @"dict"
-				   inDirectory: @"KeyBindings"];
-      if (fullPath != nil)
+      NSString *fullPath;
+      fullPath =
+	[[[libraryPath stringByAppendingPathComponent: @"KeyBindings"]
+	  stringByAppendingPathComponent: fileName]
+	  stringByAppendingPathExtension: @"dict"];
+      if ([fileManager fileExistsAtPath: fullPath])
 	{
 	  [self loadBindingsFromFile: fullPath];
 	}
@@ -339,8 +336,6 @@ static NSInputManager *currentInputManager = nil;
 - (NSInputManager *) initWithName: (NSString *)inputServerName
 			     host: (NSString *)hostName
 {
-  NSString *defaultKeyBindings;
-  NSArray *customKeyBindings;
   NSUserDefaults *defaults;
   CREATE_AUTORELEASE_POOL (pool);
 
@@ -391,15 +386,13 @@ static NSInputManager *currentInputManager = nil;
   }
   
 
-  /* FIXME all the following is gonna change.  */
-
   /* Normally, when we start up, we load all the keybindings we find
      in the following files, in this order:
-     
-     $GNUSTEP_SYSTEM_ROOT/Libraries/Resources/KeyBindings/DefaultKeyBindings.dict
-     $GNUSTEP_LOCAL_ROOT/Libraries/Resources/KeyBindings/DefaultKeyBindings.dict
-     $GNUSTEP_NETWORK_ROOT/Libraries/Resources/KeyBindings/DefaultKeyBindings.dict
-     $GNUSTEP_USER_ROOT/Libraries/Resources/KeyBindings/DefaultKeyBindings.dict
+
+     $GNUSTEP_SYSTEM_ROOT/Library/KeyBindings/DefaultKeyBindings.dict
+     $GNUSTEP_LOCAL_ROOT/Library/KeyBindings/DefaultKeyBindings.dict
+     $GNUSTEP_NETWORK_ROOT/Library/KeyBindings/DefaultKeyBindings.dict
+     $GNUSTEP_USER_ROOT/Library/KeyBindings/DefaultKeyBindings.dict
      
      This gives you a first way of adding your customized keybindings
      - adding a DefaultKeyBindings.dict to your GNUSTEP_USER_ROOT, and
@@ -408,45 +401,47 @@ static NSInputManager *currentInputManager = nil;
      with your own.  These keybindings are normally used by all your
      applications (this is why they are in 'DefaultKeyBindings').
 
-     In addition, you can specify a list of additional key bindings
-     files to be loaded by setting the GSCustomKeyBindings default to
-     an array of file names.  We will attempt to load all those
-     keybindings in a way similar to what we do with the
-     DefaultKeyBindings.  We load them after the default ones, in the
-     order you specify.  This allows you to have application-specific
-     keybindings, where you put different keybindings in different
-     files, and run different applications with different
-     GSCustomKeyBindings, telling them to use different keybindings
-     files.
+     You can change this behaviour, by setting the GSKeyBindingsFiles
+     default to something else.  The GSKeyBindingsFiles default
+     contains an array of files which is loaded, in that order.  Each
+     file is searched first in GNUSTEP_SYSTEM_ROOT, then
+     GNUSTEP_LOCAL_ROOT, the GNUSTEP_NETWORK_ROOT, then
+     GNUSTEP_USER_ROOT.
 
-     Last, in special cases you might want to have the
-     DefaultKeybindings totally ignored.  In this case, you set the
-     GSDefaultKeyBindings variable to a different filename (different
-     from 'DefaultKeybindings').  We attempt to load all keybindings
-     stored in the files with that name where we normally would load
-     DefaultKeybindings.  */
+     Examples - 
+
+       GSKeyBindingsFiles = (DefaultKeyBindings, NicolaKeyBindings);
+
+     will first load DefaultKeyBindings.dict (as by default), then
+     NicolaKeyBindings.dict.
+
+       GSKeyBindingsFiles = (NicolaKeyBindings);
+
+     will not load DefaultKeyBindings.dict but only
+     NicolaKeyBindings.dict.
+
+     The default of course is 
+
+       GSKeyBindingsFiles = (DefaultKeyBindings);
+
+  */
 
   /* First, load the DefaultKeyBindings.  */
-  defaultKeyBindings = [defaults stringForKey: @"GSDefaultKeyBindings"];
-  
-  if (defaultKeyBindings == nil)
+  {
+    NSArray *keyBindingsFiles = [defaults arrayForKey: @"GSKeyBindingsFiles"];
+    
+    if (keyBindingsFiles == nil)
+      {
+	keyBindingsFiles = [NSArray arrayWithObject: @"DefaultKeyBindings"];
+      }
+    
     {
-      defaultKeyBindings = @"DefaultKeyBindings";
-    }
-  
-  [self loadBindingsWithName: defaultKeyBindings];
-  
-  /* Then, if any, the CustomKeyBindings, in the specified order.  */
-  customKeyBindings = [defaults arrayForKey: @"GSCustomKeyBindings"];
-  
-  if (customKeyBindings != nil)
-    {
-      int i, count = [customKeyBindings count];
       Class string = [NSString class];
-
-      for (i = 0; i < count; i++)
+      unsigned int i;
+    
+      for (i = 0; i < [keyBindingsFiles count]; i++)
 	{
-	  NSString *filename = [customKeyBindings objectAtIndex: i];
+	  NSString *filename = [keyBindingsFiles objectAtIndex: i];
 	  
 	  if ([filename isKindOfClass: string])
 	    {
@@ -454,6 +449,17 @@ static NSInputManager *currentInputManager = nil;
 	    }
 	}
     }
+  }
+
+  /* Then, load any manually specified keybinding.  */
+  {
+     NSDictionary *keyBindings = [defaults dictionaryForKey: @"GSKeyBindings"];
+
+     if ([keyBindings isKindOfClass: [NSDictionary class]])
+       {
+	 [_rootBindingTable loadBindingsFromDictionary: keyBindings];
+       }
+  }
 
   RELEASE (pool);
   
@@ -462,6 +468,7 @@ static NSInputManager *currentInputManager = nil;
 
 - (void) dealloc
 {
+  TEST_RELEASE (_pendingKeyEvents);
   RELEASE (_rootBindingTable);
   [super dealloc];
 }
