@@ -55,6 +55,7 @@
 #include <Foundation/NSArray.h>
 #include <Foundation/NSAutoreleasePool.h>
 #include <Foundation/NSException.h>
+#include <Foundation/NSKeyedArchiver.h>
 #include <Foundation/NSNotification.h>
 #include <Foundation/NSFormatter.h>
 #include <Foundation/NSDebug.h>
@@ -72,7 +73,7 @@
 
 static NSNotificationCenter *nc;
 
-#define	STRICT	0
+#define	NSMATRIX_STRICT_CHECKING	0
 
 #ifdef MIN
 # undef MIN
@@ -378,7 +379,7 @@ static SEL getSel;
   if (column < 0)
     {
       column = 0;
-#if	STRICT == 0
+#if	NSMATRIX_STRICT_CHECKING == 0
       NSLog(@"insert negative column (%d) in matrix", column);
 #else
       [NSException raise: NSRangeException
@@ -467,7 +468,7 @@ static SEL getSel;
   if (row < 0)
     {
       row = 0;
-#if	STRICT == 0
+#if	NSMATRIX_STRICT_CHECKING == 0
       NSLog(@"insert negative row (%d) in matrix", row);
 #else
       [NSException raise: NSRangeException
@@ -633,6 +634,11 @@ static SEL getSel;
       _numCols--;
       _maxCols--;
 
+      if (_maxCols == 0)
+	{
+	  _numRows = _maxRows = 0;
+	}
+
       if (column == _selectedColumn)
 	{
 	  _selectedCell = nil;
@@ -648,7 +654,7 @@ static SEL getSel;
     }
   else
     {
-#if	STRICT == 0
+#if	NSMATRIX_STRICT_CHECKING == 0
       NSLog(@"remove non-existent column (%d) from matrix", column);
 #else
       [NSException raise: NSRangeException
@@ -679,6 +685,11 @@ static SEL getSel;
       _maxRows--;
       _numRows--;
 
+      if (_maxRows == 0)
+	{
+	  _numCols = _maxCols = 0;
+	}
+
       if (row == _selectedRow)
 	{
 	  _selectedCell = nil;
@@ -694,7 +705,7 @@ static SEL getSel;
     }
   else
     {
-#if	STRICT == 0
+#if	NSMATRIX_STRICT_CHECKING == 0
       NSLog(@"remove non-existent row (%d) from matrix", row);
 #else
       [NSException raise: NSRangeException
@@ -2203,7 +2214,7 @@ static SEL getSel;
                   // we clear the existing selection
                   // unless the Alternate or Shift keys have been pressed.
 		  if (!(mouseDownFlags & NSShiftKeyMask)
-		      && !(mouseDownFlags & NSAlternateKeyMask))
+		    && !(mouseDownFlags & NSAlternateKeyMask))
 		    {
 		      [self deselectAllCells];
 		    }
@@ -2477,77 +2488,165 @@ static SEL getSel;
 
   [super initWithCoder: aDecoder];
 
-  _myZone = [self zone];
-  [aDecoder decodeValueOfObjCType: @encode (int) at: &_mode];
-  [aDecoder decodeValueOfObjCType: @encode (BOOL) at: &_allowsEmptySelection];
-  [aDecoder decodeValueOfObjCType: @encode (BOOL) at: &_selectionByRect];
-  [aDecoder decodeValueOfObjCType: @encode (BOOL) at: &_autosizesCells];
-  [aDecoder decodeValueOfObjCType: @encode (BOOL) at: &_autoscroll];
-  _cellSize = [aDecoder decodeSize];
-  _intercell = [aDecoder decodeSize];
-  [aDecoder decodeValueOfObjCType: @encode (id) at: &_backgroundColor];
-  [aDecoder decodeValueOfObjCType: @encode (id) at: &_cellBackgroundColor];
-  [aDecoder decodeValueOfObjCType: @encode (BOOL) at: &_drawsBackground];
-  [aDecoder decodeValueOfObjCType: @encode (BOOL) at: &_drawsCellBackground];
-
-  class = NSClassFromString ((NSString *)[aDecoder decodeObject]);
-  if (class != Nil)
+  if ([aDecoder allowsKeyedCoding])
     {
-      [self setCellClass: class]; 
-    }
+      if ([aDecoder containsValueForKey: @"NSBackgroundColor"])
+        {
+	  [self setBackgroundColor: [aDecoder decodeObjectForKey: @"NSBackgroundColor"]];
+	}
+      if ([aDecoder containsValueForKey: @"NSCellBackgroundColor"])
+        {
+	  [self setCellBackgroundColor: [aDecoder decodeObjectForKey: @"NSCellBackgroundColor"]];
+	}
+      if ([aDecoder containsValueForKey: @"NSProtoCell"])
+        {
+	  [self setPrototype: [aDecoder decodeObjectForKey: @"NSProtoCell"]];
+	}
+      if ([aDecoder containsValueForKey: @"NSCellClass"])
+        {
+	    class = NSClassFromString((NSString *)[aDecoder decodeObjectForKey: @"NSCellClass"]);
+	    if (class != Nil)
+	      {
+		[self setCellClass: class]; 
+	      }
+	}
+      if ([aDecoder containsValueForKey: @"NSCellSize"])
+        {
+	  // Don't use method here as this would change the frame
+	  _cellSize = [aDecoder decodeSizeForKey: @"NSCellSize"];
+	}
+      if ([aDecoder containsValueForKey: @"NSIntercellSpacing"])
+        {
+	  // Don't use method here as this would change the frame
+	  _intercell = [aDecoder decodeSizeForKey: @"NSIntercellSpacing"];
+	}
+      if ([aDecoder containsValueForKey: @"NSMatrixFlags"])
+        {
+	  int mFlags;
 
-  cell = [aDecoder decodeObject];
-  if (cell != nil)
-    {
-      [self setPrototype: cell];
-    }
+	  mFlags = [aDecoder decodeIntForKey: @"NSMatrixFlags"];
+	  // FIXME
+	}
+      if ([aDecoder containsValueForKey: @"NSNumCols"])
+        {
+	  columns = [aDecoder decodeIntForKey: @"NSNumCols"];
+	}
+      if ([aDecoder containsValueForKey: @"NSNumRows"])
+        {
+	  rows = [aDecoder decodeIntForKey: @"NSNumRows"];
+	}
 
-  if (_cellPrototype == nil)
-    {
-      [self setCellClass: [isa cellClass]];
-    }
+      array = [aDecoder decodeObjectForKey: @"NSCells"];
+      [self renewRows: rows  columns: columns];
+      count = [array count];
+      if (count != rows * columns)
+        {
+	  NSLog (@"Trying to decode an invalid NSMatrix: cell number does not fit matrix dimension");
+	  // Quick fix to do what we can
+	  if (count > rows * columns)
+	    {
+	      count = rows * columns;
+	    }
+	}
 
-  [aDecoder decodeValueOfObjCType: @encode (int) at: &rows];
-  [aDecoder decodeValueOfObjCType: @encode (int) at: &columns];
+      for (i = 0; i < count; i++)
+        {
+	  int row, column;
+	  cell = [array objectAtIndex: i];
+	  
+	  row = i / columns;
+	  column = i % columns;
+	  
+	  [self putCell:cell   atRow: row   column: column];
+	  if ([cell state])
+	    {
+	      [self selectCellAtRow: row   column: column];
+	    }
+	}
 
-  /* NB: This works without changes for NSForm */
-  array = [aDecoder decodeObject];
-  [self renewRows: rows  columns: columns];
-  count = [array count];
-  if (count != rows * columns)
-    {
-      NSLog (@"Trying to decode an invalid NSMatrix: cell number does not fit matrix dimension");
-      // Quick fix to do what we can
-      if (count > rows * columns)
-	{
-	  count = rows * columns;
+      if ([aDecoder containsValueForKey: @"NSSelectedCol"])
+        {
+	  _selectedColumn = [aDecoder decodeIntForKey: @"NSSelectedCol"];
+	}
+      if ([aDecoder containsValueForKey: @"NSSelectedRow"])
+        {
+	  _selectedRow = [aDecoder decodeIntForKey: @"NSSelectedRow"];
 	}
     }
-
-  _selectedRow = _selectedColumn = 0;
-
-  for (i = 0; i < count; i++)
+  else
     {
-      int row, column;
-      cell = [array objectAtIndex: i];
-
-      row = i / columns;
-      column = i % columns;
+      _myZone = [self zone];
+      [aDecoder decodeValueOfObjCType: @encode (int) at: &_mode];
+      [aDecoder decodeValueOfObjCType: @encode (BOOL) at: &_allowsEmptySelection];
+      [aDecoder decodeValueOfObjCType: @encode (BOOL) at: &_selectionByRect];
+      [aDecoder decodeValueOfObjCType: @encode (BOOL) at: &_autosizesCells];
+      [aDecoder decodeValueOfObjCType: @encode (BOOL) at: &_autoscroll];
+      _cellSize = [aDecoder decodeSize];
+      _intercell = [aDecoder decodeSize];
+      [aDecoder decodeValueOfObjCType: @encode (id) at: &_backgroundColor];
+      [aDecoder decodeValueOfObjCType: @encode (id) at: &_cellBackgroundColor];
+      [aDecoder decodeValueOfObjCType: @encode (BOOL) at: &_drawsBackground];
+      [aDecoder decodeValueOfObjCType: @encode (BOOL) at: &_drawsCellBackground];
       
-      [self putCell:cell   atRow: row   column: column];
-      if ([cell state])
-	 {
-	   [self selectCellAtRow: row   column: column];
-	 }
-     }
+      class = NSClassFromString ((NSString *)[aDecoder decodeObject]);
+      if (class != Nil)
+        {
+	  [self setCellClass: class]; 
+	}
+      
+      cell = [aDecoder decodeObject];
+      if (cell != nil)
+        {
+	  [self setPrototype: cell];
+	}
+      
+      if (_cellPrototype == nil)
+        {
+	  [self setCellClass: [isa cellClass]];
+	}
+      
+      [aDecoder decodeValueOfObjCType: @encode (int) at: &rows];
+      [aDecoder decodeValueOfObjCType: @encode (int) at: &columns];
 
-  [aDecoder decodeValueOfObjCType: @encode (id) at: &_delegate];
-  [aDecoder decodeValueOfObjCType: @encode (id) at: &_target];
-  [aDecoder decodeValueOfObjCType: @encode (SEL) at: &_action];
-  [aDecoder decodeValueOfObjCType: @encode (SEL) at: &_doubleAction];
-  [aDecoder decodeValueOfObjCType: @encode (SEL) at: &_errorAction];
-  [aDecoder decodeValueOfObjCType: @encode (BOOL) at: &_tabKeyTraversesCells];
-  [self setKeyCell: [aDecoder decodeObject]];
+      /* NB: This works without changes for NSForm */
+      array = [aDecoder decodeObject];
+      [self renewRows: rows  columns: columns];
+      count = [array count];
+      if (count != rows * columns)
+        {
+	  NSLog (@"Trying to decode an invalid NSMatrix: cell number does not fit matrix dimension");
+	  // Quick fix to do what we can
+	  if (count > rows * columns)
+	    {
+	      count = rows * columns;
+	    }
+	}
+
+      _selectedRow = _selectedColumn = 0;
+
+      for (i = 0; i < count; i++)
+        {
+	  int row, column;
+	  cell = [array objectAtIndex: i];
+	  
+	  row = i / columns;
+	  column = i % columns;
+	  
+	  [self putCell:cell   atRow: row   column: column];
+	  if ([cell state])
+	    {
+	      [self selectCellAtRow: row   column: column];
+	    }
+	}
+      
+      [aDecoder decodeValueOfObjCType: @encode (id) at: &_delegate];
+      [aDecoder decodeValueOfObjCType: @encode (id) at: &_target];
+      [aDecoder decodeValueOfObjCType: @encode (SEL) at: &_action];
+      [aDecoder decodeValueOfObjCType: @encode (SEL) at: &_doubleAction];
+      [aDecoder decodeValueOfObjCType: @encode (SEL) at: &_errorAction];
+      [aDecoder decodeValueOfObjCType: @encode (BOOL) at: &_tabKeyTraversesCells];
+      [self setKeyCell: [aDecoder decodeObject]];
+    }
   
   return self;
 }
@@ -3353,7 +3452,6 @@ static SEL getSel;
 {
   if (_textObject)
     {
-      [_textObject setString: @""];
       [_selectedCell endEditing: _textObject];
       _textObject = nil;
       return YES;
@@ -3437,7 +3535,7 @@ static SEL getSel;
 //NSLog(@"%x - mr: %d mc:%d nr:%d nc:%d r:%d c:%d", (unsigned)self, _maxRows, _maxCols, _numRows, _numCols, row, col);
   if (row < 0)
     {
-#if	STRICT == 0
+#if	NSMATRIX_STRICT_CHECKING == 0
       NSLog(@"renew negative row (%d) in matrix", row);
 #else
       [NSException raise: NSRangeException
@@ -3447,7 +3545,7 @@ static SEL getSel;
     }
   if (col < 0)
     {
-#if	STRICT == 0
+#if	NSMATRIX_STRICT_CHECKING == 0
       NSLog(@"renew negative column (%d) in matrix", col);
 #else
       [NSException raise: NSRangeException
@@ -3503,18 +3601,19 @@ static SEL getSel;
       int	end = row - 1;
 
       _cells = NSZoneRealloc(_myZone, _cells, row * sizeof(id*));
-      _selectedCells = NSZoneRealloc(_myZone, _selectedCells, row * sizeof(BOOL*));
+      _selectedCells
+	= NSZoneRealloc(_myZone, _selectedCells, row * sizeof(BOOL*));
 
       /* Allocate the new rows and fill them */
       for (i = oldMaxR; i < row; i++)
 	{
-	  _cells[i] = NSZoneMalloc(_myZone, col * sizeof(id));
+	  _cells[i] = NSZoneMalloc(_myZone, _maxCols * sizeof(id));
 	  _selectedCells[i] = NSZoneMalloc(GSAtomicMallocZone(),
-	    col * sizeof(BOOL));
+	    _maxCols * sizeof(BOOL));
 
 	  if (i == end)
 	    {
-	      for (j = 0; j < col; j++)
+	      for (j = 0; j < _maxCols; j++)
 		{
 		  _cells[i][j] = nil;
 		  _selectedCells[i][j] = NO;
@@ -3530,7 +3629,7 @@ static SEL getSel;
 	    }
 	  else
 	    {
-	      for (j = 0; j < col; j++)
+	      for (j = 0; j < _maxCols; j++)
 		{
 		  _cells[i][j] = nil;
 		  _selectedCells[i][j] = NO;
@@ -3577,9 +3676,9 @@ static SEL getSel;
 	  NSCell	*aCell = _cells[i][j];
 
           if ([aCell isEnabled]
-              && ([aCell state] != state || [aCell isHighlighted] != highlight
-                  || (state == NSOffState && _selectedCells[i][j] != NO)
-                  || (state != NSOffState && _selectedCells[i][j] == NO)))
+	    && ([aCell state] != state || [aCell isHighlighted] != highlight
+	      || (state == NSOffState && _selectedCells[i][j] != NO)
+	      || (state != NSOffState && _selectedCells[i][j] == NO)))
             {
 	      [aCell setState: state];
 
@@ -3608,8 +3707,7 @@ static SEL getSel;
 	{
 	  if ([_cells[row][j] isEnabled] && [_cells[row][j] isSelectable])
 	    {
-	      _selectedCell = [self selectTextAtRow: row
-				    column: j];
+	      _selectedCell = [self selectTextAtRow: row column: j];
 	      _selectedRow = row;
 	      _selectedColumn = j;
 	      return YES;
@@ -3623,8 +3721,7 @@ static SEL getSel;
 	{
 	  if ([_cells[i][j] isEnabled] && [_cells[i][j] isSelectable])
 	    {
-	      _selectedCell = [self selectTextAtRow: i
-				    column: j];
+	      _selectedCell = [self selectTextAtRow: i column: j];
 	      _selectedRow = i;
 	      _selectedColumn = j;
 	      return YES;
