@@ -27,11 +27,17 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */ 
 
-#include <gnustep/gui/config.h>
+#include "config.h"
 #include <Foundation/NSUserDefaults.h>
 #include <Foundation/NSDictionary.h>
-#include <AppKit/NSMenuItem.h>
-#include <AppKit/NSMenu.h>
+#include <Foundation/NSDebug.h>
+#include <Foundation/NSException.h>
+#include <GNUstepBase/GSCategories.h>
+#include "AppKit/NSMenuItem.h"
+#include "AppKit/NSMenu.h"
+#include "AppKit/NSImage.h"
+#include "AppKit/NSEvent.h"
+#include "AppKit/NSCell.h"
 
 static BOOL usesUserKeyEquivalents = NO;
 static Class imageClass;
@@ -67,7 +73,7 @@ static Class imageClass;
 {
   if (self == [NSMenuItem class])
     {
-      [self setVersion: 1];
+      [self setVersion: 2];
       imageClass = [NSImage class];
     }
 }
@@ -84,7 +90,7 @@ static Class imageClass;
 
 + (id <NSMenuItem>) separatorItem
 {
-  return [GSMenuSeparator new];
+  return AUTORELEASE([GSMenuSeparator new]);
 }
 
 - (id) init
@@ -121,8 +127,8 @@ static Class imageClass;
   _enabled = YES;
   //_image = nil;
   // Set the images according to the spec. On: check mark; off: dash.
-  [self setOnStateImage: [imageClass imageNamed: @"common_2DCheckMark"]];
-  [self setMixedStateImage: [imageClass imageNamed: @"common_2DDash"]];
+  [self setOnStateImage: [imageClass imageNamed: @"NSMenuCheckmark"]];
+  [self setMixedStateImage: [imageClass imageNamed: @"NSMenuMixedState"]];
   //_offStateImage = nil;
   //_target = nil;
   _action = aSelector;
@@ -223,7 +229,7 @@ static Class imageClass;
 
 - (NSString*) userKeyEquivalent
 {
-  NSString *userKeyEquivalent = [[[[NSUserDefaults standardUserDefaults]
+  NSString *userKeyEquivalent = [(NSDictionary*)[[[NSUserDefaults standardUserDefaults]
 				      persistentDomainForName: NSGlobalDomain]
 				     objectForKey: @"NSCommandKeys"]
 				    objectForKey: _title];
@@ -411,8 +417,6 @@ static Class imageClass;
 {
   NSMenuItem *copy = (NSMenuItem*)NSCopyObject (self, 0, zone);
 
-  NSDebugLog (@"menu item '%@' copy", _title);
-
   // We reset the menu to nil to allow the reuse of the copy
   copy->_menu = nil;
   copy->_title = [_title copyWithZone: zone];
@@ -443,32 +447,77 @@ static Class imageClass;
   [aCoder encodeObject: _offStateImage];
   [aCoder encodeObject: _mixedStateImage];
   [aCoder encodeValueOfObjCType: @encode(BOOL) at: &_changesState];
-  [aCoder encodeConditionalObject: _target];
   [aCoder encodeValueOfObjCType: @encode(SEL) at: &_action];
   [aCoder encodeValueOfObjCType: "i" at: &_tag];
   [aCoder encodeConditionalObject: _representedObject];
   [aCoder encodeObject: _submenu];
+  [aCoder encodeConditionalObject: _target];
 }
 
 - (id) initWithCoder: (NSCoder*)aDecoder
 {
-  [aDecoder decodeValueOfObjCType: @encode(id) at: &_title];
-  [aDecoder decodeValueOfObjCType: @encode(id) at: &_keyEquivalent];
-  [aDecoder decodeValueOfObjCType: "I" at: &_keyEquivalentModifierMask];
-  [aDecoder decodeValueOfObjCType: "I" at: &_mnemonicLocation];
-  [aDecoder decodeValueOfObjCType: "i" at: &_state];
-  [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_enabled];
-  [aDecoder decodeValueOfObjCType: @encode(id) at: &_image];
-  [aDecoder decodeValueOfObjCType: @encode(id) at: &_onStateImage];
-  [aDecoder decodeValueOfObjCType: @encode(id) at: &_offStateImage];
-  [aDecoder decodeValueOfObjCType: @encode(id) at: &_mixedStateImage];
-  [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_changesState];
-  _target = [aDecoder decodeObject];
-  [aDecoder decodeValueOfObjCType: @encode(SEL) at: &_action];
-  [aDecoder decodeValueOfObjCType: "i" at: &_tag];
-  [aDecoder decodeValueOfObjCType: @encode(id) at: &_representedObject];
-  [aDecoder decodeValueOfObjCType: @encode(id) at: &_submenu];
+  if ([aDecoder allowsKeyedCoding])
+    {
+      NSString *title = [aDecoder decodeObjectForKey: @"NSTitle"];
+      NSString *action = [aDecoder decodeObjectForKey: @"NSAction"];
+      NSString *key = [aDecoder decodeObjectForKey: @"NSKeyEquiv"];
+      //NSMenu *menu = [aDecoder decodeObjectForKey: @"NSMenu"];
+      NSImage *mixedImage = [aDecoder decodeObjectForKey: @"NSMixedImage"];
+      NSImage *onImage = [aDecoder decodeObjectForKey: @"NSOnImage"];
+       id target = [aDecoder decodeObjectForKey: @"NSTarget"];
 
+      self = [self initWithTitle: title
+		   action: NSSelectorFromString(action)
+		   keyEquivalent: key];
+      [self setTarget: target];
+      [self setMixedStateImage: mixedImage];
+      [self setOnStateImage: onImage];
+
+      if ([aDecoder containsValueForKey: @"NSKeyEquivModMask"])
+        {
+	    //int keyMask = [aDecoder decodeIntForKey: @"NSKeyEquivModMask"];
+	}
+      if ([aDecoder containsValueForKey: @"NSMnemonicLoc"])
+        {
+	  int loc = [aDecoder decodeIntForKey: @"NSMnemonicLoc"];
+	  [self setMnemonicLocation: loc];
+	}
+      if ([aDecoder containsValueForKey: @"NSState"])
+        {
+	  int state = [aDecoder decodeIntForKey: @"NSState"];
+
+	  [self setState: state];
+	}
+    }
+  else
+    {
+      int version = [aDecoder versionForClassName: 
+				  @"NSMenuItem"];
+    
+      [aDecoder decodeValueOfObjCType: @encode(id) at: &_title];
+      [aDecoder decodeValueOfObjCType: @encode(id) at: &_keyEquivalent];
+      [aDecoder decodeValueOfObjCType: "I" at: &_keyEquivalentModifierMask];
+      [aDecoder decodeValueOfObjCType: "I" at: &_mnemonicLocation];
+      [aDecoder decodeValueOfObjCType: "i" at: &_state];
+      [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_enabled];
+      [aDecoder decodeValueOfObjCType: @encode(id) at: &_image];
+      [aDecoder decodeValueOfObjCType: @encode(id) at: &_onStateImage];
+      [aDecoder decodeValueOfObjCType: @encode(id) at: &_offStateImage];
+      [aDecoder decodeValueOfObjCType: @encode(id) at: &_mixedStateImage];
+      [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_changesState];
+      if (version == 1)
+        {
+	  _target = [aDecoder decodeObject];
+	}
+      [aDecoder decodeValueOfObjCType: @encode(SEL) at: &_action];
+      [aDecoder decodeValueOfObjCType: "i" at: &_tag];
+      [aDecoder decodeValueOfObjCType: @encode(id) at: &_representedObject];
+      [aDecoder decodeValueOfObjCType: @encode(id) at: &_submenu];
+      if (version == 2)
+        {
+	  _target = [aDecoder decodeObject];
+	}
+    }
   return self;
 }
 
