@@ -185,12 +185,12 @@ _addLeftBorderOffsetToRect(NSRect aRect)
                     selector: @selector(itemRemoved:)
                         name: NSMenuDidRemoveItemNotification
                       object: _attachedMenu];
+
+      // Force menu view's layout to be recalculated.
+      [self setNeedsSizing: YES];
+      
+      [self update];
     }
-
-  // Force menu view's layout to be recalculated.
-  [self setNeedsSizing: YES];
-
-  [self update];
 }
 
 - (NSMenu*) menu
@@ -1284,7 +1284,7 @@ _addLeftBorderOffsetToRect(NSRect aRect)
   NSRect	currentFrame;
   NSRect	originalFrame;
   NSPoint	currentTopLeft;
-  NSPoint	originalTopLeft;
+  NSPoint	originalTopLeft = NSZeroPoint; /* Silence compiler.  */
   BOOL          restorePosition;
   /*
    * Only for non transient menus do we want
