@@ -156,7 +156,7 @@
     }
 }
 
-- (void) insertText: (NSString*)aString
+- (void) insertText: (id)aString
 {
   if (_next_responder)
     [_next_responder insertText: aString];
@@ -338,18 +338,51 @@
  */
 - (void) encodeWithCoder: (NSCoder*)aCoder
 {
-  [aCoder encodeValueOfObjCType: @encode(NSInterfaceStyle)
-			     at: &_interface_style];
-  [aCoder encodeObject: _menu];
+  if([aCoder allowsKeyedCoding])
+    {
+      if(_interface_style != NSNoInterfaceStyle)
+	{
+	  [aCoder encodeInt: _interface_style
+		  forKey: @"NSInterfaceStyle"];
+	}
+
+      if([self menu] != nil)
+	{
+	  [aCoder encodeObject: [self menu]
+		  forKey: @"NSMenu"];	  
+	}
+    }
+  else
+    {
+      [aCoder encodeValueOfObjCType: @encode(NSInterfaceStyle)
+	      at: &_interface_style];
+      [aCoder encodeObject: _menu];
+    }
 }
 
 - (id) initWithCoder: (NSCoder*)aDecoder
 {
-  id obj;
+  id obj = nil;
+
+  if([aDecoder allowsKeyedCoding])
+    {
+      if([aDecoder containsValueForKey: @"NSInterfaceStyle"])
+	{
+	  _interface_style = [aDecoder decodeIntForKey: @"NSInterfaceStyle"];
+	}
+
+      if([aDecoder containsValueForKey: @"NSMenu"])
+	{
+	  obj = [aDecoder decodeObjectForKey: @"NSMenu"];
+	}
+    }
+  else
+    {
+      [aDecoder decodeValueOfObjCType: @encode(NSInterfaceStyle)
+		at: &_interface_style];
+      obj = [aDecoder decodeObject];
+    }
 
-  [aDecoder decodeValueOfObjCType: @encode(NSInterfaceStyle)
-			       at: &_interface_style];
-  obj = [aDecoder decodeObject];
   [self setMenu: obj];
 
   return self;
