@@ -23,21 +23,24 @@
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
 */
 
-/* FIXME - HAVE_RINT */
-#include <gnustep/gui/config.h>
 #include <math.h>
+#include "config.h"
 
 #include <Foundation/NSArray.h>
 #include <Foundation/NSDebug.h>
 #include <Foundation/NSException.h>
 #include <Foundation/NSValue.h>
-#include <AppKit/NSColor.h>
-#include <AppKit/NSFont.h>
-#include <AppKit/NSGraphics.h>
-#include <AppKit/NSRulerMarker.h>
-#include <AppKit/NSRulerView.h>
-#include <AppKit/NSScrollView.h>
-#include <AppKit/PSOperators.h>
+#include "AppKit/NSAttributedString.h"
+#include "AppKit/NSBezierPath.h"
+#include "AppKit/NSColor.h"
+#include "AppKit/NSEvent.h"
+#include "AppKit/NSFont.h"
+#include "AppKit/NSGraphics.h"
+#include "AppKit/NSRulerMarker.h"
+#include "AppKit/NSRulerView.h"
+#include "AppKit/NSScrollView.h"
+
+DEFINE_RINT_IF_MISSING
 
 #define MIN_LABEL_DISTANCE 40
 #define MIN_MARK_DISTANCE 5
@@ -47,18 +50,20 @@
 #define BIG_MARK_SIZE 6
 #define LABEL_MARK_SIZE 11
 
-#define BASE_LINE_LOCATION 0
 #define RULER_THICKNESS 16
+#define MARKER_THICKNESS 15
 
-#define ASSIGN_FIRST_VALUE(firstVal, tRect) (firstVal = (int)((tRect - _zeroLocation) / labelDistInRuler) * labelDistInRuler)
-
-#define ASSIGN_FIRST_MARKER(firstMarker, tRect) (firstMarker = ceil(((tRect - _zeroLocation) - firstVal) / (_unitToRuler * markDistInUnits)))
-
-#define ASSIGN_DIFERENCE(diference, tRect) (diference = (firstMarker * (markDistInUnits * _unitToRuler)) + firstVal + _zeroLocation - tRect)
-
-#define SET_MARK_COUNT(markCount, tRect, dif) (markCount = floor(1 + (tRect - dif) / (_unitToRuler * markDistInUnits)))
-
-#define DRAW_HASH_MARK(context, size)  {if (_orientation == NSHorizontalRuler) DPSrlineto(context, 0, size); else DPSrlineto(context, size, 0);}
+#define DRAW_HASH_MARK(path, size)			\
+		do {					\
+		  if (_orientation == NSHorizontalRuler)\
+		    {					\
+		      [path relativeLineToPoint: NSMakePoint(0, size)];	\
+		    }					\
+		  else					\
+		    {					\
+		      [path relativeLineToPoint: NSMakePoint(size, 0)];	\
+		    }					\
+		} while (0)
 
 @interface GSRulerUnit : NSObject
 {
@@ -162,7 +167,7 @@ unitToPointsConversionFactor: (float)factor
 
 /*
  * Class variables
-*/
+ */
 static NSMutableDictionary *units = nil;
 
 /*
@@ -177,7 +182,6 @@ static NSMutableDictionary *units = nil;
       NSArray *array2;
       NSArray *array10;
 
-      NSDebugLog(@"Initialize NSRulerView class\n");
       [self setVersion: 0.01];
 
       units = [[NSMutableDictionary alloc] init];
@@ -221,7 +225,14 @@ static NSMutableDictionary *units = nil;
       [self setRuleThickness: RULER_THICKNESS];
       [self setOriginOffset: 0.0];
       [self setReservedThicknessForAccessoryView: 0.0];
-      [self setReservedThicknessForMarkers: 0.0];
+      if (o == NSHorizontalRuler)
+        {
+          [self setReservedThicknessForMarkers: MARKER_THICKNESS];
+	}
+      else
+        {
+          [self setReservedThicknessForMarkers: 0.0];
+	}
       [self invalidateHashMarks];
     }
   return self;
@@ -238,7 +249,7 @@ static NSMutableDictionary *units = nil;
 				unitToPointsConversionFactor: conversionFactor
 				stepUpCycle: stepUpCycle
 				stepDownCycle: stepDownCycle];
-  [units setObject: u  forKey: uName];
+  [units setObject: u forKey: uName];
 }
 
 - (void) setMeasurementUnits: (NSString *)uName
@@ -252,7 +263,7 @@ static NSMutableDictionary *units = nil;
 		   format: @"Unknown measurement unit %@", uName];
     }
   ASSIGN(_unit, newUnit);
-  [self setNeedsDisplay: YES];
+  [self invalidateHashMarks];
 }
 
 - (NSString *) measurementUnits
@@ -262,16 +273,19 @@ static NSMutableDictionary *units = nil;
 
 - (void) setClientView: (NSView *)aView
 {
+  if (_clientView == aView)
+    return;
+
   if (_clientView != nil  
       && [_clientView respondsToSelector: 
 			@selector(rulerView:willSetClientView:)]) 
     {
-      [_clientView rulerView: self  willSetClientView: aView];
+      [_clientView rulerView: self willSetClientView: aView];
     }
   /* NB: We should not RETAIN the clientView.  */
   _clientView = aView;
   [self setMarkers: nil];
-  [self setNeedsDisplay: YES];
+  [self invalidateHashMarks];
 }
 
 - (BOOL) isOpaque
@@ -299,7 +313,7 @@ static NSMutableDictionary *units = nil;
 - (void) setOriginOffset: (float)offset
 {
   _originOffset = offset;
-  [self setNeedsDisplay: YES];
+  [self invalidateHashMarks];
 }
 
 - (float) originOffset
@@ -307,6 +321,32 @@ static NSMutableDictionary *units = nil;
   return _originOffset;
 }
 
+- (void) _verifyReservedThicknessForMarkers
+{
+  NSEnumerator *en;
+  NSRulerMarker *marker;
+  float maxThickness = _reservedThicknessForMarkers;
+  
+  if (_markers == nil)
+    {
+      return;
+    }
+  en = [_markers objectEnumerator];
+  while ((marker = [en nextObject]) != nil)
+    {
+      float markerThickness;
+      markerThickness = [marker thicknessRequiredInRuler];
+      if (markerThickness > maxThickness)
+        {
+	  maxThickness = markerThickness;
+	}
+    }
+  if (maxThickness > _reservedThicknessForMarkers)
+    {
+      [self setReservedThicknessForMarkers: maxThickness];
+    }
+}
+
 - (void) setMarkers: (NSArray *)newMarkers
 {
   if (newMarkers != nil && _clientView == nil)
@@ -317,6 +357,7 @@ static NSMutableDictionary *units = nil;
   if (newMarkers != nil)
     {
       ASSIGN(_markers, [NSMutableArray arrayWithArray: newMarkers]);
+      [self _verifyReservedThicknessForMarkers];
     }
   else
     {
@@ -365,8 +406,102 @@ static NSMutableDictionary *units = nil;
 - (BOOL) trackMarker: (NSRulerMarker *)aMarker
       withMouseEvent: (NSEvent *)theEvent
 {
-  /* FIXME/TODO: not implemented */
-  return NO;
+  NSParameterAssert(aMarker != nil);
+  
+  return [aMarker trackMouse: theEvent adding: YES];
+}
+
+- (NSRect) _rulerRect
+{
+  NSRect rect = [self bounds]; 
+  if (_orientation == NSHorizontalRuler)
+    {
+      rect.size.height = _ruleThickness;
+      if ([self isFlipped])
+        {
+	  rect.origin.y = [self baselineLocation];
+	}
+      else
+        {
+	  rect.origin.y = [self baselineLocation] - _ruleThickness;
+	}
+    }
+  else
+    {
+      rect.size.width = _ruleThickness;
+      rect.origin.x = [self baselineLocation];
+    }
+  return rect;
+}
+
+- (NSRect) _markersRect
+{
+  NSRect rect = [self bounds]; 
+  if (_orientation == NSHorizontalRuler)
+    {
+      rect.size.height = _reservedThicknessForMarkers;
+      if ([self isFlipped])
+        {
+	  rect.origin.y = _reservedThicknessForAccessoryView;
+	}
+      else
+        {
+	  rect.origin.y = _ruleThickness;
+	}
+    }
+  else
+    {
+      rect.size.width = _reservedThicknessForMarkers;
+      rect.origin.x = _reservedThicknessForAccessoryView;
+    }
+  return rect;
+}
+
+- (NSRulerMarker *) _markerAtPoint: (NSPoint)point
+{
+  NSEnumerator *markerEnum;
+  NSRulerMarker *marker;
+  BOOL flipped = [self isFlipped];
+  
+  /* test markers in reverse order so that markers drawn on top
+     are tested before those underneath */
+  markerEnum = [_markers reverseObjectEnumerator];
+  while ( (marker = [markerEnum nextObject]) != nil)
+    {
+      if (NSMouseInRect (point, [marker imageRectInRuler], flipped))
+        {
+	  return marker;
+	}
+    }
+  return nil;
+}
+
+- (void) mouseDown: (NSEvent*)theEvent
+{
+  NSPoint clickPoint;
+  BOOL flipped = [self isFlipped];
+  
+  clickPoint = [self convertPoint: [theEvent locationInWindow]
+                         fromView: nil];
+  if (NSMouseInRect (clickPoint, [self _rulerRect], flipped))
+    {
+      if (_clientView != nil  
+          && [_clientView respondsToSelector: 
+			    @selector(rulerView:handleMouseDown:)]) 
+        {
+          [_clientView rulerView: self handleMouseDown: theEvent];
+        }
+    }
+  else if (NSMouseInRect (clickPoint, [self _markersRect], flipped))
+    {
+      NSRulerMarker *clickedMarker;
+      
+      clickedMarker = [self _markerAtPoint: clickPoint];
+      if (clickedMarker != nil)
+        {
+	  [clickedMarker trackMouse: theEvent adding: NO];
+	}
+    }
 }
 
 - (void) moveRulerlineFromLocation: (float)oldLoc 
@@ -406,25 +541,22 @@ static NSMutableDictionary *units = nil;
 {
   if (! _cacheIsValid)
     {
-      NSRect rect;
-      NSRect unitRect;
+      NSSize unitSize;
       float  cf; 
       int    convIndex;
 
-      /* calculate the position of the zero coordinate in the ruler
-       * and the size one unit in document view has in the ruler */
+      /* calculate the size one unit in document view has in the ruler */
       cf = [_unit conversionFactor];
-      rect = NSMakeRect(_originOffset, _originOffset, cf, cf);
-      unitRect = [self convertRect: rect  
+      unitSize = [self convertSize: NSMakeSize(cf, cf)  
 		       fromView: [_scrollView documentView]];
 
       if (_orientation == NSHorizontalRuler) 
         {
-          _unitToRuler = NSWidth(unitRect);
+          _unitToRuler = unitSize.width;
         }
       else 
         {
-          _unitToRuler = NSHeight(unitRect);
+          _unitToRuler = unitSize.height;
         }
 
       /* Calculate distance between marks.  */
@@ -448,7 +580,7 @@ static NSMutableDictionary *units = nil;
       /* calculate number of small marks in each bigger mark */
       _marksToMidMark = rint([self _stepForIndex: convIndex + 1]);
       _marksToBigMark = _marksToMidMark 
-                     * rint([self _stepForIndex: convIndex + 2]);
+                      * rint([self _stepForIndex: convIndex + 2]);
     
       /* Calculate distance between labels.
          It must not be less than MIN_LABEL_DISTANCE. */
@@ -479,127 +611,155 @@ static NSMutableDictionary *units = nil;
     }
 }
 
-- (void) drawHashMarksAndLabelsInRect: (NSRect)drawRect
+- (void) drawHashMarksAndLabelsInRect: (NSRect)aRect
 {
-  NSGraphicsContext *ctxt = GSCurrentContext();
-  NSFont *font;
   NSView *docView;
+  NSRect docBounds;
+  NSRect baselineRect;
+  NSRect visibleBaselineRect;
+  float firstBaselineLocation;
   float firstVisibleLocation;
-  float visibleLength;
+  float lastVisibleLocation;
   int firstVisibleMark;
   int lastVisibleMark;
   int mark;
-  NSRect baseLineRect;
+  int firstVisibleLabel;
+  int lastVisibleLabel;
+  int label;
   float baselineLocation = [self baselineLocation];
   NSPoint zeroPoint;
+  float zeroLocation;
+  NSBezierPath *path;
+  NSFont *font = [NSFont systemFontOfSize: [NSFont smallSystemFontSize]];
+  NSDictionary *attr = [[NSDictionary alloc] 
+			   initWithObjectsAndKeys: 
+			       font, NSFontAttributeName,
+			       [NSColor blackColor], NSForegroundColorAttributeName,
+			       nil];
 
   docView = [_scrollView documentView];
-  
-  zeroPoint = [self convertPoint: NSMakePoint(_originOffset, _originOffset) 
-		    fromView: docView];
+  docBounds = [docView bounds];
 
+  /* Calculate the location of 'zero' hash mark */
+  // _originOffset is an offset from document bounds origin, in doc coords
+  zeroPoint.x = docBounds.origin.x + _originOffset;
+  zeroPoint.y = docBounds.origin.y + _originOffset;
+  zeroPoint = [self convertPoint: zeroPoint fromView: docView];
   if (_orientation == NSHorizontalRuler)
     {
-      _zeroLocation = zeroPoint.x;
+      zeroLocation = zeroPoint.x;
     }
   else
     {
-      _zeroLocation = zeroPoint.y;
+      zeroLocation = zeroPoint.y;
     }
 
   [self _verifyCachedValues];
 
-  /* TODO: should be a user default */
-  font = [NSFont systemFontOfSize: 8];
-  [font set];
-  [[NSColor blackColor] set];
-
-  baseLineRect = [self convertRect: [docView bounds]  fromView: docView];
-
+  /* Calculate the base line (corresponds to the document bounds) */
+  baselineRect = [self convertRect: docBounds  fromView: docView];
   if (_orientation == NSHorizontalRuler) 
     {
-      baseLineRect.origin.y = baselineLocation; 
-      baseLineRect.size.height = 1;
-      baseLineRect = NSIntersectionRect(baseLineRect, drawRect);
-      firstVisibleLocation = NSMinX(baseLineRect);
-      visibleLength = NSWidth(baseLineRect);
+      baselineRect.origin.y = baselineLocation; 
+      baselineRect.size.height = 1;
+      firstBaselineLocation = NSMinX(baselineRect);
+      visibleBaselineRect = NSIntersectionRect(baselineRect, aRect);
+      firstVisibleLocation = NSMinX(visibleBaselineRect);
+      lastVisibleLocation = NSMaxX(visibleBaselineRect);
     }
   else 
     {
-      baseLineRect.origin.x = baselineLocation;
-      baseLineRect.size.width = 1;
-      baseLineRect = NSIntersectionRect(baseLineRect, drawRect);
-      firstVisibleLocation = NSMinY(baseLineRect);
-      visibleLength = NSHeight(baseLineRect);
+      baselineRect.origin.x = baselineLocation;
+      baselineRect.size.width = 1;
+      firstBaselineLocation = NSMinY(baselineRect);
+      visibleBaselineRect = NSIntersectionRect(baselineRect, aRect);
+      firstVisibleLocation = NSMinY(visibleBaselineRect);
+      lastVisibleLocation = NSMaxY(visibleBaselineRect);
     }
 
   /* draw the base line */
-  NSRectFill(baseLineRect);
+  [[NSColor blackColor] set];
+  NSRectFill(visibleBaselineRect);
 
   /* draw hash marks */
-  firstVisibleMark = ceil((firstVisibleLocation - _zeroLocation)
+  firstVisibleMark = ceil((firstVisibleLocation - zeroLocation)
                           / _markDistance);
-  lastVisibleMark = floor((firstVisibleLocation + visibleLength - _zeroLocation) 
+  lastVisibleMark = floor((lastVisibleLocation - zeroLocation) 
                           / _markDistance);
+  path = [NSBezierPath new];
   
   for (mark = firstVisibleMark; mark <= lastVisibleMark; mark++)
     {
       float markLocation;
 
-      markLocation = _zeroLocation + mark * _markDistance;
+      markLocation = zeroLocation + mark * _markDistance;
       if (_orientation == NSHorizontalRuler)
         {
-          DPSmoveto(ctxt, markLocation, baselineLocation);
+	  [path moveToPoint: NSMakePoint(markLocation, baselineLocation)];
         }
       else
         {
-          DPSmoveto(ctxt, baselineLocation, markLocation);
+	  [path moveToPoint: NSMakePoint(baselineLocation, markLocation)];
         }
 
       if ((mark % _marksToLabel) == 0) 
         {
-          float labelValue;
-          NSString *label;
-
-          DRAW_HASH_MARK(ctxt, LABEL_MARK_SIZE);
-
-          /* draw label */
-          /* FIXME: shouldn't be using NSCell to draw labels? */
-          labelValue = (markLocation - _zeroLocation) / _unitToRuler;
-          if ([self isFlipped] == NO && labelValue != 0.0)
-	    {
-	      label = [NSString stringWithFormat: _labelFormat, -labelValue];
-	    }
-          else
-	    {
-	      label = [NSString stringWithFormat: _labelFormat, labelValue];
-	    }
-          if (_orientation == NSHorizontalRuler)
-            {
-              DPSrmoveto(ctxt, 2, 0);
-            }
-          else
-            {
-              float labelWidth;
-              labelWidth = [font widthOfString: label];
-              DPSrmoveto(ctxt, 3 - labelWidth, 9);
-            }
-          DPSshow(ctxt, [label cString]);
+          DRAW_HASH_MARK(path, LABEL_MARK_SIZE);
         }
       else if ((mark % _marksToBigMark) == 0) 
         {
-          DRAW_HASH_MARK(ctxt, BIG_MARK_SIZE);
+          DRAW_HASH_MARK(path, BIG_MARK_SIZE);
         }
       else if ((mark % _marksToMidMark) == 0)
         {
-          DRAW_HASH_MARK(ctxt, MID_MARK_SIZE);
+          DRAW_HASH_MARK(path, MID_MARK_SIZE);
         }
       else 
         {
-          DRAW_HASH_MARK(ctxt, MARK_SIZE);
+          DRAW_HASH_MARK(path, MARK_SIZE);
         }
     }
-  DPSstroke(ctxt);
+  [path stroke];
+  RELEASE(path);
+
+  /* draw labels */
+  /* FIXME: shouldn't be using NSCell to draw labels? */
+  firstVisibleLabel = floor((firstVisibleLocation - zeroLocation)
+                          / (_marksToLabel * _markDistance));
+  lastVisibleLabel = floor((lastVisibleLocation - zeroLocation) 
+                          / (_marksToLabel * _markDistance));
+  /* firstVisibleLabel can be to the left of the visible ruler area.
+     This is OK because just part of the label can be visible to the left
+     when scrolling. However, it should not be drawn if outside of the
+     baseline. */
+  if (zeroLocation + firstVisibleLabel * _marksToLabel * _markDistance
+      < firstBaselineLocation)
+    {
+      firstVisibleLabel++;
+    }
+  
+  for (label = firstVisibleLabel; label <= lastVisibleLabel; label++)
+    {
+      float labelLocation = zeroLocation + label * _marksToLabel * _markDistance;
+      float labelValue = (labelLocation - zeroLocation) / _unitToRuler;
+      NSString *labelString = [NSString stringWithFormat: _labelFormat, labelValue];
+      NSSize size = [labelString sizeWithAttributes: attr];
+      NSPoint labelPosition;
+
+      if (_orientation == NSHorizontalRuler)
+        {
+	  labelPosition.x = labelLocation + 1;
+	  labelPosition.y = baselineLocation + LABEL_MARK_SIZE + 4 - size.height;
+        }
+      else
+        {
+	  labelPosition.x = baselineLocation + LABEL_MARK_SIZE + 4 - size.width;
+	  labelPosition.y = labelLocation + 1;
+        }
+      [labelString drawAtPoint: labelPosition withAttributes: attr];
+    }
+
+  RELEASE(attr);
 }
 
 - (void) drawMarkersInRect: (NSRect)aRect
@@ -617,12 +777,13 @@ static NSMutableDictionary *units = nil;
 - (void) invalidateHashMarks
 {
   _cacheIsValid = NO;
+  [self setNeedsDisplay:YES];
 }
 
-- (void) setScrollView: (NSScrollView *)sView
+- (void) setScrollView: (NSScrollView *)scrollView
 {
   /* We do NOT retain the scrollView; the scrollView is retaining us.  */
-  _scrollView = sView;
+  _scrollView = scrollView;
 }
 
 - (NSScrollView *) scrollView
@@ -653,7 +814,6 @@ static NSMutableDictionary *units = nil;
 
 - (void) setReservedThicknessForMarkers: (float)thickness
 {
-  /*  NSLog(@"requiredThicknessForMarkers: %f", thickness); */
   _reservedThicknessForMarkers = thickness;
   [_scrollView tile];
 }
@@ -712,7 +872,6 @@ static NSMutableDictionary *units = nil;
 - (void) dealloc
 {
   RELEASE(_unit);
-  RELEASE(_scrollView);
   RELEASE(_clientView);
   RELEASE(_accessoryView);
   RELEASE(_markers);
