@@ -79,6 +79,7 @@ static BOOL _gs_display_reading_progress = NO;
 // Methods to manage default settings
 - (id) _initWithoutGModel;
 - (void) _getOriginalSize;
+- (void) _setDefaultDirectory;
 - (void) _resetDefaults;
 // Methods invoked by buttons
 - (void) _setHomeDirectory;
@@ -340,10 +341,30 @@ static BOOL _gs_display_reading_progress = NO;
   _originalSize = [[self contentView] frame].size;
 }
 
+/* Set the current directory to a useful default value */
+- (void) _setDefaultDirectory
+{
+  NSString *path;
+
+  if (_directory == nil)
+    {
+      path = [[NSUserDefaults standardUserDefaults] 
+		 objectForKey: @"NSDefaultOpenDirectory"];
+      if (path == nil)
+        {
+	  // FIXME: Should we use this or the home directory?
+	  ASSIGN(_directory, [_fm currentDirectoryPath]);
+	}
+      else
+        {
+	  ASSIGN(_directory, path);
+	}
+    }
+}
+
 - (void) _resetDefaults
 {
-  if (_directory == nil)
-    ASSIGN (_directory, [_fm currentDirectoryPath]);
+  [self _setDefaultDirectory];
   [self setPrompt: @"Name:"];
   [self setTitle: @"Save"];
   [self setRequiredFileType: @""];
@@ -490,30 +511,22 @@ selectCellWithString: (NSString*)title
 {
   if (path == nil)
     {
-      if (_directory == nil)
-	ASSIGN(_directory, [_fm currentDirectoryPath]);
+      [self _setDefaultDirectory];
     }
   else
     {
-      ASSIGN (_directory, path);
+      ASSIGN(_directory, path);
     }
   if (filename == nil)
     filename = @"";
-  ASSIGN (_fullFileName, [path stringByAppendingPathComponent: filename]);
+  ASSIGN(_fullFileName, [_directory stringByAppendingPathComponent: filename]);
   [_browser setPath: _fullFileName];
 
-  [self _selectCellName:filename];
+  [self _selectCellName: filename];
   [[_form cellAtIndex: 0] setStringValue: filename];
   [_form selectTextAtIndex: 0];
   [_form setNeedsDisplay: YES];
 
-  /*
-   * We need to take care of the possibility of 
-   * the panel being aborted.  We return NSCancelButton 
-   * in that case.
-   */
-  _OKButtonPressed = NO;
-
   [self _browser: _browser
 	selectCellWithString: [[_browser selectedCell] stringValue] 
 	inColumn: [_browser selectedColumn]];
@@ -524,7 +537,8 @@ selectCellWithString: (NSString*)title
 {
   if (*isDir == NO)
     {
-      if ([extension isEqualToString: _requiredFileType] == NO)
+      if (_requiredFileType != nil && [_requiredFileType length] != 0
+          && [extension isEqualToString: _requiredFileType] == NO)
 	return NO;
     }
   else if ([extension length] == 0)
@@ -533,7 +547,8 @@ selectCellWithString: (NSString*)title
     }
   else if (_treatsFilePackagesAsDirectories == NO)
     {
-      if ([extension isEqualToString: _requiredFileType] == YES)
+      if (_requiredFileType == nil || [_requiredFileType length] == 0
+              || [extension isEqualToString: _requiredFileType] == YES)
 	  *isDir = NO;
     }
 
@@ -856,14 +871,20 @@ selectCellWithString: (NSString*)title
  * any selected files that don't already have that extension;
  * The argument type should not include the period that begins 
  * the extension.  Invoke this method each time the Save panel 
- * is used for another file type within the application.
+ * is used for another file type within the application.  If
+ * you do not invoke it, or set it to empty string or nil, no
+ * extension will be appended, indicated by an empty string
+ * returned from -requiredFileType .
  */
 - (void) setRequiredFileType: (NSString*)fileType
 {
   ASSIGN(_requiredFileType, fileType);
 }
 
-/** Returns the required file type. The default is no required file type.  */
+/**
+ * Returns the required file type. The default, indicated by empty string,
+ * is no required file type.
+ */
 - (NSString*) requiredFileType
 {
   return _requiredFileType;
@@ -1008,12 +1029,9 @@ selectCellWithString: (NSString*)title
   if (_fullFileName == nil)
    return @"";
 
-  if (_requiredFileType == nil)
+  if (_requiredFileType == nil || [_requiredFileType isEqual: @""] == YES)
     return _fullFileName;
 
-  if ([_requiredFileType isEqual: @""] == YES)
-    return _fullFileName;
-					    
   // add filetype extension only if the filename does not include it already
   if ([[_fullFileName pathExtension] isEqual: _requiredFileType] == YES)
     return _fullFileName;
@@ -1028,8 +1046,7 @@ selectCellWithString: (NSString*)title
 
 - (void) cancel: (id)sender
 {
-  _fullFileName = nil;
-  _directory = nil;
+  ASSIGN(_directory, [_browser pathToColumn:[_browser lastColumn]]);
   [NSApp stopModalWithCode: NSCancelButton];
   [self close];
 }
@@ -1139,7 +1156,6 @@ selectCellWithString: (NSString*)title
     if (![_delegate panel: self isValidFilename: [self filename]])
       return;
 
-  _OKButtonPressed = YES;
   [NSApp stopModalWithCode: NSOKButton];
   [self close];
 }
@@ -1192,16 +1208,17 @@ selectCellWithString: (NSString*)title
     _delegateHasUserEnteredFilename = NO;
 
   if ([aDelegate respondsToSelector: @selector(panel:directoryDidChange:)])
-     _delegateHasDirectoryDidChange = YES;
+    _delegateHasDirectoryDidChange = YES;
   else 
     _delegateHasDirectoryDidChange = NO;      
 
   if ([aDelegate respondsToSelector: @selector(panelSelectionDidChange:)])
-     _delegateHasSelectionDidChange = YES;
+    _delegateHasSelectionDidChange = YES;
   else 
     _delegateHasSelectionDidChange = NO;      
 
   [super setDelegate: aDelegate];
+  [self validateVisibleColumns];
 }
 
 - (void) setCanSelectHiddenExtension: (BOOL) flag
@@ -1550,7 +1567,8 @@ createRowsForColumn: (int)column
 {
   NSArray	*cells = [[sender matrixInColumn: column] cells];
   unsigned	count = [cells count], i;
-  
+  NSString	*path = [sender pathToColumn: column];
+
   // iterate through the cells asking the delegate if each filename is valid
   // if it says no for any filename, the column is not valid
   if (_delegateHasShowFilenameFilter == YES)
@@ -1558,7 +1576,8 @@ createRowsForColumn: (int)column
       {
 	if (![_delegate panel: self 
 			shouldShowFilename:
-			  [[cells objectAtIndex: i] stringValue]])
+			  [path stringByAppendingPathComponent:
+			    [[cells objectAtIndex: i] stringValue]]])
 	  return NO;
       }
 
