@@ -27,13 +27,12 @@
 */ 
 
 #include <Foundation/Foundation.h>
-#include <AppKit/NSApplication.h>
-#include <AppKit/NSScreen.h>
-#include <AppKit/NSInterfaceStyle.h>
-#include <AppKit/NSGraphicsContext.h>
-#include <AppKit/DPSOperators.h>
-#include <AppKit/NSGraphics.h>
-#include <AppKit/AppKitExceptions.h>
+#include "AppKit/NSApplication.h"
+#include "AppKit/NSScreen.h"
+#include "AppKit/NSInterfaceStyle.h"
+#include "AppKit/NSGraphicsContext.h"
+#include "AppKit/AppKitExceptions.h"
+#include "GNUstepGUI/GSDisplayServer.h"
 
 @interface NSScreen (Private)
 - (id) _initWithScreenNumber: (int)screen;
@@ -55,25 +54,30 @@
 
 static NSMutableArray *screenArray = nil;
 
+/**
+ * Resets the cached list of screens.
+ */
 + (void) resetScreens
 {
   screenArray = nil;
 }
 
+/**
+ * Returns an NSArray containing NSScreen instances representing all of the
+ * screen devices attached to the computer.
+ */
 + (NSArray*) screens
 {
   int count = 0, index = 0;
-  int *windows = NULL;
-  NSGraphicsContext	*ctxt;
+  NSArray *screens;
+  GSDisplayServer *srv;
 
   if (screenArray != nil)
     return screenArray;
 
-  ctxt = [NSApp context];
-  // Get the number of screens.
-  DPScountscreenlist(ctxt, 0, &count);
-
-  // If the list is empty quit...
+  srv = GSCurrentServer();
+  screens = [srv screenList];
+  count = [screens count];
   if (count == 0)
     {
       // something is wrong. This shouldn't happen.
@@ -82,9 +86,6 @@ static NSMutableArray *screenArray = nil;
       return nil;
     }
 
-  windows = NSZoneMalloc(NSDefaultMallocZone(), (count+1)*sizeof(int));
-  DPSscreenlist(ctxt, 0, count, windows);
-
   screenArray = [NSMutableArray new];
 
   // Iterate over the list
@@ -92,22 +93,26 @@ static NSMutableArray *screenArray = nil;
     {
       NSScreen *screen = nil;
       
-      screen = [[NSScreen alloc] _initWithScreenNumber: windows[index]];
+      screen = [[NSScreen alloc] _initWithScreenNumber: 
+      			[[screens objectAtIndex: index] intValue]];
       [screenArray addObject: AUTORELEASE(screen)];
     }
 
-  // free the list
-  NSZoneFree(NSDefaultMallocZone(), windows);
-  
   return [NSArray arrayWithArray: screenArray];
 }
 
 // Creating NSScreen Instances
+/**
+ * Gets information about the main screen.
+ */
 + (NSScreen*) mainScreen
 {
   return [[self screens] objectAtIndex: 0];
 }
 
+/**
+ * Gets information about the screen with the highest depth (i.e. bits per pixel).
+ */
 + (NSScreen*) deepestScreen
 {
   NSArray *screenArray = [self screens];
@@ -139,18 +144,23 @@ static NSMutableArray *screenArray = nil;
  * Instance methods
  */
 
-// NSScreen does not respond to the init method.
+/**
+ * NSScreen does not respond to the init method.
+ */
 - (id) init
 {
   [self doesNotRecognizeSelector: _cmd];
   return nil;
 }
 
-// Get all of the infomation for a given screen.
+/**
+ * Get all of the infomation for a given screen.
+ */
 - (id) _initWithScreenNumber: (int)screen
 {
-  float			x, y, w, h;
-  NSGraphicsContext	*ctxt = [NSApp context];
+  GSDisplayServer *srv;
+
+  srv = GSCurrentServer();
 
   self = [super init];
 
@@ -167,25 +177,17 @@ static NSMutableArray *screenArray = nil;
       return nil;
     }
 
-  if (ctxt == nil)
+  if (srv == nil)
     {
       NSLog(@"Internal error: No current context\n");
       RELEASE(self);
       return nil;
     }
 
-  if ([ctxt isDrawingToScreen] == NO)
-    {
-      NSLog(@"Internal error: trying to find screen with wrong context\n");
-      RELEASE(self);
-      return nil;
-    }
-  
   // Fill in all of the i-vars with appropriate values.
   _screenNumber = screen;
-  DPScurrentwindowbounds(ctxt, _screenNumber, &x, &y, &w, &h);
-  _frame = NSMakeRect(x, y, w, h);
-  _depth = GSWindowDepthForScreen(ctxt, _screenNumber);
+  _frame = [srv boundsForScreen: _screenNumber];
+  _depth = [srv windowDepthForScreen: _screenNumber];
   _supportedWindowDepths = NULL;
 
   return self;
@@ -202,19 +204,30 @@ static NSMutableArray *screenArray = nil;
   return NO;
 }
 
-/*
- * Reading Screen Information
+/**
+ * Returns the depth of the screen in bits.
  */
 - (NSWindowDepth) depth
 {
   return _depth;
 }
 
+/**
+ * The full frame of the screen.
+ */
 - (NSRect) frame
 {
   return _frame;
 }
 
+/**
+ * <p>
+ * This method generates a dictionary containing information
+ * about the screen device.  The resulting dictionary will have the following
+ * entires: NSScreenNumber, NSDeviceSize, NSDeviceResolution, NSDeviceBitsPerSample,
+ * NSDeviceColorSpaceName.
+ * </p>
+ */
 - (NSDictionary*) deviceDescription
 {
   NSMutableDictionary	*devDesc;
@@ -240,8 +253,7 @@ static NSMutableArray *screenArray = nil;
 	      forKey: NSDeviceSize];
 
   // Add the NSDeviceResolution dictionary item
-  screenResolution = GSResolutionForScreen([NSApp context], 
-					   _screenNumber);
+  screenResolution = [GSCurrentServer() resolutionForScreen: _screenNumber];
   [devDesc setObject: [NSValue valueWithSize: screenResolution]
 	      forKey: NSDeviceResolution];
 
@@ -259,14 +271,18 @@ static NSMutableArray *screenArray = nil;
 }
 
 // Mac OS X methods
+/**
+ * Returns the window depths this screen will support.
+ */
 - (const NSWindowDepth*) supportedWindowDepths
 {
   // If the variable isn't initialized, get the info and
   // store it for the future.
   if (_supportedWindowDepths == NULL)
     {
-      _supportedWindowDepths = (NSWindowDepth*)GSAvailableDepthsForScreen(
-	  [NSApp context], _screenNumber);
+      _supportedWindowDepths = 
+        (NSWindowDepth*)[GSCurrentServer()
+			       availableDepthsForScreen: _screenNumber];
 
       // Check the results
       if (_supportedWindowDepths == NULL)
@@ -279,24 +295,37 @@ static NSMutableArray *screenArray = nil;
   return _supportedWindowDepths;
 }
 
+/**
+ * Returns the NSRect representing the visible area of the screen.
+ */
 - (NSRect) visibleFrame
 {
   NSRect visFrame = _frame;
 
-  switch ([NSApp interfaceStyle])
+  switch (NSInterfaceStyleForKey(@"NSIntefaceStyle", nil))
     {
       case NSMacintoshInterfaceStyle:
 	// What is the size of the Mac menubar?
 	visFrame.size.height -= 25;
 	return visFrame;
-      case NSWindows95InterfaceStyle:
+      case GSWindowMakerInterfaceStyle:
       case NSNextStepInterfaceStyle:
+	visFrame.size.width -= 64;
+	return visFrame;
+      
+      case NSWindows95InterfaceStyle:
       case NSNoInterfaceStyle:
       default:
 	return _frame;
     }
 }
 
+/** Returns the screen number */
+- (int) screenNumber
+{
+  return _screenNumber;
+}
+
 // Release the memory for the depths array.
 - (void) dealloc
 {
