@@ -34,13 +34,13 @@
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
 */
 
-#include <Foundation/Foundation.h>
-#include <AppKit/NSCell.h>
-#include <AppKit/NSFileWrapper.h>
-#include <AppKit/NSImage.h>
-#include <AppKit/NSEvent.h>
-#include <AppKit/NSTextContainer.h>
-#include <AppKit/NSTextAttachment.h>
+#include "AppKit/NSCell.h"
+#include "AppKit/NSFileWrapper.h"
+#include "AppKit/NSImage.h"
+#include "AppKit/NSEvent.h"
+#include "AppKit/NSTextContainer.h"
+#include "AppKit/NSTextAttachment.h"
+#include "AppKit/NSTextView.h"
 
 
 @implementation NSTextAttachmentCell
@@ -75,8 +75,7 @@
 {
   NSRect aRect;
   
-  // FIXME: We ignore the proposedLineFragment
-  aRect.origin = position;
+  aRect.origin = [self cellBaselineOffset];
   aRect.size = [self cellSize];
   return aRect;
 }
@@ -272,23 +271,27 @@
 
 @implementation NSTextAttachment
 
-- (id)init
+- (id) init
 {
   return [self initWithFileWrapper: nil];
 }
 
-- (void)dealloc
+- (void) dealloc
 {
   DESTROY(_fileWrapper);
   DESTROY(_cell);
+  [super dealloc];
 }
 
-- (id)initWithFileWrapper:(NSFileWrapper *)fileWrapper
+- (id) initWithFileWrapper:(NSFileWrapper *)fileWrapper
 {
-  ASSIGN(_fileWrapper, fileWrapper);
-  _cell = [[NSTextAttachmentCell alloc ] init];
-  [_cell setAttachment: self];
-
+  self = [super init];
+  if (self != nil)
+    {
+      ASSIGN(_fileWrapper, fileWrapper);
+      _cell = [[NSTextAttachmentCell alloc ] init];
+      [_cell setAttachment: self];
+    }
   return self;
 }
 
