@@ -35,6 +35,7 @@
 #include "AppKit/NSGraphics.h"
 #include "AppKit/NSTextFieldCell.h"
 #include "AppKit/NSText.h"
+#include "AppKit/NSEvent.h"
 
 static NSColor	*bgCol;
 static NSColor	*txtCol;
@@ -67,7 +68,7 @@ static NSColor	*txtCol;
 {
   if (self == [NSTextFieldCell class])
     {
-      [self setVersion: 1];
+      [self setVersion: 2];
       [[NSNotificationCenter defaultCenter] 
 	addObserver: self
 	selector: @selector(_systemColorsChanged:)
@@ -94,6 +95,7 @@ static NSColor	*txtCol;
   ASSIGN(_background_color, bgCol);
   _textfieldcell_draws_background = NO;
   _textfieldcell_is_opaque = NO;
+  _action_mask = NSKeyUpMask | NSKeyDownMask;
   return self;
 }
 
@@ -199,7 +201,7 @@ static NSColor	*txtCol;
 - (id) initWithCoder: (NSCoder*)aDecoder
 {
   self = [super initWithCoder: aDecoder];
-
+ 
   if ([aDecoder allowsKeyedCoding])
     {
       [self setBackgroundColor: [aDecoder decodeObjectForKey: @"NSBackgroundColor"]];
@@ -214,6 +216,22 @@ static NSColor	*txtCol;
     {
       BOOL tmp;
 
+      if ([aDecoder versionForClassName:@"NSTextFieldCell"] < 2)
+        {
+	  /* Replace the old default _action_mask with the new default one
+	     if it's set. There isn't really a way to modify this value
+	     on an NSTextFieldCell encoded in a .gorm file. The old default value
+	     causes problems with newer NSTableViews which uses this to discern 
+	     whether it should trackMouse:inRect:ofView:untilMouseUp: or not.
+	     This also disables the action from being sent on an uneditable and
+	     unselectable text fields.
+	  */
+	    if (_action_mask == NSLeftMouseUpMask)
+	      {
+		_action_mask = NSKeyUpMask | NSKeyDownMask;
+	      }
+	}
+
       [aDecoder decodeValueOfObjCType: @encode(id) at: &_background_color];
       [aDecoder decodeValueOfObjCType: @encode(id) at: &_text_color];
       [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &tmp];
