@@ -175,14 +175,11 @@ static NSString	*GSWorkspaceNotification = @"GSWorkspaceNotification";
 
 @end
 
-
 
 @interface NSWorkspace (Private)
 
 // Icon handling
 - (NSImage*) _extIconForApp: (NSString*)appName info: (NSDictionary*)extInfo;
-- (NSImage*) _getImageWithName: (NSString*)name
-		     alternate: (NSString*)alternate;
 - (NSImage*) unknownFiletypeImage;
 - (NSImage*) _saveImageFor: (NSString*)iconPath;
 - (NSString*) thumbnailForFile: (NSString *)file;
@@ -813,56 +810,22 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 	|| [pathExtension isEqualToString: @"debug"]
 	|| [pathExtension isEqualToString: @"profile"])
 	{
-	  NSBundle	*bundle;
-
-	  isApplication = YES;
-	  bundle = [NSBundle bundleWithPath: fullPath];
-	  iconPath = [[bundle infoDictionary] objectForKey: @"NSIcon"];
-	  if (iconPath && [iconPath isAbsolutePath] == NO)
+          isApplication = YES;
+	  
+	  image = [self appIconForApp: fullPath];
+	  
+	  if (image == nil)
 	    {
-	      NSString	*file = iconPath;
-
-	      iconPath = [bundle pathForImageResource: file];
-
 	      /*
-	       * If there is no icon in the Resources of the app, try
-	       * looking directly in the app wrapper.
-	       */
-	      if (iconPath == nil)
-		{
-		  iconPath = [fullPath stringByAppendingPathComponent: file];
-		  if ([mgr isReadableFileAtPath: iconPath] == NO)
-		    {
-		      iconPath = nil;
-		    }
-		}
-	    }
-	  /*
-	   *	If there is no icon specified in the Info.plist for app
-	   *	try 'wrapper/app.tiff'
-	   */
-	  if (iconPath == nil)
-	    {
-	      NSString	*str;
-
-	      str = [fullPath lastPathComponent];
-	      str = [str stringByDeletingPathExtension];
-	      iconPath = [fullPath stringByAppendingPathComponent: str];
-	      iconPath = [iconPath stringByAppendingPathExtension: @"tiff"];
-	      if ([mgr isReadableFileAtPath: iconPath] == NO)
-		{
-		  iconPath = nil;
-		  /*
-		   * Just use the appropriate icon for the path extension
-		   */
-		  image = [self _iconForExtension: pathExtension];
-		}
+               * Just use the appropriate icon for the path extension
+               */
+              return [self _iconForExtension: pathExtension];
 	    }
 	}
 
       /*
-       *	If we have no iconPath, try 'dir/.dir.tiff' as a
-       *	possible locations for the directory icon.
+       * If we have no iconPath, try 'dir/.dir.tiff' as a
+       * possible locations for the directory icon.
        */
       if (iconPath == nil)
 	{
@@ -887,9 +850,8 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 		{
 		  if (rootImage == nil)
 		    {
-		      rootImage = RETAIN([self _getImageWithName:
-			@"Root_PC.tiff"
-			alternate: @"common_Root_PC.tiff"]);
+		      rootImage = RETAIN([NSImage _standardImageWithName:
+			@"Root_PC.tiff"]);
 		    }
 
 		  image = rootImage;
@@ -898,9 +860,8 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 		{
 		  if (homeImage == nil)
 		    {
-		      homeImage = RETAIN([self _getImageWithName:
-			@"HomeDirectory.tiff"
-			alternate: @"common_HomeDirectory.tiff"]);
+		      homeImage = RETAIN([NSImage _standardImageWithName:
+			@"HomeDirectory.tiff"]);
 		    }
 		  image = homeImage;
 		}
@@ -908,8 +869,8 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 		{
 		  if (folderImage == nil)
 		    {
-		      folderImage = RETAIN([self _getImageWithName:
-			@"Folder.tiff" alternate: @"common_Folder.tiff"]);
+		      folderImage = RETAIN([NSImage _standardImageWithName:
+			@"Folder.tiff"]);
 		    }
 		  image = folderImage;
 		}
@@ -950,9 +911,8 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 		{
 		  if (unknownTool == nil)
 		    {
-		      unknownTool = RETAIN([self _getImageWithName:
-			@"UnknownTool.tiff" alternate:
-			@"common_UnknownTool.tiff"]);
+		      unknownTool = RETAIN([NSImage _standardImageWithName:
+			@"UnknownTool.tiff"]);
 		    }
 		  image = unknownTool;
 		}
@@ -1481,21 +1441,66 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
  */
 - (NSImage*) appIconForApp: (NSString*)appName
 {
-  NSBundle	*bundle = [self bundleForApp: appName];
-  NSString	*iconPath;
-
+  NSBundle *bundle;
+  NSImage *image = nil;
+  NSFileManager *mgr = [NSFileManager defaultManager];
+  NSString *iconPath = nil;
+  NSString *fullPath;
+  
+  fullPath = [self fullPathForApplication: appName];
+  bundle = [self bundleForApp: fullPath];
   if (bundle == nil)
     {
       return nil;
     }
+  
   iconPath = [[bundle infoDictionary] objectForKey: @"NSIcon"];
-
-  if (![iconPath isAbsolutePath])
+  
+  if (iconPath && [iconPath isAbsolutePath] == NO)
     {
-      iconPath = [[bundle bundlePath] stringByAppendingPathComponent: iconPath];
+      NSString *file = iconPath;
+
+      iconPath = [bundle pathForImageResource: file];
+
+      /*
+       * If there is no icon in the Resources of the app, try
+       * looking directly in the app wrapper.
+       */
+      if (iconPath == nil)
+        {
+          iconPath = [fullPath stringByAppendingPathComponent: file];
+          if ([mgr isReadableFileAtPath: iconPath] == NO)
+            {
+              iconPath = nil;
+            }
+        }
+    }
+    
+  /*
+   * If there is no icon specified in the Info.plist for app
+   * try 'wrapper/app.tiff'
+   */
+  if (iconPath == nil)
+    {
+      NSString *str;
+
+      str = [fullPath lastPathComponent];
+      str = [str stringByDeletingPathExtension];
+      iconPath = [fullPath stringByAppendingPathComponent: str];
+      iconPath = [iconPath stringByAppendingPathExtension: @"tiff"];
+      
+      if ([mgr isReadableFileAtPath: iconPath] == NO)
+        {
+          iconPath = nil;
+        }
     }
   
-  return [self _saveImageFor: iconPath];
+  if (iconPath != nil)
+    {
+      image = [self _saveImageFor: iconPath];
+    }
+  
+  return image;
 }
 
 /**
@@ -1640,7 +1645,7 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
   NSDictionary	*typeInfo = [extInfo objectForKey: appName];
   NSString	*file = [typeInfo objectForKey: @"NSIcon"];
 
-  if (file)
+  if (file && [file length] != 0)
     {
       if ([file isAbsolutePath] == NO)
 	{
@@ -1668,26 +1673,14 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
   return nil;
 }
 
-- (NSImage*) _getImageWithName: (NSString*)name
-		     alternate: (NSString*)alternate
-{
-  NSImage	*image = nil;
-
-  image = [NSImage imageNamed: name];
-  if (image == nil)
-    image = [NSImage imageNamed: alternate];
-  return image;
-}
-
-/** Returns the default icon to display for a directory */
+/** Returns the default icon to display for a file */
 - (NSImage*) unknownFiletypeImage
 {
   static NSImage *image = nil;
 
   if (image == nil)
     {
-      image = RETAIN([self _getImageWithName: @"Unknown.tiff"
-				   alternate: @"common_Unknown.tiff"]);
+      image = RETAIN([NSImage _standardImageWithName: @"Unknown.tiff"]);
     }
 
   return image;
@@ -1819,9 +1812,8 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
 	    {
 	      if (unknownApplication == nil)
 		{
-		  unknownApplication = RETAIN([self _getImageWithName:
-		    @"UnknownApplication.tiff" alternate:
-		    @"common_UnknownApplication.tiff"]);
+		  unknownApplication = RETAIN([NSImage _standardImageWithName:
+		    @"UnknownApplication.tiff"]);
 		}
 	      icon = unknownApplication;
 	    }
