@@ -26,9 +26,9 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#include <gnustep/gui/config.h>
+#include "config.h"
 #include <Foundation/NSString.h>
-#include <AppKit/NSEvent.h>
+#include "AppKit/NSEvent.h"
 
 // Global strings
 NSString *NSModalPanelRunLoopMode = @"ModalPanelMode";
@@ -73,6 +73,8 @@ NSString *NSWindowServerCommunicationException = @"WindowServerCommunication";
 NSString *NSWordTablesReadException = @"WordTablesRead";
 NSString *NSWordTablesWriteException = @"WordTablesWrite";
 
+NSString *GSWindowServerInternalException = @"WindowServerInternal";
+
 // Application notifications
 NSString *NSApplicationDidBecomeActiveNotification
               = @"ApplicationDidBecomeActive";
@@ -103,6 +105,7 @@ NSString *NSDeviceBlackColorSpace = @"NSDeviceBlackColorSpace";
 NSString *NSDeviceRGBColorSpace = @"NSDeviceRGBColorSpace";
 NSString *NSDeviceCMYKColorSpace = @"NSDeviceCMYKColorSpace";
 NSString *NSNamedColorSpace = @"NSNamedColorSpace";
+NSString *NSPatternColorSpace = @"NSPatternColorSpace";
 NSString *NSCustomColorSpace = @"NSCustomColorSpace";
 
 // NSColor Global gray values
@@ -227,6 +230,9 @@ NSString *NSGeneralPboardType = @"NSGeneralPboardType";
 NSString *NSPDFPboardType = @"NSPDFPboardType";
 NSString *NSPICTPboardType = @"NSPICTPboardType";
 NSString *NSURLPboardType = @"NSURLPboardType";
+NSString *NSHTMLPboardType = @"NSHTMLPboardType";
+NSString *NSVCardPboardType = @"NSVCardPboardType";
+NSString *NSFilesPromisePboardType = @"NSFilesPromisePboardType";
 
 // Pasteboard Name Globals
 NSString *NSDragPboard = @"NSDragPboard";
@@ -277,6 +283,8 @@ NSString *NSPrintHorizontalPagination = @"PrintHorizontalPagination";
 NSString *NSPrintVerticalPagination = @"PrintVerticalPagination";
 NSString *NSPrintVerticallyCentered = @"PrintVerticallyCentered";
 
+NSString *NSPrintPageDirection = @"NSPrintPageDirection";
+
 // Print Job Disposition Values
 NSString  *NSPrintCancelJob = @"PrintCancelJob";
 NSString  *NSPrintFaxJob = @"PrintFaxJob";
@@ -419,6 +427,19 @@ NSString *NSBaselineOffsetAttributeName = @"NSBaselineOffsetAttributeName";
 NSString *NSKernAttributeName = @"NSKernAttributeName";
 NSString *NSLinkAttributeName = @"NSLinkAttributeName";
 
+// NSToolbar notifications
+NSString *NSToolbarDidRemoveItemNotification = @"NSToolbarDidRemoveItemNotification";
+NSString *NSToolbarWillAddItemNotification = @"NSToolbarWillAddItemNotification";
+
+// NSToolbarItem constants
+NSString *NSToolbarSeparatorItemIdentifier = @"NSToolbarSeparatorItemIdentifier";
+NSString *NSToolbarSpaceItemIdentifier = @"NSToolbarSpaceItemIdentifier";
+NSString *NSToolbarFlexibleSpaceItemIdentifier = @"NSToolbarFlexibleSpaceItemIdentifier";
+NSString *NSToolbarShowColorsItemIdentifier = @"NSToolbarShowColorsItemIdentifier";
+NSString *NSToolbarShowFontsItemIdentifier = @"NSToolbarShowFontsItemIdentifier";
+NSString *NSToolbarCustomizeToolbarItemIdentifier = @"NSToolbarCustomizeToolbarItemIdentifier";
+NSString *NSToolbarPrintItemIdentifier = @"NSToolbarPrintItemIdentifier";
+
 /*
  * NSTextView userInfo for notifications 
  */
