@@ -1,6 +1,6 @@
 
 #include <Foundation/Foundation.h>
-#include <AppKit/AppKit.h>
+#include "AppKit/AppKit.h"
 
 void __objc_gui_linking(void)
 {
