@@ -68,6 +68,8 @@ readNSString (StringContext *ctxt)
   BOOL italic;
   BOOL underline;
   int script;
+
+  float real_fi, real_li;
 }
 
 - (NSFont*) currentFont;
@@ -102,7 +104,7 @@ readNSString (StringContext *ctxt)
 {
   RTFAttribute *new =  (RTFAttribute *)NSCopyObject (self, 0, zone);
 
-  new->paragraph = [paragraph copyWithZone: zone];
+  new->paragraph = [paragraph mutableCopyWithZone: zone];
   RETAIN(new->fontName);
   RETAIN(new->fgColour);
   RETAIN(new->bgColour);
@@ -142,12 +144,34 @@ readNSString (StringContext *ctxt)
 					    size: fontSize];
   if (font == nil)
     {
-	NSDebugMLLog(@"RTFParser", 
-		     @"Could not find font %@ size %f traits %d weight %d", 
-		     fontName, fontSize, traits, weight);
-	font = [NSFont userFontOfSize: fontSize];
-    }
+      /* Before giving up and using a default font, we try if this is
+       * not the case of a font with a composite name, such as
+       * 'Helvetica-Light'.  In that case, even if we don't have
+       * exactly an 'Helvetica-Light' font family, we might have an
+       * 'Helvetica' one.  */
+      NSRange range = [fontName rangeOfString:@"-"];
 
+      if (range.location != NSNotFound)
+	{
+	  NSString *fontFamily = [fontName substringToIndex: range.location];
+
+	  font = [[NSFontManager sharedFontManager] fontWithFamily: fontFamily
+						    traits: traits
+						    weight: weight
+						    size: fontSize];
+	}
+      
+      if (font == nil)
+	{
+	  NSDebugMLLog(@"RTFParser", 
+		       @"Could not find font %@ size %f traits %d weight %d", 
+		       fontName, fontSize, traits, weight);
+
+	  /* Last resort, default font.  :-(  */
+	  font = [NSFont userFontOfSize: fontSize];
+	}
+    }
+  
   return font;
 }
 
@@ -166,7 +190,9 @@ readNSString (StringContext *ctxt)
 
 - (void) resetParagraphStyle
 {
-  ASSIGN(paragraph, [NSMutableParagraphStyle defaultParagraphStyle]);
+  DESTROY(paragraph);
+  paragraph = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
+  real_fi = real_li = 0.0;
 
   tabChanged = NO;
   changed = YES;
@@ -195,11 +221,14 @@ readNSString (StringContext *ctxt)
 				      location: location];
 
   if (!tabChanged)
-  {
-    // remove all tab stops
-    [paragraph setTabStops: [NSArray arrayWithObject: tab]];
-    tabChanged = YES;
-  }
+    {
+      NSArray *a;
+      a = [[NSArray alloc] initWithObjects: tab, nil];
+      // remove all tab stops
+      [paragraph setTabStops: a];
+      DESTROY(a);
+      tabChanged = YES;
+    }
   else
     {
       [paragraph addTabStop: tab];
@@ -208,19 +237,35 @@ readNSString (StringContext *ctxt)
   changed = YES;
   RELEASE(tab);
 }
- 
+
 @end
 
+static BOOL classInheritsFromNSMutableAttributedString (Class c)
+{
+  Class mutable = [NSMutableAttributedString class];
+  
+  while (c != Nil)
+    {
+      if (c == mutable)
+	{
+	  return YES;
+	}
+      c = [c superclass];
+    }
+  return NO;
+}
+
 @interface RTFConsumer (Private)
 
 - (NSAttributedString*) parseRTF: (NSData *)rtfData 
-	      documentAttributes: (NSDictionary **)dict;
+	      documentAttributes: (NSDictionary **)dict
+			   class: (Class)class;
 - (NSDictionary*) documentAttributes;
-- (NSAttributedString*) result;
 
 - (RTFAttribute*) attr;
 - (void) push;
 - (void) pop;
+- (void) appendString: (NSString*)string;
 
 @end
 
@@ -234,22 +279,31 @@ readNSString (StringContext *ctxt)
   if (flag)
     {
       if ([format isEqual: @"RTFD"])
-	cClass = [RTFDProducer class];
+	{
+	  cClass = [RTFDProducer class];
+	}
       else if ([format isEqual: @"RTF"])
-	cClass = [RTFProducer class];
+	{
+	  cClass = [RTFProducer class];
+	}
     }
   else
     {
       if ([format isEqual: @"RTFD"])
-	cClass = [RTFDConsumer class];
+	{
+	  cClass = [RTFDConsumer class];
+	}
       else if ([format isEqual: @"RTF"])
-	cClass = [RTFConsumer class];
+	{
+	  cClass = [RTFConsumer class];
+	}
     }
   return cClass;
 }
 
 + (NSAttributedString*) parseFile: (NSFileWrapper *)wrapper
 	       documentAttributes: (NSDictionary **)dict
+			    class: (Class)class
 {
   RTFConsumer *consumer = [RTFConsumer new];
   NSAttributedString *text = nil;
@@ -257,7 +311,8 @@ readNSString (StringContext *ctxt)
   if ([wrapper isRegularFile])
     {
       text = [consumer parseRTF: [wrapper regularFileContents]
-		       documentAttributes: dict];
+		       documentAttributes: dict
+		       class: class];
     }
   else if ([wrapper isDirectory])
     {
@@ -269,23 +324,26 @@ readNSString (StringContext *ctxt)
       if ((contents = [files objectForKey: @"TXT.rtf"]) != nil)
 	{
 	  text = [consumer parseRTF: [contents regularFileContents]
-			   documentAttributes: dict];
+			   documentAttributes: dict
+			   class: class];
 	}
     }
-
+  
   RELEASE(consumer);
 
   return text;
 }
 
 + (NSAttributedString*) parseData: (NSData *)rtfData 
-	      documentAttributes: (NSDictionary **)dict
+	       documentAttributes: (NSDictionary **)dict
+			    class: (Class)class
 {
   RTFConsumer *consumer = [RTFConsumer new];
   NSAttributedString *text;
 
   text = [consumer parseRTF: rtfData
-		   documentAttributes: dict];
+		   documentAttributes: dict
+		   class: class];
   RELEASE(consumer);
 
   return text;
@@ -299,6 +357,7 @@ readNSString (StringContext *ctxt)
   fonts = nil;
   attrs = nil;
   colours = nil;
+  _class = Nil;
 
   return self;
 }
@@ -317,21 +376,15 @@ readNSString (StringContext *ctxt)
 
 @implementation RTFDConsumer
 
-+ (NSAttributedString*) parseFile: (NSFileWrapper *)wrapper
-	       documentAttributes: (NSDictionary **)dict
-{
-  return [super parseFile: wrapper
-		documentAttributes: dict];
-}
-
 + (NSAttributedString*) parseData: (NSData *)rtfData 
 	       documentAttributes: (NSDictionary **)dict
+			    class: (Class)class
 {
   NSAttributedString *str;
   NSFileWrapper *wrapper = [[NSFileWrapper alloc] 
-			       initWithSerializedRepresentation: rtfData];
-
-  str = [self parseFile: wrapper documentAttributes: dict];
+			     initWithSerializedRepresentation: rtfData];
+  
+  str = [self parseFile: wrapper  documentAttributes: dict  class: class];
   RELEASE (wrapper);
 
   return str;
@@ -353,7 +406,15 @@ readNSString (StringContext *ctxt)
 
   ignore = 0;  
   DESTROY(result);
-  result = [[NSMutableAttributedString alloc] init];
+  
+  if (classInheritsFromNSMutableAttributedString (_class))
+    {
+      result = [[_class alloc] init];
+    }
+  else
+    {
+      result = [[NSMutableAttributedString alloc] init];
+    }
   ASSIGN(documentAttributes, [NSMutableDictionary dictionary]);
   ASSIGN(fonts, [NSMutableDictionary dictionary]);
   ASSIGN(attrs, [NSMutableArray array]);
@@ -381,14 +442,9 @@ readNSString (StringContext *ctxt)
   ((RTFAttribute*)[attrs lastObject])->changed = YES;
 }
 
-- (NSAttributedString*) result
-{
-  RETAIN(result);
-  return AUTORELEASE(result);
-}
-
 - (NSAttributedString*) parseRTF: (NSData *)rtfData 
 	      documentAttributes: (NSDictionary **)dict
+			   class: (Class)class
 {
   CREATE_AUTORELEASE_POOL(pool);
   RTFscannerCtxt scanner;
@@ -400,10 +456,12 @@ readNSString (StringContext *ctxt)
 			  encoding: NSASCIIStringEncoding];
 
   // Reset this RFTConsumer, as it might already have been used!
+  _class = class;
   [self reset];
 
   initStringContext(&stringCtxt, rtfString);
   lexInitContext(&scanner, &stringCtxt, (int (*)(void*))readNSString);
+  [result beginEditing];
   NS_DURING
     GSRTFparse((void *)self, &scanner);
   NS_HANDLER
@@ -411,6 +469,7 @@ readNSString (StringContext *ctxt)
 	  [localException reason]);
   //[localException raise];
   NS_ENDHANDLER
+  [result endEditing];
 
   RELEASE(rtfString);
   RELEASE(pool);
@@ -420,29 +479,89 @@ readNSString (StringContext *ctxt)
       *dict = [self documentAttributes];
     }
 
-  return [self result];
+  if (classInheritsFromNSMutableAttributedString (_class))
+    {
+      RETAIN (result);
+      AUTORELEASE (result);
+      return result;
+    }
+  else
+    {
+      return AUTORELEASE ([[_class alloc] initWithAttributedString: result]);
+    }
+}
+
+- (void) appendString: (NSString*)string
+{
+  int  oldPosition = [result length];
+  int  textlen = [string length]; 
+  NSRange insertionRange = NSMakeRange(oldPosition,0);
+  NSMutableDictionary *attributes;
+
+  if (!ignore && textlen)
+    {
+      RTFAttribute* attr = [self attr];
+      [result replaceCharactersInRange: insertionRange 
+	      withString: string];
+
+      if (attr->changed)
+        {
+	  NSParagraphStyle *ps = [attr->paragraph copy];
+	  attributes = [[NSMutableDictionary alloc]
+			 initWithObjectsAndKeys:
+			   [attr currentFont], NSFontAttributeName,
+			   ps, NSParagraphStyleAttributeName,
+			   nil];
+	  DESTROY(ps);
+	  if (attr->underline)
+	    {
+	      [attributes setObject: [attr underline]
+			  forKey: NSUnderlineStyleAttributeName];
+	    }
+	  if (attr->script)
+	    {
+	      [attributes setObject: [attr script]
+			  forKey: NSSuperscriptAttributeName];
+	    }
+	  if (attr->fgColour != nil)
+	    {
+	      [attributes setObject: attr->fgColour 
+			  forKey: NSForegroundColorAttributeName];
+	    }
+	  if (attr->bgColour != nil)
+	    {
+	      [attributes setObject: attr->bgColour 
+			  forKey: NSBackgroundColorAttributeName];
+	    }
+	  
+	  [result setAttributes: attributes 
+		  range: NSMakeRange(oldPosition, textlen)];
+	  DESTROY(attributes);
+	  attr->changed = NO;
+	}
+    }
 }
 
 @end
 
 #undef IGNORE
-#define	FONTS	((RTFConsumer *)ctxt)->fonts
-#define	COLOURS	((RTFConsumer *)ctxt)->colours
-#define	RESULT	((RTFConsumer *)ctxt)->result
-#define	IGNORE	((RTFConsumer *)ctxt)->ignore
-#define	TEXTPOSITION [RESULT length]
+#define FONTS	((RTFConsumer *)ctxt)->fonts
+#define COLOURS	((RTFConsumer *)ctxt)->colours
+#define RESULT	((RTFConsumer *)ctxt)->result
+#define IGNORE	((RTFConsumer *)ctxt)->ignore
+#define TEXTPOSITION [RESULT length]
 #define DOCUMENTATTRIBUTES ((RTFConsumer*)ctxt)->documentAttributes
 
-#define	CTXT	[((RTFConsumer *)ctxt) attr]
-#define	CHANGED	CTXT->changed
-#define	PARAGRAPH CTXT->paragraph
-#define	FONTNAME CTXT->fontName
-#define	SCRIPT CTXT->script
-#define	ITALIC CTXT->italic
-#define	BOLD CTXT->bold
-#define	UNDERLINE CTXT->underline
-#define	FGCOLOUR CTXT->fgColour
-#define	BGCOLOUR CTXT->bgColour
+#define CTXT [((RTFConsumer *)ctxt) attr]
+#define CHANGED CTXT->changed
+#define PARAGRAPH CTXT->paragraph
+#define FONTNAME CTXT->fontName
+#define SCRIPT CTXT->script
+#define ITALIC CTXT->italic
+#define BOLD CTXT->bold
+#define UNDERLINE CTXT->underline
+#define FGCOLOUR CTXT->fgColour
+#define BGCOLOUR CTXT->bgColour
 
 #define PAPERSIZE @"PaperSize"
 #define LEFTMARGIN @"LeftMargin"
@@ -458,8 +577,9 @@ readNSString (StringContext *ctxt)
 /* handle errors (this is the yacc error mech)	*/
 void GSRTFerror (const char *msg)
 {
-  [NSException raise:NSInvalidArgumentException 
-	       format:@"Syntax error in RTF: %s", msg];
+/*  [NSException raise:NSInvalidArgumentException
+	       format:@"Syntax error in RTF: %s", msg];*/
+  NSDebugLLog(@"RTFParser",@"Syntax error in RTF: %s", msg);
 }
 
 void GSRTFgenericRTFcommand (void *ctxt, RTFcmd cmd)
@@ -473,14 +593,12 @@ void GSRTFgenericRTFcommand (void *ctxt, RTFcmd cmd)
 void GSRTFstart (void *ctxt)
 {
   NSDebugLLog(@"RTFParser", @"Start RTF parsing");
-  [RESULT beginEditing];
 }
 
 // Finished to parse one piece of RTF.
 void GSRTFstop (void *ctxt)
 {
   //<!> close all open bolds et al.
-  [RESULT endEditing];
   NSDebugLLog(@"RTFParser", @"End RTF parsing");
 }
 
@@ -511,49 +629,23 @@ void GSRTFcloseBlock (void *ctxt, BOOL ignore)
 
 void GSRTFmangleText (void *ctxt, const char *text)
 {
-  int  oldPosition = TEXTPOSITION;
-  int  textlen = strlen(text); 
-  NSRange insertionRange = NSMakeRange(oldPosition,0);
-  NSMutableDictionary *attributes;
+  NSData *data = [[NSData alloc] initWithBytes: (void*)text 
+				 length: strlen(text)];
+  NSString *str = [[NSString alloc] initWithData: data
+				    encoding: NSISOLatin1StringEncoding];
 
-  if (!IGNORE && textlen)
-    {
-      [RESULT replaceCharactersInRange: insertionRange 
-	      withString: [NSString stringWithCString:text]];
+  [(RTFConsumer *)ctxt appendString: str];
+  DESTROY(str);
+  DESTROY(data);
+}
 
-      if (CHANGED)
-        {
-	  attributes = [NSMutableDictionary 
-			 dictionaryWithObjectsAndKeys:
-			   [CTXT currentFont], NSFontAttributeName,
-			   PARAGRAPH, NSParagraphStyleAttributeName,
-			   nil];
-	  if (UNDERLINE)
-	    {
-	      [attributes setObject: [CTXT underline]
-			  forKey: NSUnderlineStyleAttributeName];
-	    }
-	  if (SCRIPT)
-	    {
-	      [attributes setObject: [CTXT script]
-			  forKey: NSSuperscriptAttributeName];
-	    }
-	  if (FGCOLOUR != nil)
-	    {
-	      [attributes setObject: FGCOLOUR 
-			  forKey: NSForegroundColorAttributeName];
-	    }
-	  if (BGCOLOUR != nil)
-	    {
-	      [attributes setObject: BGCOLOUR 
-			  forKey: NSBackgroundColorAttributeName];
-	    }
-
-	  [RESULT setAttributes: attributes 
-		  range: NSMakeRange(oldPosition, textlen)];
-	  CHANGED = NO;
-	}
-    }
+void GSRTFunicode (void *ctxt, int uchar)
+{
+  unichar chars = uchar;
+  NSString *str = [[NSString alloc] initWithCharacters: &chars 
+				    length: 1];
+  [(RTFConsumer *)ctxt appendString: str];
+  DESTROY(str);
 }
 
 void GSRTFregisterFont (void *ctxt, const char *fontName, 
@@ -561,7 +653,7 @@ void GSRTFregisterFont (void *ctxt, const char *fontName,
 {
   NSString		*fontNameString;
   NSNumber		*fontId = [NSNumber numberWithInt: fontNumber];
-
+  
   if (!fontName || !*fontName)
     {	
       [NSException raise: NSInvalidArgumentException 
@@ -569,8 +661,16 @@ void GSRTFregisterFont (void *ctxt, const char *fontName,
 		   TEXTPOSITION];
     }
   // exclude trailing ';' from fontName
-  fontNameString = [NSString stringWithCString: fontName 
-			     length: strlen(fontName)-1];
+  if (';' == fontName[strlen(fontName)-1])
+    {
+      fontNameString = [NSString stringWithCString: fontName 
+				 length: strlen(fontName)-1];
+    }
+  else 
+    {
+      fontNameString = [NSString stringWithCString: fontName 
+				 length: strlen(fontName)];
+    }
   [FONTS setObject: fontNameString forKey: fontId];
 }
 
@@ -684,7 +784,10 @@ void GSRTFfirstLineIndent (void *ctxt, int indent)
   NSMutableParagraphStyle *para = PARAGRAPH;
   float findent = twips2points(indent);
 
-  // FIXME: This should changed the left indent of the paragraph, if < 0
+  CTXT->real_fi = findent;
+
+  findent = CTXT->real_li + CTXT->real_fi;
+
   // for attributed strings only positiv indent is allowed
   if ((findent >= 0.0) && ([para firstLineHeadIndent] != findent))
     {
@@ -698,12 +801,21 @@ void GSRTFleftIndent (void *ctxt, int indent)
   NSMutableParagraphStyle *para = PARAGRAPH;
   float findent = twips2points(indent);
 
+  CTXT->real_li = findent;
+
   // for attributed strings only positiv indent is allowed
   if ((findent >= 0.0) && ([para headIndent] != findent))
     {
       [para setHeadIndent: findent];
       CHANGED = YES;
     }
+
+  findent = CTXT->real_li + CTXT->real_fi;
+  if ((findent >= 0.0) && ([para firstLineHeadIndent] != findent))
+    {
+      [para setFirstLineHeadIndent: findent];
+      CHANGED = YES;
+    }
 }
 
 void GSRTFrightIndent (void *ctxt, int indent)
@@ -714,7 +826,7 @@ void GSRTFrightIndent (void *ctxt, int indent)
   // for attributed strings only positiv indent is allowed
   if ((findent >= 0.0) && ([para tailIndent] != findent))
     {
-      [para setTailIndent: findent];
+      [para setTailIndent: -findent];
       CHANGED = YES;
     }
 }
@@ -781,6 +893,7 @@ void GSRTFspaceAbove (void *ctxt, int space)
   if (fspace >= 0.0)
     {
       [para setParagraphSpacing: fspace];
+      CHANGED = YES;
     }
 }
 
@@ -793,14 +906,17 @@ void GSRTFlineSpace (void *ctxt, int space)
     {
       [para setMinimumLineHeight: 0.0];
       [para setMaximumLineHeight: 0.0];
+      CHANGED = YES;
     }
   else if (fspace < 0.0)
     {
       [para setMaximumLineHeight: -fspace];
+      CHANGED = YES;
     }
   else
     {
       [para setMinimumLineHeight: fspace];
+      CHANGED = YES;
     }
 }
 
@@ -835,7 +951,7 @@ void GSRTFaddDefaultColor (void *ctxt)
 
 void GSRTFcolorbg (void *ctxt, int color)
 {
-  if ([COLOURS count] <= color)
+  if ([COLOURS count] <= (unsigned int)color)
     {
       ASSIGN (BGCOLOUR, [NSColor whiteColor]);
     }
@@ -843,11 +959,12 @@ void GSRTFcolorbg (void *ctxt, int color)
     {
       ASSIGN (BGCOLOUR, [COLOURS objectAtIndex: color]);
     }
+  CHANGED = YES;
 }
 
 void GSRTFcolorfg (void *ctxt, int color)
 {
-  if ([COLOURS count] <= color)
+  if ([COLOURS count] <= (unsigned int)color)
     {
       ASSIGN (FGCOLOUR, [NSColor blackColor]);
     }
@@ -855,6 +972,7 @@ void GSRTFcolorfg (void *ctxt, int color)
     {
       ASSIGN (FGCOLOUR, [COLOURS objectAtIndex: color]);
     }
+  CHANGED = YES;
 }
 
 void GSRTFsubscript (void *ctxt, int script)
