@@ -24,6 +24,7 @@
 #include	<Foundation/NSArray.h>
 #include	<Foundation/NSBundle.h>
 #include	<Foundation/NSDictionary.h>
+#include	<Foundation/NSSet.h>
 #include	<Foundation/NSFileManager.h>
 #include	<Foundation/NSString.h>
 #include	<Foundation/NSProcessInfo.h>
@@ -42,9 +43,10 @@ static NSMutableDictionary *validateService(NSDictionary *service, NSString* pat
 static NSString		*appsName = @".GNUstepAppList";
 static NSString		*cacheName = @".GNUstepServices";
 
-static	BOOL verbose = NO;
+static	int verbose = 0;
 static	NSMutableDictionary	*serviceMap;
-static	NSMutableDictionary	*filterMap;
+static	NSMutableArray		*filterList;
+static	NSMutableSet		*filterSet;
 static	NSMutableDictionary	*printMap;
 static	NSMutableDictionary	*spellMap;
 static	NSMutableDictionary	*applicationMap;
@@ -86,15 +88,16 @@ main(int argc, char** argv, char **env_c)
   proc = [NSProcessInfo processInfo];
   if (proc == nil)
     {
-      NSLog(@"unable to get process information!\n");
+      NSLog(@"unable to get process information!");
       [pool release];
-      exit(0);
+      exit(EXIT_SUCCESS);
     }
 
   [NSSerializer shouldBeCompact: YES];
 
   serviceMap = [NSMutableDictionary dictionaryWithCapacity: 64];
-  filterMap = [NSMutableDictionary dictionaryWithCapacity: 66];
+  filterList = [NSMutableArray arrayWithCapacity: 16];
+  filterSet = [NSMutableSet setWithCapacity: 64];
   printMap = [NSMutableDictionary dictionaryWithCapacity: 8];
   spellMap = [NSMutableDictionary dictionaryWithCapacity: 8];
   applicationMap = [NSMutableDictionary dictionaryWithCapacity: 64];
@@ -107,7 +110,11 @@ main(int argc, char** argv, char **env_c)
     {
       if ([[args objectAtIndex: index] isEqual: @"--verbose"])
 	{
-	  verbose = YES;
+	  verbose++;
+	}
+      if ([[args objectAtIndex: index] isEqual: @"--quiet"])
+	{
+	  verbose--;
 	}
       if ([[args objectAtIndex: index] isEqual: @"--help"])
 	{
@@ -118,9 +125,10 @@ main(int argc, char** argv, char **env_c)
 "\n"
 "You may use 'make_services --test filename' to test that the property list\n"
 "in 'filename' contains a valid services definition.\n"
-"You may use 'make_services --verbose' to produce descriptive output.\n",
+"You may use 'make_services --verbose' to produce descriptive output.\n"
+"or --quiet to suppress any output (not recommended)\n",
 [cacheName cString]);
-	  exit(0);
+	  exit(EXIT_SUCCESS);
 	}
       if ([[args objectAtIndex: index] isEqual: @"--test"])
 	{
@@ -139,17 +147,17 @@ main(int argc, char** argv, char **env_c)
 		    {
 		      validateEntry(svcs, file);
 		    }
-		  else
+		  else if (verbose >= 0)
 		    {
-		      NSLog(@"bad info - %@\n", file);
+		      NSLog(@"bad info - %@", file);
 		    }
 		}
-	      else
+	      else if (verbose >= 0)
 		{
-		  NSLog(@"bad info - %@\n", file);
+		  NSLog(@"bad info - %@", file);
 		}
 	    }
-	  exit(0);
+	  exit(EXIT_SUCCESS);
 	}
     }
 
@@ -196,22 +204,24 @@ main(int argc, char** argv, char **env_c)
     {
       if ([mgr createDirectoryAtPath: usrRoot attributes: nil] == NO)
 	{
-	  NSLog(@"couldn't create %@\n", usrRoot);
+	  if (verbose >= 0)
+	    NSLog(@"couldn't create %@", usrRoot);
 	  [pool release];
-	  exit(1);
+	  exit(EXIT_FAILURE);
 	}
     }
 
   str = usrRoot;	/* Record for adding into roots array */
 
-  usrRoot = [str stringByAppendingPathComponent: @"Services"];
+  usrRoot = [str stringByAppendingPathComponent: @"Library/Services"];
   if (([mgr fileExistsAtPath: usrRoot isDirectory: &isDir] && isDir) == 0)
     {
       if ([mgr createDirectoryAtPath: usrRoot attributes: nil] == NO)
 	{
-	  NSLog(@"couldn't create %@\n", usrRoot);
+	  if (verbose >= 0)
+	    NSLog(@"couldn't create %@", usrRoot);
 	  [pool release];
-	  exit(1);
+	  exit(EXIT_FAILURE);
 	}
     }
 
@@ -286,7 +296,7 @@ main(int argc, char** argv, char **env_c)
   fullMap = [NSMutableDictionary dictionaryWithCapacity: 5];
   [fullMap setObject: services forKey: @"ByPath"];
   [fullMap setObject: serviceMap forKey: @"ByService"];
-  [fullMap setObject: filterMap forKey: @"ByFilter"];
+  [fullMap setObject: filterList forKey: @"ByFilter"];
   [fullMap setObject: printMap forKey: @"ByPrint"];
   [fullMap setObject: spellMap forKey: @"BySpell"];
 
@@ -306,9 +316,10 @@ main(int argc, char** argv, char **env_c)
       data = [NSSerializer serializePropertyList: fullMap];
       if ([data writeToFile: str atomically: YES] == NO)
 	{
-	  NSLog(@"couldn't write %@\n", str);
+	  if (verbose >= 0)
+	    NSLog(@"couldn't write %@", str);
 	  [pool release];
-	  exit(1);
+	  exit(EXIT_FAILURE);
 	}
     }
 
@@ -329,14 +340,15 @@ main(int argc, char** argv, char **env_c)
       data = [NSSerializer serializePropertyList: applicationMap];
       if ([data writeToFile: str atomically: YES] == NO)
 	{
-	  NSLog(@"couldn't write %@\n", str);
+	  if (verbose >= 0)
+	    NSLog(@"couldn't write %@", str);
 	  [pool release];
-	  exit(1);
+	  exit(EXIT_FAILURE);
 	}
     }
 
   [pool release];
-  exit(0);
+  exit(EXIT_SUCCESS);
 }
 
 /*
@@ -358,7 +370,8 @@ static void addExtensionsForApplication(NSDictionary *info, NSString *app)
     {
       if ([o0 isKindOfClass: aClass] == NO)
         {
-          NSLog(@"bad app NSTypes (not an array) - %@\n", app);
+	  if (verbose >= 0)
+	    NSLog(@"bad app NSTypes (not an array) - %@", app);
           return;
         }
       a0 = (NSArray*)o0;
@@ -372,7 +385,8 @@ static void addExtensionsForApplication(NSDictionary *info, NSString *app)
 
           if ([o1 isKindOfClass: dClass] == NO)
             {
-              NSLog(@"bad app NSTypes (type not a dictionary) - %@\n", app);
+	      if (verbose >= 0)
+		NSLog(@"bad app NSTypes (type not a dictionary) - %@", app);
               return;
             }
 	  /*
@@ -386,7 +400,8 @@ static void addExtensionsForApplication(NSDictionary *info, NSString *app)
             }
           if ([o1 isKindOfClass: aClass] == NO)
             {
-              NSLog(@"bad app NSType (extensions not an array) - %@\n", app);
+	      if (verbose >= 0)
+		NSLog(@"bad app NSType (extensions not an array) - %@", app);
               return;
             }
           a1 = (NSArray*)o1;
@@ -397,60 +412,64 @@ static void addExtensionsForApplication(NSDictionary *info, NSString *app)
               NSMutableDictionary	*d;
 
               e = [[a1 objectAtIndex: j] lowercaseString];
+	      if ([e length] == 0)
+		{
+		  if (verbose >= 0)
+		    NSLog(@"Illegal (nul) extension ignored for - %@", app);
+		  return;
+		}
               d = [extensionsMap objectForKey: e];
               if (d == nil)
                 {
                   d = [NSMutableDictionary dictionaryWithCapacity: 1];
                   [extensionsMap setObject: d forKey: e];
                 }
-              if ([d objectForKey: app] == NO)
+              if ([d objectForKey: app] == nil)
                 {
                   [d setObject: t forKey: app];
-                } 
+                }
             }
         }
     }
   else
     {
-      /*
-       *    If we have an old format list of extensions
-       *    handled by this application - ensure that
-       *    the name of the application is listed in
-       *    the dictionary of applications handling each of
-       *    the extensions.
-       */
+      NSDictionary	*extensions;
+
       o0 = [info objectForKey: @"NSExtensions"];
       if (o0 == nil)
         {
           return;
         }
-      if ([o0 isKindOfClass: aClass] == NO)
+      if ([o0 isKindOfClass: dClass] == NO)
         {
-          NSLog(@"bad app NSExtensions (not an array) - %@\n", app);
+	  if (verbose >= 0)
+	    NSLog(@"bad app NSExtensions (not a dictionary) - %@", app);
           return;
         }
-      a0 = (NSArray*)o0;
+      extensions = (NSDictionary *) o0;
+      a0 = [extensions allKeys];
       i = [a0 count];
       while (i-- > 0)
         {
-          NSString		*e;
-          NSMutableDictionary	*d;
+          id	tmp = [extensions objectForKey: [a0 objectAtIndex: i]];
+          id	name;
+          id	dict;
 
-          e = [[a0 objectAtIndex: i] lowercaseString];
-          d = [extensionsMap objectForKey: e];
-          if (d == nil)
-            {
-	      d = [NSMutableDictionary dictionaryWithCapacity: 1];
-	      [extensionsMap setObject: d forKey: e];
-            }
-          if ([d objectForKey: app] == nil)
-            {
-	      NSDictionary	*info;
-
-	      info = [NSDictionary dictionaryWithObjectsAndKeys:
-			nil];
-              [d setObject: info forKey: app];
-            } 
+          if ([tmp isKindOfClass: dClass] == NO)
+	    {
+	      if (verbose >= 0)
+		NSLog(@"bad app NSExtensions (value isn't a dictionary) - %@",
+		      app);
+              continue;
+	    }
+          name = [[a0 objectAtIndex: i] lowercaseString];
+          dict = [extensionsMap objectForKey: name];
+          if (dict == nil)
+	    {
+              dict = [NSMutableDictionary dictionaryWithCapacity: 1];
+	    }
+          [dict setObject: tmp forKey: app];
+          [extensionsMap setObject: dict forKey: name];
         }
     }
 }
@@ -496,8 +515,9 @@ scanDirectory(NSMutableDictionary *services, NSString *path)
                    * this name, we skip this one - the first one takes
                    * precedence.
                    */
-                  NSLog(@"duplicate app (%@) at '%@' and '%@'\n",
-		    name, oldPath, newPath);
+		  if (verbose >= 0)
+		    NSLog(@"duplicate app (%@) at '%@' and '%@'",
+			  name, oldPath, newPath);
                   continue;
                 }
 
@@ -524,14 +544,14 @@ scanDirectory(NSMutableDictionary *services, NSString *path)
 
 		  addExtensionsForApplication(info, name);
 		}
-	      else
+	      else if (verbose >= 0)
 		{
-		  NSLog(@"bad app info - %@\n", newPath);
+		  NSLog(@"bad app info - %@", newPath);
 		}
 	    }
-	  else
+	  else if (verbose >= 0)
 	    {
-	      NSLog(@"bad application - %@\n", newPath);
+	      NSLog(@"bad application - %@", newPath);
 	    }
 	}
       else if (ext != nil && [ext isEqualToString: @"service"])
@@ -558,19 +578,19 @@ scanDirectory(NSMutableDictionary *services, NSString *path)
 			  [services setObject: entry forKey: newPath];
 			}
 		    }
-		  else
+		  else if (verbose >= 0)
 		    {
-		      NSLog(@"missing info - %@\n", newPath);
+		      NSLog(@"missing info - %@", newPath);
 		    }
 		}
-	      else
+	      else if (verbose >= 0)
 		{
-		  NSLog(@"bad service info - %@\n", newPath);
+		  NSLog(@"bad service info - %@", newPath);
 		}
 	    }
-	  else
+	  else if (verbose >= 0)
 	    {
-	      NSLog(@"bad services bundle - %@\n", newPath);
+	      NSLog(@"bad services bundle - %@", newPath);
 	    }
 	}
       else
@@ -607,6 +627,10 @@ scanDynamic(NSMutableDictionary *services, NSString *path)
 	  continue;
 	}
 
+      /* *.service bundles are handled in scanDirectory */
+      if ([[name pathExtension] isEqualToString: @"service"])
+	continue;
+
       infPath = [path stringByAppendingPathComponent: name];
 
       info = [NSDictionary dictionaryWithContentsOfFile: infPath];
@@ -625,9 +649,9 @@ scanDynamic(NSMutableDictionary *services, NSString *path)
 		}
 	    }
 	}
-      else
+      else if (verbose >= 0)
 	{
-	  NSLog(@"bad app info - %@\n", infPath);
+	  NSLog(@"bad app info - %@", infPath);
 	}
     }
   [arp release];
@@ -642,7 +666,8 @@ validateEntry(id svcs, NSString *path)
 
   if ([svcs isKindOfClass: aClass] == NO)
     {
-      NSLog(@"NSServices entry not an array - %@\n", path);
+      if (verbose >= 0)
+	NSLog(@"NSServices entry not an array - %@", path);
       return nil;
     }
 
@@ -664,9 +689,9 @@ validateEntry(id svcs, NSString *path)
 	      [newServices addObject: newService];
 	    }
 	}
-      else
+      else if (verbose >= 0)
 	{
-	  NSLog(@"NSServices entry %u not a dictionary - %@\n",
+	  NSLog(@"NSServices entry %u not a dictionary - %@",
 	    pos, path);
 	}
     }
@@ -718,7 +743,8 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 
       if (type == nil)
 	{
-	  NSLog(@"NSServices entry %u spurious field (%@)- %@\n", pos, k, path);
+	  if (verbose >= 0)
+	    NSLog(@"NSServices entry %u spurious field (%@)- %@", pos, k, path);
 	}
       else
 	{
@@ -727,8 +753,9 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 	    {
 	      if ([obj isKindOfClass: sClass] == NO)
 		{
-		  NSLog(@"NSServices entry %u field %@ is not a string "
-		    @"- %@\n", pos, k, path);
+		  if (verbose >= 0)
+		    NSLog(@"NSServices entry %u field %@ is not a string "
+			  @"- %@", pos, k, path);
 		  return nil;
 		}
 	      [result setObject: obj forKey: k];
@@ -739,15 +766,17 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 
 	      if ([obj isKindOfClass: aClass] == NO)
 		{
-		  NSLog(@"NSServices entry %u field %@ is not an array "
-		    @"- %@\n", pos, k, path);
+		  if (verbose >= 0)
+		    NSLog(@"NSServices entry %u field %@ is not an array "
+		    @"- %@", pos, k, path);
 		  return nil;
 		}
 	      a = (NSArray*)obj;
 	      if ([a count] == 0)
 		{
-		  NSLog(@"NSServices entry %u field %@ is an empty array "
-		    @"- %@\n", pos, k, path);
+		  if (verbose >= 0)
+		    NSLog(@"NSServices entry %u field %@ is an empty array "
+			  @"- %@", pos, k, path);
 		}
 	      else
 		{
@@ -757,8 +786,9 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 		    {
 		      if ([[a objectAtIndex: i] isKindOfClass: sClass] == NO)
 			{
-			  NSLog(@"NSServices entry %u field %@ element %u is "
-			    @"not a string - %@\n", pos, k, i, path);
+			  if (verbose >= 0)
+			    NSLog(@"NSServices entry %u field %@ element %u is "
+				  @"not a string - %@", pos, k, i, path);
 			  return nil;
 			}
 		    }
@@ -771,15 +801,17 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 
 	      if ([obj isKindOfClass: dClass] == NO)
 		{
-		  NSLog(@"NSServices entry %u field %@ is not a dictionary "
-		    @"- %@\n", pos, k, path);
+		  if (verbose >= 0)
+		    NSLog(@"NSServices entry %u field %@ is not a dictionary "
+			  @"- %@", pos, k, path);
 		  return nil;
 		}
 	      d = (NSDictionary*)obj;
 	      if ([d objectForKey: @"default"] == nil)
 		{
-		  NSLog(@"NSServices entry %u field %@ has no default value "
-		    @"- %@\n", pos, k, path);
+		  if (verbose >= 0)
+		    NSLog(@"NSServices entry %u field %@ has no default value "
+			  @"- %@", pos, k, path);
 		}
 	      else
 		{
@@ -789,8 +821,9 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 		    {
 		      if ([obj isKindOfClass: sClass] == NO)
 			{
-			  NSLog(@"NSServices entry %u field %@ contains "
-			    @"non-string value - %@\n", pos, k, path);
+			  if (verbose >= 0)
+			    NSLog(@"NSServices entry %u field %@ contains "
+				  @"non-string value - %@", pos, k, path);
 			  return nil;
 			}
 		    }
@@ -808,7 +841,75 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
   /*
    *	Now check that we have the required fields for the service.
    */
-  if ((obj = [result objectForKey: @"NSMessage"]) != nil)
+  if ((obj = [result objectForKey: @"NSFilter"]) != nil)
+    {
+      NSString		*str;
+      NSArray		*snd;
+      NSArray		*ret;
+      BOOL		notPresent = NO;
+
+      str = [result objectForKey: @"NSInputMechanism"];
+      if (str != nil)
+	{
+	  if ([str isEqualToString: @"NSUnixStdio"] == NO
+	    && [str isEqualToString: @"NSMapFile"] == NO
+	    && [str isEqualToString: @"NSIdentity"] == NO)
+	  {
+	    if (verbose >= 0)
+	      NSLog(@"NSServices entry %u bad input mechanism - %@", pos, path);
+	    return nil;
+	  }
+	}
+      else if ([result objectForKey: @"NSPortName"] == nil)
+	{
+	  if (verbose >= 0)
+	    NSLog(@"NSServices entry %u NSPortName missing - %@", pos, path);
+	  return nil;
+	}
+
+      snd = [result objectForKey: @"NSSendTypes"];
+      ret = [result objectForKey: @"NSReturnTypes"];
+      if ([snd count] == 0 || [ret count] == 0)
+	{
+	  if (verbose >= 0)
+	    NSLog(@"NSServices entry %u types empty or missing - %@", pos, path);
+	  return nil;
+	}
+      else
+	{
+	  unsigned	i = [snd count];
+
+	  /*
+	   * See if this filter handles any send/return combination
+	   * which is not alreadly present.
+	   */
+	  while (notPresent == NO && i-- > 0)
+	    {
+	      unsigned	j = [ret count];
+
+	      while (notPresent == NO && j-- > 0)
+		{
+		  str = [NSString stringWithFormat: @"%@==>%@",
+		    [snd objectAtIndex: i], [ret objectAtIndex: j]];
+		  if ([filterSet member: str] == nil)
+		    {
+		      notPresent = YES;
+		      [filterSet addObject: str];
+		      [filterList addObject: result];
+		    }
+		}
+	    }
+	}
+      if (notPresent == NO)
+	{
+	  if (verbose)
+	    {
+	      NSLog(@"Ignoring duplicate %u in %@ -\n%@", pos, path, result);
+	    }
+	  return nil;
+	}
+    }
+  else if ((obj = [result objectForKey: @"NSMessage"]) != nil)
     {
       NSDictionary	*item;
       NSEnumerator	*e;
@@ -817,18 +918,21 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 
       if ([result objectForKey: @"NSPortName"] == nil)
 	{
-	  NSLog(@"NSServices entry %u NSPortName missing - %@\n", pos, path);
+	  if (verbose >= 0)
+	    NSLog(@"NSServices entry %u NSPortName missing - %@", pos, path);
 	  return nil;
 	}
-      if ([result objectForKey: @"NSSendTypes"] == nil &&
-	  [result objectForKey: @"NSReturnTypes"] == nil)
+      if ([result objectForKey: @"NSSendTypes"] == nil
+	&& [result objectForKey: @"NSReturnTypes"] == nil)
 	{
-	  NSLog(@"NSServices entry %u types missing - %@\n", pos, path);
+	  if (verbose >= 0)
+	    NSLog(@"NSServices entry %u types missing - %@", pos, path);
 	  return nil;
 	}
       if ((item = [result objectForKey: @"NSMenuItem"]) == nil)
 	{
-	  NSLog(@"NSServices entry %u NSMenuItem missing - %@\n", pos, path);
+	  if (verbose >= 0)
+	    NSLog(@"NSServices entry %u NSMenuItem missing - %@", pos, path);
 	  return nil;
 	}
 
@@ -858,76 +962,11 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 	{
 	  if (verbose)
 	    {
-	      NSLog(@"Ignoring entry %u in %@ -\n%@\n", pos, path, result);
+	      NSLog(@"Ignoring entry %u in %@ -\n%@", pos, path, result);
 	    }
 	  return nil;	/* Ignore - already got service with this name	*/
 	}
     }
-  else if ((obj = [result objectForKey: @"NSFilter"]) != nil)
-    {
-      NSString	*str;
-      NSArray	*snd;
-      NSArray	*ret;
-      unsigned	spos;
-      BOOL	used = NO;
-
-      str = [result objectForKey: @"NSInputMechanism"];
-      if (str)
-	{
-	  if ([str isEqualToString: @"NSUnixStdio"] == NO &&
-	      [str isEqualToString: @"NSMapFile"] == NO &&
-	      [str isEqualToString: @"NSIdentity"] == NO)
-	  {
-	    NSLog(@"NSServices entry %u bad input mechanism - %@\n", pos, path);
-	    return nil;
-	  }
-	}
-
-      snd = [result objectForKey: @"NSSendTypes"];
-      ret = [result objectForKey: @"NSReturnTypes"];
-      if (snd == nil || ret == nil)
-	{
-	  NSLog(@"NSServices entry %u types missing - %@\n", pos, path);
-	  return nil;
-	}
-
-      /*
-       *	For each send-type/return-type combination, see if we
-       *	already have a filter - if so - ignore this one.
-       */
-      spos = [snd count];
-      while (spos-- > 0)
-	{
-	  NSString		*stype = [snd objectAtIndex: spos];
-	  NSMutableDictionary	*sdict = [filterMap objectForKey: stype];
-	  unsigned		rpos;
-
-	  if (sdict == nil)
-	    {
-	      sdict = [NSMutableDictionary dictionaryWithCapacity: [snd count]];
-	      [filterMap setObject: sdict forKey: stype];
-	    }
-	  rpos = [ret count];
-	  while (rpos-- > 0)
-	    {
-	      NSString			*rtype = [ret objectAtIndex: rpos];
-
-	      if ([sdict objectForKey: rtype] == nil)
-		{
-		  [sdict setObject: result forKey: rtype];
-		  used = YES;
-		}
-	    }
-	}
-      if (used == NO)
-	{
-	  if (verbose)
-	    {
-	      NSLog(@"Ignoring entry %u in %@ -\n%@\n", pos, path, result);
-	    }
-	  return nil;	/* Ignore - already got filter for types.	*/
-	}
-    }
   else if ((obj = [result objectForKey: @"NSPrintFilter"]) != nil)
     {
       NSDictionary	*item;
@@ -937,7 +976,8 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 
       if ((item = [result objectForKey: @"NSMenuItem"]) == nil)
 	{
-	  NSLog(@"NSServices entry %u NSMenuItem missing - %@\n", pos, path);
+	  if (verbose >= 0)
+	    NSLog(@"NSServices entry %u NSMenuItem missing - %@", pos, path);
 	  return nil;
 	}
       /*
@@ -966,7 +1006,7 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 	{
 	  if (verbose)
 	    {
-	      NSLog(@"Ignoring entry %u in %@ -\n%@\n", pos, path, result);
+	      NSLog(@"Ignoring entry %u in %@ -\n%@", pos, path, result);
 	    }
 	  return nil;	/* Ignore - already got filter with this name	*/
 	}
@@ -979,7 +1019,8 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 
       if ((item = [result objectForKey: @"NSLanguages"]) == nil)
 	{
-	  NSLog(@"NSServices entry %u NSLanguages missing - %@\n", pos, path);
+	  if (verbose >= 0)
+	    NSLog(@"NSServices entry %u NSLanguages missing - %@", pos, path);
 	  return nil;
 	}
       /*
@@ -1001,14 +1042,15 @@ validateService(NSDictionary *service, NSString *path, unsigned pos)
 	{
 	  if (verbose)
 	    {
-	      NSLog(@"Ignoring entry %u in %@ -\n%@\n", pos, path, result);
+	      NSLog(@"Ignoring entry %u in %@ -\n%@", pos, path, result);
 	    }
 	  return nil;	/* Ignore - already got speller with language.	*/
 	}
     }
   else
     {
-      NSLog(@"NSServices entry %u unknown service/filter - %@\n", pos, path);
+      if (verbose >= 0)
+	NSLog(@"NSServices entry %u unknown service/filter - %@", pos, path);
       return nil;
     }
   
